package org.processmining.plugins.bpmn.analysis;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;

import javax.swing.JOptionPane;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XExtendedEvent;
import org.deckfour.xes.extension.std.XOrganizationalExtension;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.info.impl.XLogInfoImpl;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.connections.ConnectionManager;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.PluginContextID;
import org.processmining.framework.plugin.PluginExecutionResult;
import org.processmining.framework.plugin.PluginParameterBinding;
import org.processmining.framework.plugin.Progress;
import org.processmining.framework.plugin.events.Logger;
import org.processmining.framework.util.Pair;
import org.processmining.models.connections.bpmn.BPMNFlexConnection;
import org.processmining.models.connections.bpmn.analysis.BPMNAnalysisConnection;
import org.processmining.models.connections.flexiblemodel.FlexCodecConnection;
import org.processmining.models.connections.flexiblemodel.FlexEndTaskNodeConnection;
import org.processmining.models.connections.flexiblemodel.FlexRepResultConnection;
import org.processmining.models.connections.flexiblemodel.FlexStartTaskNodeConnection;
import org.processmining.models.connections.flexiblemodel.conversion.ReplayFlexOfOrigFlexConnection;
import org.processmining.models.flexiblemodel.EndTaskNodesSet;
import org.processmining.models.flexiblemodel.Flex;
import org.processmining.models.flexiblemodel.FlexNode;
import org.processmining.models.flexiblemodel.StartTaskNodesSet;
import org.processmining.models.graphbased.directed.ContainableDirectedGraphElement;
import org.processmining.models.graphbased.directed.DirectedGraphNode;
import org.processmining.models.graphbased.directed.bpmn.BPMNDiagram;
import org.processmining.models.graphbased.directed.bpmn.BPMNEdge;
import org.processmining.models.graphbased.directed.bpmn.BPMNNode;
import org.processmining.models.graphbased.directed.bpmn.elements.Activity;
import org.processmining.models.graphbased.directed.bpmn.elements.Gateway;
import org.processmining.models.graphbased.directed.bpmn.elements.Swimlane;
import org.processmining.models.graphbased.directed.socialnetwork.SocialNetwork;
import org.processmining.plugins.bpmn.analysis.data.BPMNAnalysisData;
import org.processmining.plugins.bpmn.analysis.data.BPMNAnalysisDataList;
import org.processmining.plugins.bpmn.analysis.data.ConformanceInfo;
import org.processmining.plugins.bpmn.analysis.settings.ModelToLogConnection;
import org.processmining.plugins.bpmn.analysis.settings.ui.BPMNAnalysisSettingsGUI;
import org.processmining.plugins.bpmn.analysis.settings.ui.ReplayerSettingsUI;
import org.processmining.plugins.bpmn.analysis.utils.Triple;
import org.processmining.plugins.flex.replayer.algorithms.CancellationAwareAStarAlgorithm;
import org.processmining.plugins.flex.replayer.algorithms.ExtendedCostBasedAStarLogReplayAlgorithm;
import org.processmining.plugins.flex.replayer.algorithms.IFlexLogReplayAlgorithm;
import org.processmining.plugins.flex.replayer.performance.ui.SimpleMappingStep;
import org.processmining.plugins.flex.replayer.performance.util.FlexToFlexMapping;
import org.processmining.plugins.flex.replayer.ui.ParamSettingExtendedCostBasedReplay;
import org.processmining.plugins.flex.replayer.util.FlexBinding;
import org.processmining.plugins.flex.replayer.util.FlexCodec;
import org.processmining.plugins.flex.replayresult.FlexRepResult;
import org.processmining.plugins.flex.replayresult.performance.FlexPerfRepInfo;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;
import org.processmining.plugins.socialnetwork.miner.SNHoWMiner;
import org.processmining.plugins.socialnetwork.miner.SNMinerOptions;
import org.processmining.plugins.socialnetwork.miner.SNWTMiner;
import org.processmining.plugins.socialnetwork.miner.miningoperation.BasicOperation;
import org.processmining.plugins.socialnetwork.miner.miningoperation.OperationFactory;
import org.processmining.plugins.socialnetwork.miner.miningoperation.UtilOperation;

import cern.colt.matrix.DoubleFactory2D;
import cern.colt.matrix.DoubleMatrix2D;


public abstract class BPMNAnalysis {
	public static final String UNKNOWN = "unknown";
	public static final String ARTIFICIAL_PROM = "Artificial (ProM)";
	
	private static final String START_STEP_MESSAGE = "Starting BPMN analysis";
	
	private final int PROGRESS_MAX_VALUE = 2000; 
	
	/**
	 * Progress bar initialization
	 * @param message
	 */
	private void initProgressBar(PluginContext context, String message, XLog log) {
		Progress progress = context.getProgress();
		progress.setCaption(message);
		progress.setIndeterminate(false);
		progress.setMinimum(0);
		progress.setMaximum(log.size() + PROGRESS_MAX_VALUE);
	}
	
	/**
	 * Progress bar finalization
	 * @param message
	 */
	private void finalizeProgressBar(PluginContext context) {
		Progress progress = context.getProgress();
		progress.setValue(PROGRESS_MAX_VALUE);
	}
	
	/**
	 * Progress bar fill half
	 * @param message
	 */
	private void fillHalfOfProgressBar(PluginContext context) {
		Progress progress = context.getProgress();
		progress.setValue(PROGRESS_MAX_VALUE/2);
	}
	
	/**
	 * Change progress caption
	 * @param message
	 */
	private void changeProgressBarCaption(PluginContext context, String message) {
		Progress progress = context.getProgress();
		progress.setCaption(message);
	}
	
	/**
	 * @param flex
	 * @return
	 */
	private BPMNDiagram discoverBPMNWithoutSwimlanes(PluginContext context, XLog log, Flex flex) {
		System.out.println("Converting C-NET to BPMN");
		String message = "C-Net to BPMN Converter";
		initProgressBar(context, message, log);
		BPMNDiagram bpmnWithoutSwimlanes = null;
		try {
			bpmnWithoutSwimlanes = context.tryToFindOrConstructFirstObject(BPMNDiagram.class, BPMNFlexConnection.class,
					BPMNFlexConnection.BPMNDiagram, flex);
		} catch (ConnectionCannotBeObtained e) {
			e.printStackTrace();
		}
		finalizeProgressBar(context);
		return bpmnWithoutSwimlanes;
	}
	
	private Object[] replayLog(PluginContext context, Flex flex, XLog log,
			Object[] flexReplayerConfiguration) {
		
		String message = "Get StartTaskNodesSet of the given C-Net";
		FlexStartTaskNodeConnection connStartTask = null;
		try {
			connStartTask = context.getConnectionManager().getFirstConnection(
					FlexStartTaskNodeConnection.class, context, flex);
		} catch (ConnectionCannotBeObtained e) {
			e.printStackTrace();
		}
		StartTaskNodesSet startTaskNode = connStartTask
				.getObjectWithRole(FlexStartTaskNodeConnection.STARTTASKNODES);

		message = "Get EndTaskNodesSet of the given C-Net";
		FlexEndTaskNodeConnection connEndTask = null;
		try {
			connEndTask = context.getConnectionManager().getFirstConnection(
					FlexEndTaskNodeConnection.class, context, flex);
		} catch (ConnectionCannotBeObtained e) {
			e.printStackTrace();
		}
		EndTaskNodesSet endTaskNode = connEndTask.getObjectWithRole(FlexEndTaskNodeConnection.ENDTASKNODES);

		@SuppressWarnings("unchecked")
		Map<FlexNode, XEventClass> mappings = (Map<FlexNode, XEventClass>) flexReplayerConfiguration[ReplayerSettingsUI.MAPPING];

		IFlexLogReplayAlgorithm iFlexLogReplayAlgorithm = (IFlexLogReplayAlgorithm) flexReplayerConfiguration[ReplayerSettingsUI.ALGORITHM];
		Object[] parameters = (Object[]) flexReplayerConfiguration[ReplayerSettingsUI.PARAMETERS];
		/*
		 * parameters[ExtendedCostBasedAStarLogReplayAlgorithm.
		 * MAXEXPLOREDINSTANCESINTVAL] = 20000;
		 * parameters[ExtendedCostBasedAStarLogReplayAlgorithm
		 * .ANALYSISTYPEINTVAL] = 0;
		 */
		finalizeProgressBar(context);

		// RUN REPLAY LOG ON C-NET FOR PERFORMANCE PLUG-IN FOR THE LOG
		message = "Replay the log for Performance";
		initProgressBar(context, message, log);
		System.out.println("REPLAYING All Traces for performance analysis.");
		context.log("REPLAYING All Traces for performance analysis.");

		if (context.getProgress().isCancelled()) {
			return cancelExecution(context, "Plugin execution was cancelled by user.", null);
		}
		/* call without UI */
		FlexPerfRepInfo flexPerfRepResult = runPerformanceReplayerWithChildContext(context, flex, log,
				mappings, startTaskNode, endTaskNode, iFlexLogReplayAlgorithm, parameters);
		System.out.println("finished replay");
		if (flexPerfRepResult == null) {
			return cancelExecution(context, "Could not execute performance replayer on model.", "Performance Replayer");
		}
		finalizeProgressBar(context);
		return new Object[]{flexPerfRepResult};
	}
	
	private BPMNDiagram discoverSwimlanes(PluginContext context, XLog log, 
			BPMNDiagram bpmnWithoutSwimlanes, BPMNAnalysisData bpmnAnalysisData, Flex flex) {
		System.out.println("Discover Swimlanes");
		
		String message = "Obtain Swimlanes";
		initProgressBar(context, message, log);
		BPMNDiagram bpmnWithSwimlanes = null;
		try {
			bpmnWithSwimlanes = context.tryToFindOrConstructFirstNamedObject(BPMNDiagram.class,
					"Swimlane Discovery Plug-in", null, null, bpmnWithoutSwimlanes, log);
		} catch (ConnectionCannotBeObtained e) {
			e.printStackTrace();
		}
		finalizeProgressBar(context);

		// filter out unknown and artificial resources
		XLog unnecessaryOriginatorsRemovedCompleteLog = removeUnnecessaryOriginators(log);
		// GET Swimlanes
		message = "Obtain User Activity Matrix";
		initProgressBar(context, message, log);
		// OBTAIN ORIGINATOR-TASK (OT)/USER-EVENT MATRIX
		List<Triple<String, String, Integer>> userActivityTripleList = getUserActivityTripleList(
				unnecessaryOriginatorsRemovedCompleteLog, bpmnAnalysisData/*
																		 * ,
																		 * true
																 */);
		finalizeProgressBar(context);
		// OBTAIN SNs FOR COMPLETE LOG
		getSNforCompleteLog(unnecessaryOriginatorsRemovedCompleteLog, bpmnAnalysisData, context);
		// GET Swimlanes
		message = "Obtain Social Network Diagrams for activities";
		initProgressBar(context, message, log);
		// OBTAIN ACTIVITY BASED SOCIAL HANDOVER OF WORK NETWORK AND ACTIVITY BASED ORIGINATOR-TASK (OT)/USER-EVENT MATRIX
		fillActivityBasedSNAndOTMatricesToBPMNAnalysisData(unnecessaryOriginatorsRemovedCompleteLog,
				flex, bpmnWithoutSwimlanes, userActivityTripleList, bpmnAnalysisData, context);
		finalizeProgressBar(context);
		message = "Obtain Social Network Diagrams for swimlanes";
		initProgressBar(context, message, log);
		// OBTAIN SWIMLANE BASED SOCIAL HANDOVER OF WORK NETWORK AND SWIMLANE BASED ORIGINATOR-TASK (OT)/USER-EVENT MATRIX
		if (!bpmnWithSwimlanes.getSwimlanes().isEmpty()) {
			fillSwimlaneBasedSNAndOTMatricesToBPMNAnalysisData(unnecessaryOriginatorsRemovedCompleteLog,
					bpmnWithSwimlanes, userActivityTripleList, bpmnAnalysisData, context);
		}
		finalizeProgressBar(context);
		return bpmnWithSwimlanes;
	}
	
	private FlexCodec retriveFlexCodec(PluginContext context, Flex originalModel, XLog log) {
		ConnectionManager conn = context.getConnectionManager();
		FlexCodec codec = null;
		String message = null;
		try {
			System.out.println("Get model-to-replay mapping");
			
			message = "FlexToFlexMapping";
			FlexToFlexMapping mappingToOriginalModel = context.tryToFindOrConstructFirstObject(
					FlexToFlexMapping.class, ReplayFlexOfOrigFlexConnection.class,
					ReplayFlexOfOrigFlexConnection.MAPPINGTOORIGINAL, originalModel, log);
			message = "ReplayFlexOfOrigFlexConnection";
			ReplayFlexOfOrigFlexConnection connFlexOfOrigFlexConnection = conn.getFirstConnection(
					ReplayFlexOfOrigFlexConnection.class, context, log, originalModel, mappingToOriginalModel);
			Flex replayModel = connFlexOfOrigFlexConnection
					.getObjectWithRole(ReplayFlexOfOrigFlexConnection.REPLAYMODEL);

			FlexCodecConnection codecConn = conn
					.getFirstConnection(FlexCodecConnection.class, context, replayModel);
			codec = (FlexCodec) codecConn.getObjectWithRole(FlexCodecConnection.FLEXCODEC);
		} catch (ConnectionCannotBeObtained e) {
			e.printStackTrace();
			context.log("Can't obtain connection for " + message);
			System.out.println("Exception occured while obtaining " + message);
		}
		return codec;
	}
	
	private BPMNAnalysisData retrieveConformance(PluginContext context,
			XLog log, FlexCodec codec, SortedSet<Integer> caseIDIndexList,
			Integer count, BPMNAnalysisDataList bpmnAnalysisDataList,
			FlexPerfRepInfo flexPerfRepInfo) {
		BPMNAnalysisData bpmnAnalysisData = new BPMNAnalysisData();
		String message = "ReplayFlexOfOrigFlexConnection";
		FlexRepResult replayResult = null;
		try {
			System.out.println("Get replay result");

			FlexRepResultConnection connection = context.getConnectionManager()
					.getFirstConnection(FlexRepResultConnection.class, context,
							log);

			replayResult = connection
					.getObjectWithRole(FlexRepResultConnection.FLEXREPRESULT);

		} catch (ConnectionCannotBeObtained conExc) {
			System.out.println(conExc.getMessage());
			conExc.printStackTrace();
			context.getFutureResult(0).cancel(true);
			context.getFutureResult(1).cancel(true);
			context.getFutureResult(2).cancel(true);
			context.log("Can't obtain connection for " + message);
			return null;
		}
		if (replayResult == null) {
			context.getFutureResult(0).cancel(true);
			context.getFutureResult(1).cancel(true);
			context.getFutureResult(2).cancel(true);
			context.log("Can't obtain connection for " + message);
			return null;
		}
		if (replayResult.isEmpty()) {
			context.getFutureResult(0).cancel(true);
			context.getFutureResult(1).cancel(true);
			context.getFutureResult(2).cancel(true);
			context.log("Can't obtain connection for " + message);
			return null;
		}

		double fitness;
		double minFitness = 1.00;
		double maxFitness = -1.00;
		double accConformance = 0.000000;
		int numCaseInvolved = 0;
		int numEvtsInATrace = 0;
		int cost = 0;

		if (replayResult.size() > 1) {
			count = new Integer(1);
			for (SyncReplayResult syncReplayResult : replayResult) {

				if (context.getProgress().isCancelled()) {
					context.getFutureResult(0).cancel(true);
					context.getFutureResult(1).cancel(true);
					context.getFutureResult(2).cancel(true);
					System.out.println("The plug-in execution is cancelled!");
					context.log("The plug-in execution is cancelled!");
					return null;
				}

				System.out.println("REPLAYING result #" + count
						+ " for performance analysis: "
						+ syncReplayResult.isReliable());
				context.log("REPLAYING result #" + count
						+ " for performance analysis.");

				message = "Obtain Info for SyncReplayResult: " + count;
				initProgressBar(context, message, log);

				numEvtsInATrace = 0;
				cost = 0;

				// FOR EACH SYNCHREPLAYRESULT PRODUCES A PATH SO GET THE LOG
				// THAT CONTAINS THE CASE IDs OF SYNCHREPLAYRESULT AND GET
				// CONFORMANCE AND PERFORMANCE RESULT
				caseIDIndexList.addAll(syncReplayResult.getTraceIndex());

				// calculate fitness values
				message = "Calculate conformance value for SyncReplayResult: "
						+ count;
				changeProgressBarCaption(context, message);

				for (StepTypes stepType : syncReplayResult.getStepTypes()) {
					switch (stepType) {
					case L:
						cost += replayResult.getWeightMoveOnLogOnly();
						numEvtsInATrace++;
						break;
					case MINVI:
						break;
					case MREAL:
						cost += replayResult.getWeightMoveOnModelOnlyReal();
						break;
					case LMNOGOOD:
						cost += replayResult.getWeightViolation();
						numEvtsInATrace++;
						break;
					default:
						numEvtsInATrace++;
					}
				}
				numCaseInvolved += caseIDIndexList.size();

				int positiveMinimumPenalty = 0;
				if (replayResult.getWeightViolation() > 0
						&& replayResult.getWeightMoveOnLogOnly() > 0) {
					positiveMinimumPenalty = replayResult.getWeightViolation() < replayResult
							.getWeightMoveOnLogOnly() ? replayResult
							.getWeightViolation() : replayResult
							.getWeightMoveOnLogOnly();
				} else {
					positiveMinimumPenalty = replayResult.getWeightViolation() > 0 ? replayResult
							.getWeightViolation() : replayResult
							.getWeightMoveOnLogOnly();
				}

				fitness = 1 - (((double) cost) / (double) (numEvtsInATrace * positiveMinimumPenalty));

				if (Double.compare(minFitness, fitness) > 0) {
					minFitness = fitness;
				}
				if (Double.compare(maxFitness, fitness) < 0) {
					maxFitness = fitness;
				}
//				if (codec != null) {
//					bpmnAnalysisData.setConformanceView(getConformanceView(
//							codec, syncReplayResult));
//				}

				accConformance = (fitness * caseIDIndexList.size());
				ConformanceInfo conformanceInfo = new ConformanceInfo(
						accConformance / numCaseInvolved, minFitness,
						maxFitness);
				bpmnAnalysisData.setConformanceInfo(conformanceInfo);
				// complete the BPMNAnalysisData for all traces and put
				// bpmnAnalysisData to bpmnAnalysisDataList
				bpmnAnalysisData.setDescription("All Traces");
				;
				bpmnAnalysisData.setPerfRepInfo(flexPerfRepInfo);
				setCaseArrivalRatio(log, bpmnAnalysisData, flexPerfRepInfo);
				bpmnAnalysisDataList.add(bpmnAnalysisData);

				message = "Get event log for SyncReplayResult: " + count;
			}

		}
		System.out.println("Case ID = " + caseIDIndexList);
		return bpmnAnalysisData;
	}
	
	private XLog filterLog(PluginContext context, XLog log, Flex originalModel, 
			SortedSet<Integer> caseIDIndexList, Integer count,
			boolean doesContainUserData, BPMNAnalysisData bpmnAnalysisData,
			BPMNDiagram bpmnWithoutSwimlanes, BPMNDiagram bpmnWithSwimlanes) {
		XLog filteredLog = filterEventlog(log, caseIDIndexList, count);
		String message;
		if (doesContainUserData) {
			bpmnAnalysisData.setDoesContainUserInfo(true);

			// filter out unknown and artificial resources
			XLog unnecessaryOriginatorsRemovedFilteredLog = removeUnnecessaryOriginators(filteredLog);
			message = "Get UserActivity Matrix for SyncReplayResult: " + count;
			changeProgressBarCaption(context, message);
			// OBTAIN ORIGINATOR-TASK (OT)/USER-EVENT MATRIX
			List<Triple<String, String, Integer>> userActivityTripleList2 = getUserActivityTripleList(
					unnecessaryOriginatorsRemovedFilteredLog, bpmnAnalysisData/*
																				 * ,
																				 * true
																				 */);
			// OBTAIN SNs FOR FILTERED LOG
			getSNforCompleteLog(unnecessaryOriginatorsRemovedFilteredLog, bpmnAnalysisData, context);
			message = "Get Activity based social networks for SyncReplayResult: " + count;
			changeProgressBarCaption(context, message);
			// OBTAIN ACTIVITY BASED SOCIAL HANDOVER OF WORK NETWORK AND ACTIVITY BASED ORIGINATOR-TASK (OT)/USER-EVENT MATRIX
			fillActivityBasedSNAndOTMatricesToBPMNAnalysisData(unnecessaryOriginatorsRemovedFilteredLog,
					originalModel, bpmnWithoutSwimlanes, userActivityTripleList2, bpmnAnalysisData,
					context);
			message = "Get swimlane based social networks for SyncReplayResult: " + count;
			changeProgressBarCaption(context, message);
			// OBTAIN SWIMLANE BASED SOCIAL HANDOVER OF WORK NETWORK AND SWIMLANE BASED ORIGINATOR-TASK (OT)/USER-EVENT MATRIX
			if (!bpmnWithSwimlanes.getSwimlanes().isEmpty()) {
				fillSwimlaneBasedSNAndOTMatricesToBPMNAnalysisData(
						unnecessaryOriginatorsRemovedFilteredLog, bpmnWithSwimlanes,
						userActivityTripleList2, bpmnAnalysisData, context);
			}
		}else {
			bpmnAnalysisData.setDoesContainUserInfo(false);
		}
		// obtain the performance result for filtered log that contains only the caseIds in syncReplayResult(i.e. this will give us the performance result of the most
		// frequent path)
		finalizeProgressBar(context);
		message = "Get node event class mappings for SyncReplayResult: " + count;
		changeProgressBarCaption(context, message);
		return filteredLog;
	}
	
	private void runPerformanceAnalysis(PluginContext context, IFlexLogReplayAlgorithm iFlexLogReplayAlgorithm,
			Flex originalModel, XLog filteredLog, Map<FlexNode, XEventClass> newMappings, Integer count, SortedSet<Integer> caseIDIndexList,
			BPMNAnalysisData bpmnAnalysisData, XLog log, Map<FlexPerfRepInfo, BPMNAnalysisData> perfBPMNAnalysisDataMap,
			Object[] flexReplayerConfiguration) {
		String message = "Replay the log for Performance for ";
		changeProgressBarCaption(context, message);
		FlexStartTaskNodeConnection connStartTask = null;
		try {
			connStartTask = context.getConnectionManager().getFirstConnection(
					FlexStartTaskNodeConnection.class, context, originalModel);
		} catch (ConnectionCannotBeObtained e) {
			e.printStackTrace();
		}
		StartTaskNodesSet startTaskNode = connStartTask
				.getObjectWithRole(FlexStartTaskNodeConnection.STARTTASKNODES);

		message = "Get EndTaskNodesSet of the given C-Net";
		FlexEndTaskNodeConnection connEndTask = null;
		try {
			connEndTask = context.getConnectionManager().getFirstConnection(
					FlexEndTaskNodeConnection.class, context, originalModel);
		} catch (ConnectionCannotBeObtained e) {
			e.printStackTrace();
		}
		EndTaskNodesSet endTaskNode = connEndTask
				.getObjectWithRole(FlexEndTaskNodeConnection.ENDTASKNODES);
		
		IFlexLogReplayAlgorithm iFlexLogReplayAlgorithm2 = null;
		if (iFlexLogReplayAlgorithm instanceof ExtendedCostBasedAStarLogReplayAlgorithm) {
			iFlexLogReplayAlgorithm2 = new ExtendedCostBasedAStarLogReplayAlgorithm();
		} else if (iFlexLogReplayAlgorithm instanceof CancellationAwareAStarAlgorithm) {
			iFlexLogReplayAlgorithm2 = new CancellationAwareAStarAlgorithm();
		} else {
			JOptionPane
					.showMessageDialog(
							null,
							"<html>The selected replay algorithm needs to be added to the scope of this plug-in! Please notify it to mr.i.bayraktar@gmail.com</html>",
							"Unsatisfied Replay Requirement",
							JOptionPane.ERROR_MESSAGE);
		}
		Object[] parameters = (Object[]) flexReplayerConfiguration[ReplayerSettingsUI.PARAMETERS];
		/* call without UI */
		FlexPerfRepInfo flexPerfRepResult4EachSyncReplayResult = runPerformanceReplayerWithChildContext(
				context, originalModel, filteredLog, newMappings,
				startTaskNode, endTaskNode, iFlexLogReplayAlgorithm2,
				parameters);
		boolean isReliable = true;
		FlexRepResult replayResult2 = null;
		try {
			FlexRepResultConnection connection2 = context
					.getConnectionManager()
					.getFirstConnection(FlexRepResultConnection.class, context,
							filteredLog);
			// 1. WE GET REPLAY RESULT
			replayResult2 = connection2
					.getObjectWithRole(FlexRepResultConnection.FLEXREPRESULT);

		} catch (ConnectionCannotBeObtained conExc) {
			context.log("Replay result cannot be obtained!");
			System.out.println("Replay result cannot be obtained!");
			replayResult2 = null;
		}
		if (replayResult2 != null
				|| flexPerfRepResult4EachSyncReplayResult != null) {
			if (!replayResult2.isEmpty()) {
				isReliable = replayResult2.iterator().next().isReliable();
			} else {
				isReliable = false;
			}
		} else {
			isReliable = false;
		}
		System.out.println("Replay " + count + " contains "
				+ caseIDIndexList.size() + " process instances and it is "
				+ (isReliable ? "reliable." : "not reliable."));
		context.log("Replay " + count + " contains " + caseIDIndexList.size()
				+ " process instances and it is "
				+ (isReliable ? "reliable." : "not reliable."));
		if (flexPerfRepResult4EachSyncReplayResult != null) {
			System.out.println("Replay "
					+ count
					+ " contains "
					+ flexPerfRepResult4EachSyncReplayResult
							.getUnreliableTraceIndexes().size()
					+ " unreliable process instances.");
			context.log("Replay "
					+ count
					+ " contains "
					+ flexPerfRepResult4EachSyncReplayResult
							.getUnreliableTraceIndexes().size()
					+ " unreliable process instances.");

		} else {
			System.out
					.println("\t"
							+ " Performance Report Info cannot be calcualted for Trace "
							+ count);
			context.log("\t"
					+ " Performance Report Info cannot be calcualted for Trace "
					+ count);

			isReliable = false;
		}
		if (isReliable) {
			/*
			 * call wit UI FlexPerfRepInfo
			 * flexPerfRepResult4EachSyncReplayResult =
			 * context.tryToFindOrConstructFirstObject( FlexPerfRepInfo.class,
			 * FlexPerfRepInfoConnection.class,
			 * FlexPerfRepInfoConnection.FLEXPERFREPINFO, originalModel,
			 * filteredLog);
			 */
			bpmnAnalysisData
					.setPerfRepInfo(flexPerfRepResult4EachSyncReplayResult);
			setCaseArrivalRatio(log, bpmnAnalysisData,
					flexPerfRepResult4EachSyncReplayResult);
			// put Performance report and BPMNAnalysisData to Map for reference
			// purposes
			perfBPMNAnalysisDataMap.put(flexPerfRepResult4EachSyncReplayResult,
					bpmnAnalysisData);
		}
		fillHalfOfProgressBar(context);
		count++;
	}


	public Object[] bpmnAnalysis(final UIPluginContext context, XLog log, Flex originalModel) {

		BPMNAnalysisSettingsGUI bpmnAnalysisDataSettingsGUI = new BPMNAnalysisSettingsGUI(new BPMNAnalysisSettings());
		InteractionResult result = context.showConfiguration("BPMN Analysis Settings", bpmnAnalysisDataSettingsGUI);
		if (result.equals(InteractionResult.CANCEL)) {
			context.getFutureResult(0).cancel(true);
			context.getFutureResult(1).cancel(true);
			context.getFutureResult(2).cancel(true);
			return null;
		}
		BPMNAnalysisSettings settings = bpmnAnalysisDataSettingsGUI.getSettings();

		String message = "Obtain Log Event Class mapping and replay settings";
		initProgressBar(context, message, log);

		ReplayerSettingsUI flexReplayerUI = new ReplayerSettingsUI(context);
		Object[] flexReplayerConfiguration = flexReplayerUI.getConfiguration(originalModel, log, false);

		if (flexReplayerConfiguration == null) {
			context.getFutureResult(0).cancel(true);
			context.getFutureResult(1).cancel(true);
			context.getFutureResult(2).cancel(true);
			context.log("Can't obtain connection for " + message);
			return null;
		}

		return bpmnAnalysis(context, log, originalModel, settings, flexReplayerConfiguration);
	}

	public Object[] bpmnAnalysis(final PluginContext context, XLog log, Flex originalModel) {
		System.out.println("loading default BPMN analysis settings");
		BPMNAnalysisSettings settings = new BPMNAnalysisSettings();

		System.out.println("loading default flex replayer configuration");
		Object[] flexReplayerConfiguration = new Object[3];

		ModelToLogConnection c = new ModelToLogConnection(originalModel, log);
		flexReplayerConfiguration[0] = c.getMapping();
		flexReplayerConfiguration[1] = new ExtendedCostBasedAStarLogReplayAlgorithm();
		ParamSettingExtendedCostBasedReplay p = new ParamSettingExtendedCostBasedReplay(false);
		flexReplayerConfiguration[2] = p.getAllParameters();

		return bpmnAnalysis(context, log, originalModel, settings, flexReplayerConfiguration);
	}

	public Object[] bpmnAnalysis(final PluginContext context, XLog log,
			Flex originalModel, BPMNAnalysisSettings bpmnAnalysisSettings,
			Object[] flexReplayerConfiguration) {

		System.out.println("starting BPMN analysis");

		BPMNAnalysisDataList bpmnAnalysisDataList = new BPMNAnalysisDataList();
		BPMNAnalysisData bpmnAnalysisData = new BPMNAnalysisData();

		BPMNDiagram bpmnWithoutSwimlanes = null;
		BPMNDiagram bpmnWithSwimlanes = null;
		String message = "";

		try {
			// Discover BPMN
			bpmnWithoutSwimlanes = discoverBPMNWithoutSwimlanes(context, log,
					originalModel);
			// Discover Swimlanes
			boolean doesContainUserData = doesContainUserData(log);
			if (doesContainUserData) {
				bpmnAnalysisData.setDoesContainUserInfo(true);
				bpmnWithSwimlanes = discoverSwimlanes(context, log,
						bpmnWithoutSwimlanes, bpmnAnalysisData, originalModel);
			} else {
				bpmnAnalysisData.setDoesContainUserInfo(false);

				// add default swimlane to satisfy requirements for XPDL export
				//Swimlane pool = bpmnWithoutSwimlanes.addSwimlane(SwimlaneDiscovery.DEFAULT_POOL_NAME, null);

				System.out
						.println("The Event Log contains no user data! Skipping discovery of swimlanes and social networks");
			}
			// Replay the log
			Object[] results = replayLog(context, originalModel, log,
					flexReplayerConfiguration);
			FlexPerfRepInfo flexPerfRepInfo;
			if (results[0] instanceof FlexPerfRepInfo) {
				flexPerfRepInfo = (FlexPerfRepInfo) results[0];
			} else {
				return results;
			}
			// Get codec
			FlexCodec codec = retriveFlexCodec(context, originalModel, log);

			// GET "REPLAY LOG ON C-NET FOR CONFORMANCE PLUG-IN" RESULT
			// (replayResult) PRODUCED IN REPLAY LOG ON C-NET FOR PERFORMANCE
			// PLUG-IN

			/*
			 * 1. WE GET REPLAY RESULT 2. WE FILTER THE LOG TO OBTAIN THE EVENT
			 * LOG OF A replayResult 3. WE MODIFY THE MAPPING BASED ON
			 * NON-OBSERVED ACTIVITIES 4. WE RUN PERFORMANCE ANALYSIS PLUG-IN
			 */
			System.out.println("STEP 1");
			// 1. WE GET REPLAY RESULT
			SortedSet<Integer> caseIDIndexList = new TreeSet<Integer>();
			Integer count = new Integer(0);
			BPMNAnalysisData bpmnAnalysisData2 = retrieveConformance(context,
					log, codec, caseIDIndexList, count, bpmnAnalysisDataList,
					flexPerfRepInfo);
			System.out.println("Case ID Index list = " + caseIDIndexList);

			// calculate performance infos for each replayResult
			// get the event log that contains only case ids in case id set
			// 2. WE FILTER THE LOG TO OBTAIN THE EVENT LOG OF A replayResult
			XLog filteredLog = filterLog(context, log, originalModel,
					caseIDIndexList, count, doesContainUserData,
					bpmnAnalysisData2, bpmnWithoutSwimlanes, bpmnWithSwimlanes);

			// 3. WE MODIFY THE MAPPING BASED ON NON-OBSERVED ACTIVITIES
			Map<FlexNode, XEventClass> mappings = (Map<FlexNode, XEventClass>) flexReplayerConfiguration[ReplayerSettingsUI.MAPPING];
			Map<FlexNode, XEventClass> newMappings = getNewFlexNodeEventClassMap(
					mappings, filteredLog, originalModel);

			// 4. WE RUN PERFORMANCE ANALYSIS PLUG-IN
			IFlexLogReplayAlgorithm iFlexLogReplayAlgorithm = (IFlexLogReplayAlgorithm) flexReplayerConfiguration[ReplayerSettingsUI.ALGORITHM];
			Map<FlexPerfRepInfo, BPMNAnalysisData> perfBPMNAnalysisDataMap = new HashMap<FlexPerfRepInfo, BPMNAnalysisData>();
			runPerformanceAnalysis(context, iFlexLogReplayAlgorithm,
					originalModel, filteredLog, newMappings, count,
					caseIDIndexList, bpmnAnalysisData2, log,
					perfBPMNAnalysisDataMap, flexReplayerConfiguration);

			// Fill Info
			if (!perfBPMNAnalysisDataMap.isEmpty()) {
				// find most and least frequent paths and get BPMNAnalysisData
				// belongs to those and put to bpmnAnalysisDataList
				fillBPMNAnalysisDataListBasedOnGUISettings(
						bpmnAnalysisDataList, bpmnAnalysisSettings,
						perfBPMNAnalysisDataMap);
			}
			ConnectionManager cm = context.getConnectionManager();
			
			cm.addConnection(new BPMNAnalysisConnection("Connection between"
					+ XConceptExtension.instance().extractName(log)
					+ ", Flex Model" + originalModel.getLabel()
					+ ", BPMN Model" + bpmnWithoutSwimlanes.getLabel(),
					bpmnWithoutSwimlanes, originalModel, bpmnAnalysisDataList));
			
			BPMNAnalysisConnection bpmnConnection = null;
			bpmnConnection = cm.getFirstConnection(
					BPMNAnalysisConnection.class, context, bpmnWithoutSwimlanes);
			
			message = "BPMNAnalysisWrapperPlugin";
			System.out.println("The following plug-in executed successfully: "
					+ message);
			context.log("The following plug-in executed successfully: "
					+ message);

			context.getFutureResult(0).setLabel("BPMN Without Swimlanes");
			context.getFutureResult(1).setLabel("BPMN With Swimlanes");
			context.getFutureResult(2).setLabel("BPMN Analysis Data List");
		} catch (Exception e) {
			e.printStackTrace();
		}

		return new Object[] { bpmnWithoutSwimlanes, bpmnWithSwimlanes,
				bpmnAnalysisDataList };
	}

	private boolean doesContainUserData(XLog log) {
		XLogInfo summary = XLogInfoFactory.createLogInfo(log);

		//List<String> originatorList = new ArrayList<String>();
		XEventClasses originators = summary.getResourceClasses();
		/*
		 * for (int k = 0; k < originators.getClasses().size(); k++) {
		 * originatorList.add(originators.getByIndex(k).toString()); }
		 */
		boolean result = true;

		if (originators.size() == 0)
			result = false;
		else {
			List<XEventClass> originatorsXEventClasses = new ArrayList<XEventClass>(originators.getClasses());
			for (XEventClass xEventClass : originatorsXEventClasses) {
				if ("".equals(xEventClass.toString()))
					result = false;
				else if (ARTIFICIAL_PROM.equals(xEventClass.toString())) {
					result = false;
				} else {
					result = true;
				}
			}
		}
		//TODO: Extend this for role and organizational info later on
		return result;
	}

	private FlexPerfRepInfo runPerformanceReplayerWithChildContext(PluginContext context, Flex originalModel, XLog log,
			Map<FlexNode, XEventClass> flexNodeEventClassMap, StartTaskNodesSet startTaskNodesSet,
			EndTaskNodesSet endTaskNodesSet, IFlexLogReplayAlgorithm iFlexLogReplayAlgorithm, Object[] parameters) {

		FlexPerfRepInfo flexPerfRepResult4EachSyncReplayResult = null;
		//FIXME keep a (clean) list of currently running child contexts
		//FIXME for the progress listener cancellation: cycle through this list and send cancellation messages
		final PluginContext child = context.createChildContext("replay");
		context.getPluginLifeCycleEventListeners().firePluginCreated(child);
		child.getLoggingListeners().add(new Logger() {

			public void log(String message, PluginContextID contextID, MessageLevel messageLevel) {
				// ignore
			}

			public void log(Throwable t, PluginContextID contextID) {
				//ignore
				//context.log(t);
			}

		});

		// find all plugins that can replay a log on a flexible model and with our replayer parameters
		Set<PluginParameterBinding> plugins = child.getPluginManager().getPluginsAcceptingOrdered(PluginContext.class,
				false, originalModel.getClass(), log.getClass(), flexNodeEventClassMap.getClass(),
				startTaskNodesSet.getClass(), endTaskNodesSet.getClass(), iFlexLogReplayAlgorithm.getClass(),
				parameters.getClass());

		// pick the plugin that produces performance information
		PluginParameterBinding performanceReplayPlugin = null; 
		Iterator<PluginParameterBinding> pIt = plugins.iterator();
		while (pIt.hasNext()) {
			PluginParameterBinding plugin = pIt.next();
			// need flex performance replay info 
			if (plugin.getPlugin().getReturnTypes().get(0) == org.processmining.plugins.flex.replayresult.performance.FlexPerfRepInfo.class) {
				performanceReplayPlugin = plugin;
				break;
			}
		}
		
		if (performanceReplayPlugin == null) {
			context.log("[BPMNAnalysis] Could not find performance replayer for Flex models. Skipping performance analysis.");
			System.out.println("[BPMNAnalysis] Could not find performance replayer for Flex models. Skipping performance analysis.");
			return null;
		}
			
		PluginExecutionResult result = performanceReplayPlugin.invoke(child, originalModel, log, flexNodeEventClassMap,
				startTaskNodesSet, endTaskNodesSet, iFlexLogReplayAlgorithm, parameters);

		//FIXME correctly handle exceptions
		try {
			result.synchronize();
		} catch (CancellationException e) {
			System.out.println("result.synchronize cannot be accomplished for this run of performance algorithm!");
			System.out.println(e);
			e.printStackTrace();
			flexPerfRepResult4EachSyncReplayResult = null;
		} catch (ExecutionException e) {
			System.out.println("result.synchronize cannot be accomplished for this run of performance algorithm!");
			System.out.println(e);
			e.printStackTrace();
			flexPerfRepResult4EachSyncReplayResult = null;
		} catch (InterruptedException e) {
			System.out.println("result.synchronize cannot be accomplished for this run of performance algorithm!");
			System.out.println(e);
			e.printStackTrace();
			flexPerfRepResult4EachSyncReplayResult = null;
		} catch (Exception e) {
			System.out.println("result.synchronize cannot be accomplished for this run of performance algorithm!");
			System.out.println(e);
			e.printStackTrace();
			flexPerfRepResult4EachSyncReplayResult = null;
		}
		if (result != null) {
			if (result.getResult(0) != null) {
				if (result.getResult(0) instanceof FlexPerfRepInfo) {
					flexPerfRepResult4EachSyncReplayResult = result.getResult(0);
				} else {
					flexPerfRepResult4EachSyncReplayResult = null;
				}
			}
		}
		return flexPerfRepResult4EachSyncReplayResult;
	}
	
	public static Object[] cancelExecution(PluginContext context, String message, String messageTitle) {

		if (messageTitle != null && context instanceof UIPluginContext) {
			JOptionPane.showMessageDialog(null,	message, messageTitle, JOptionPane.ERROR_MESSAGE);
		}
		
		context.getFutureResult(0).cancel(true);
		context.getFutureResult(1).cancel(true);
		context.getFutureResult(2).cancel(true);
		
		System.out.println(message);
		context.log(message);
		return null;
	}

	/*
	 * Modified from FlexLogReplayResultVisPanel
	 */
	private Pair<List<Object>, List<org.processmining.plugins.petrinet.replayresult.StepTypes>> getConformanceView(FlexCodec codec, SyncReplayResult res) {
		Map<FlexBinding, String> labelMapping = new HashMap<FlexBinding, String>();
		List<Object> strNodeInstance = new LinkedList<Object>();
		for (Object obj : res.getNodeInstance()) {
			if (obj instanceof String) {
				strNodeInstance.add(obj);
			} else {
				if (labelMapping.get(obj) != null) {
					strNodeInstance.add(labelMapping.get(obj));
				} else {
					// create new labeling
					FlexBinding flexBinding = (FlexBinding) obj;
					String resultStr = "{";
					Short encodedInputBinding = flexBinding.getEncodedInputBinding();
					if (encodedInputBinding == FlexCodec.BLANK) {
						resultStr += "Empty} ";
					} else if (encodedInputBinding == FlexCodec.EMPTYSET) {
						resultStr += "} ";
					} else {
						String lim = "";
						for (short input : codec.getIOBindingsFor(encodedInputBinding)) {
							resultStr += codec.decode(input);
							resultStr += lim;
							lim = ",";
						}
						resultStr += "} ";
					}
					resultStr += codec.decode(flexBinding.getEncodedNode());
					resultStr += " {";
					Short encodedOutputBinding = flexBinding.getEncodedOutputBinding();
					if (encodedOutputBinding == FlexCodec.BLANK) {
						resultStr += "Empty} ";
					} else if (encodedOutputBinding == FlexCodec.EMPTYSET) {
						resultStr += "} ";
					} else {
						String lim = "";
						for (short output : codec.getIOBindingsFor(encodedOutputBinding)) {
							resultStr += codec.decode(output);
							resultStr += lim;
							lim = ",";
						}
						resultStr += "} ";
					}
					labelMapping.put(flexBinding, resultStr);
					strNodeInstance.add(resultStr);
				}
			}

		}

		return new Pair<List<Object>, List<org.processmining.plugins.petrinet.replayresult.StepTypes>>(strNodeInstance, res.getStepTypes());
	}

	/**
	 * @param log
	 * @param BPMNAnalysisData
	 * @param flexPerfRepResult
	 */
	public void setCaseArrivalRatio(XLog log, BPMNAnalysisData bpmnAnalysisData, FlexPerfRepInfo flexPerfRepResult) {
		int frequency = flexPerfRepResult.getCaseInstance().getFrequency();
		if (frequency != 0)
			bpmnAnalysisData.setArrivalRate(calculateArrivalRate(log, frequency));
		else
			bpmnAnalysisData.setArrivalRate("0.0");

	}

	/**
	 * @param starttime
	 * @param endtime
	 * @param frequency
	 * @return
	 */
	public String calculateArrivalRate(XLog log, int frequency) {
		XLogInfo summary = XLogInfoFactory.createLogInfo(log);
		Date starttime = summary.getLogTimeBoundaries().getStartDate();
		Date endtime = summary.getLogTimeBoundaries().getEndDate();
		String arrivalRate = "0";

		if (starttime != null && endtime != null) {

			double timeInterval = endtime.getTime() - starttime.getTime();
			NumberFormat nf = NumberFormat.getInstance();
			nf.setMaximumFractionDigits(2);
			if (Double.compare(timeInterval, 1000 * 60 * 60 * 24) > 0) {
				// days...
				arrivalRate = (nf.format(frequency / (timeInterval / (1000 * 60 * 60 * 24))) + " process instance(s) per day");
			} else if (Double.compare(timeInterval, 1000 * 60 * 60) > 0) {
				// hours
				arrivalRate = (nf.format(frequency / (timeInterval / (1000 * 60 * 60))) + " process instance(s) per hour");
			} else if (Double.compare(timeInterval, 1000 * 60) > 0) {
				// minutes
				arrivalRate = (nf.format(frequency / (timeInterval / (1000 * 60))) + " process instance(s) per minute");
			} else if (Double.compare(timeInterval, 1000) > 0) {
				// seconds
				arrivalRate = (nf.format(frequency / (timeInterval / (1000))) + " process instance(s) per second");
			} else
				arrivalRate = (nf.format(frequency / timeInterval) + " process instance(s) per milisecond");
		}

		return arrivalRate;
	}

	/**
	 * @param log
	 * @param caseIDList
	 * @return
	 */
	public XLog getFilteredLogContainsCaseIDs(XLog log, List<String> caseIDList) {
		List<XTrace> removalXtraceList = new ArrayList<XTrace>();
		XLog filteredLog = (XLog) log.clone();
		removalXtraceList.clear();

		for (XTrace trace : filteredLog) {
			String traceId = XConceptExtension.instance().extractName(trace);
			if (!caseIDList.contains(traceId)) {
				removalXtraceList.add(trace);
			}
		}
		filteredLog.removeAll(removalXtraceList);
		return filteredLog;
	}

	/**
	 * @param log
	 * @param BPMNAnalysisData
	 */
	private void getSNforCompleteLog(XLog log, BPMNAnalysisData bpmnAnalysisData, PluginContext context) {
		String message = "";
		try {
			message = "WT Social Network";
			bpmnAnalysisData.setWT4CompleteLog(new SNWTMiner().socialnetwork(context, log));
			message = "HoW Social Network";
			bpmnAnalysisData.setHoW4CompleteLog(new SNHoWMiner().socialnetwork(context, log));
			// temp
			message = "ST Social Network";
			bpmnAnalysisData.setST4CompleteLog(getSN(log));
			// bpmnAnalysisData.setST4CompleteLog(new SNSTMiner().socialnetwork(context, log));
		} catch (Error e) {
			System.out.println("Social Network plug-in has thrown an error while getting " + message
					+ ". Please check your whether your event log is working with " + message + "plug-in");
		}

	}

	/**
	 * @param log
	 * @return
	 */
	public SocialNetwork getSN(XLog log) {
		int indexType = SNMinerOptions.SIMILAR_TASK + +SNMinerOptions.CORRELATION_COEFFICIENT;
		BasicOperation baseOprtation = OperationFactory.getOperation(indexType, log);
		DoubleMatrix2D matrix = baseOprtation.calculation();
		SocialNetwork sn = UtilOperation.generateSN(matrix, baseOprtation.getOriginatorList());
		return sn;
	}
	
	public static class BPMNAnalysisSettings {
		public int noLeastFrequentPaths;
		public int noMostFrequentPaths;
		public int noLeastTimeConsumingPaths;
		public int noMostTimeConsumingPaths;
		
		public BPMNAnalysisSettings() {
			noLeastFrequentPaths = 3;
			noMostFrequentPaths = 3;
			noLeastTimeConsumingPaths = 3;
			noMostTimeConsumingPaths = 3;
		}
	};

	/**
	 * @param bpmnAnalysisDataList
	 * @param bpmnAnalysisDataSettingsGUI
	 * @param perfbpmnAnalysisDataMap
	 */
	private void fillBPMNAnalysisDataListBasedOnGUISettings(BPMNAnalysisDataList bpmnAnalysisDataList,
			BPMNAnalysisSettings settings,
			Map<FlexPerfRepInfo, BPMNAnalysisData> perfbpmnAnalysisDataMap) {
		// find most and least frequent paths and get bpmnAnalysisData beongs to those and put to bpmnAnalysisDataList

		List<FlexPerfRepInfo> performanceReportInfoList = new ArrayList<FlexPerfRepInfo>(
				perfbpmnAnalysisDataMap.keySet());

		settings.noLeastFrequentPaths = (settings.noLeastFrequentPaths > performanceReportInfoList.size()) ? performanceReportInfoList
				.size() : settings.noLeastFrequentPaths;
				settings.noMostFrequentPaths = (settings.noMostFrequentPaths > performanceReportInfoList.size()) ? performanceReportInfoList
				.size() : settings.noMostFrequentPaths;
				settings.noLeastTimeConsumingPaths = (settings.noLeastTimeConsumingPaths > performanceReportInfoList.size()) ? performanceReportInfoList
				.size() : settings.noLeastTimeConsumingPaths;
				settings.noMostTimeConsumingPaths = (settings.noMostTimeConsumingPaths > performanceReportInfoList.size()) ? performanceReportInfoList
				.size() : settings.noMostTimeConsumingPaths;

		// Frequency Comparator
		Comparator<FlexPerfRepInfo> frequencyComparator = new Comparator<FlexPerfRepInfo>() {
			@Override
			public int compare(FlexPerfRepInfo o1, FlexPerfRepInfo o2) {
				if (o1.getCaseInstance().getFrequency() < o2.getCaseInstance().getFrequency())
					return 1;
				else if (o1.getCaseInstance().getFrequency() > o2.getCaseInstance().getFrequency())
					return -1;
				else
					return 0;
			}
		};

		Collections.sort(performanceReportInfoList, frequencyComparator);
		/*
		 * for (FlexPerfRepInfo flexPerfRepInfo2 : performanceReportInfoList) {
		 * System.out.println("Value: " +
		 * flexPerfRepInfo2.getCaseInstance().getFrequency()); }
		 */

		if (settings.noMostFrequentPaths != 0) {
			for (int i = 0; i < settings.noMostFrequentPaths; i++) {
				FlexPerfRepInfo flexPerfRepInfo2 = performanceReportInfoList.get(i);
				BPMNAnalysisData bpmnAnalysisData2 = perfbpmnAnalysisDataMap.get(flexPerfRepInfo2);
				String desc = bpmnAnalysisData2.getDescription() != null ? bpmnAnalysisData2.getDescription() + " and "
						: "";
				bpmnAnalysisData2.setDescription(desc + "Most Frequent Path " + (i + 1));
				bpmnAnalysisDataList.add(bpmnAnalysisData2);
			}
		}
		int j = 1;
		if (settings.noLeastFrequentPaths != 0) {
			for (int i = performanceReportInfoList.size(); i > (performanceReportInfoList.size() - settings.noLeastFrequentPaths); i--) {

				FlexPerfRepInfo flexPerfRepInfo2 = performanceReportInfoList.get(i - 1);
				BPMNAnalysisData bpmnAnalysisData2 = perfbpmnAnalysisDataMap.get(flexPerfRepInfo2);
				if (bpmnAnalysisData2.getDescription() != null) {
					String desc = bpmnAnalysisData2.getDescription() != null ? bpmnAnalysisData2.getDescription()
							+ " and " : "";
					bpmnAnalysisData2.setDescription(desc + "Least Frequent Path " + (j));
				} else {
					bpmnAnalysisData2.setDescription("Least Frequent Path " + (j));
					bpmnAnalysisDataList.add(bpmnAnalysisData2);
				}
				j++;
			}
		}

		// to find most and least time consuming paths and put to the map
		// Time Comparator
		Comparator<FlexPerfRepInfo> timeComparator = new Comparator<FlexPerfRepInfo>() {
			@Override
			public int compare(FlexPerfRepInfo o1, FlexPerfRepInfo o2) {
				if (o1.getCaseInstance().getAvgTimeSpan() < o2.getCaseInstance().getAvgTimeSpan())
					return 1;
				else if (o1.getCaseInstance().getAvgTimeSpan() > o2.getCaseInstance().getAvgTimeSpan())
					return -1;
				else
					return 0;
			}
		};

		Collections.sort(performanceReportInfoList, timeComparator);
		/*
		 * for (FlexPerfRepInfo flexPerfRepInfo2 : performanceReportInfoList) {
		 * System.out.println("Value: \t" + (int)
		 * flexPerfRepInfo2.getCaseInstance().getAvgTimeSpan()); }
		 */

		if (settings.noMostTimeConsumingPaths != 0) {
			for (int i = 0; i < settings.noMostFrequentPaths; i++) {
				FlexPerfRepInfo flexPerfRepInfo2 = performanceReportInfoList.get(i);
				BPMNAnalysisData bpmnAnalysisData2 = perfbpmnAnalysisDataMap.get(flexPerfRepInfo2);
				if (bpmnAnalysisData2.getDescription() != null) {
					String desc = bpmnAnalysisData2.getDescription() != null ? bpmnAnalysisData2.getDescription()
							+ " and " : "";
					bpmnAnalysisData2.setDescription(desc + "Most Time Consuming Path " + (i + 1));
				} else {
					bpmnAnalysisData2.setDescription("Most Time Consuming Path " + (i + 1));
					bpmnAnalysisDataList.add(bpmnAnalysisData2);
				}
			}
		}
		j = 1;
		if (settings.noLeastTimeConsumingPaths != 0) {
			for (int i = performanceReportInfoList.size(); i > (performanceReportInfoList.size() - settings.noLeastFrequentPaths); i--) {
				FlexPerfRepInfo flexPerfRepInfo2 = performanceReportInfoList.get(i - 1);
				BPMNAnalysisData bpmnAnalysisData2 = perfbpmnAnalysisDataMap.get(flexPerfRepInfo2);
				if (bpmnAnalysisData2.getDescription() != null) {
					String desc = bpmnAnalysisData2.getDescription() != null ? bpmnAnalysisData2.getDescription()
							+ " and " : "";
					bpmnAnalysisData2.setDescription(desc + "Least Time Consuming Path " + (j));

				} else {
					bpmnAnalysisData2.setDescription("Least Time Consuming Path " + (j));
					bpmnAnalysisDataList.add(bpmnAnalysisData2);
				}
				j++;
			}
		}
		for (BPMNAnalysisData bpmnAnalysisData2 : bpmnAnalysisDataList) {
			if (!bpmnAnalysisData2.getDescription().contains("Project")) {
				bpmnAnalysisData2.setDescription("Project " + bpmnAnalysisData2.getDescription());
			}
		}
	}

	/**
	 * @param log
	 * @param BPMNWithSwimlanes
	 * @param userActivityTripleList
	 * @param bpmnAnalysisData
	 */
	private void fillSwimlaneBasedSNAndOTMatricesToBPMNAnalysisData(XLog log, BPMNDiagram BPMNWithSwimlanes,
			List<Triple<String, String, Integer>> userActivityTripleList, BPMNAnalysisData bpmnAnalysisData,
			PluginContext context) {
		Map<Swimlane, SocialNetwork> swimlaneBasedSTSocialNetworkList = new HashMap<Swimlane, SocialNetwork>();
		Map<Swimlane, SocialNetwork> swimlaneBasedWTSocialNetworkList = new HashMap<Swimlane, SocialNetwork>();
		Map<Swimlane, SocialNetwork> swimlaneBasedHoWSocialNetworkList = new HashMap<Swimlane, SocialNetwork>();
		Map<DirectedGraphNode, List<Triple<String, String, Integer>>> swimlaneBasedOTMatrixListForSelectedSwimlaneMap = new HashMap<DirectedGraphNode, List<Triple<String, String, Integer>>>();
		Map<String, List<String>> swimlaneUserListMap = new HashMap<String, List<String>>();

		@SuppressWarnings("serial")
		Set<String> eventLabelSet = new LinkedHashSet<String>() {
			@Override
			public boolean contains(Object other) {
				boolean b = false;
				String value = null;
				if (other instanceof String) {
					value = (String) other;
					for (String string : this) {
						if (string.contains(value)) {
							b = true;
							break;
						} else {
							continue;
						}
					}
					return b;
				} else {
					return b;
				}
			}
		};
		List<String> activityNameList = getActivityClassNames(log);

		for (Swimlane swimlane : BPMNWithSwimlanes.getSwimlanes()) {
			if (doesLogContainAtLeastOneActivityInSwimlane(swimlane, activityNameList)) {

				eventLabelSet.clear();
				for (ContainableDirectedGraphElement cdge : swimlane.getChildren()) {
					if (cdge instanceof Activity) {
						getPredecessorActivitySet(BPMNWithSwimlanes, (Activity) cdge, eventLabelSet, activityNameList);
						eventLabelSet.add(((Activity) cdge).getLabel());
						getSuccessorActivitySet(BPMNWithSwimlanes, (Activity) cdge, eventLabelSet, activityNameList);
					}
				}
				//put eventLabelSet for sorting purposes into bpmnAnalysisData
				bpmnAnalysisData.getSelectedNodeUTMatrixSortedHeaderMap().put(swimlane,
						new LinkedHashSet<String>(eventLabelSet));

				// get Event Log that contains only Events In EventLabelList
				XLog filteredLog = getEventLog(log, eventLabelSet);

				// GET USER ACTIVITY MATRIX
				find(activityNameList, eventLabelSet, userActivityTripleList, swimlane, bpmnAnalysisData,
						BPMNWithSwimlanes, swimlaneBasedOTMatrixListForSelectedSwimlaneMap, swimlaneUserListMap, 5, 3);

				String message = "";
				try {
					message = "HoW Social Network";
					// RUN SOCIAL NETWORK PLUG-IN
					swimlaneBasedHoWSocialNetworkList.put(swimlane,
							new SNHoWMiner().socialnetwork(context, filteredLog));
					message = "WT Social Network";
					swimlaneBasedWTSocialNetworkList.put(swimlane, new SNWTMiner().socialnetwork(context, filteredLog));
					// temp
					message = "ST Social Network";
					swimlaneBasedSTSocialNetworkList.put(swimlane, getSN(filteredLog));
					// swimlaneBasedSTSocialNetworkList.put(swimlane, new SNSTMiner().socialnetwork(context, filteredLog));
				}  catch (Error e) {
					System.out.println("Social Network plug-in has thrown an error while getting " + message
							+ "for the activity: " + swimlane.getLabel()
							+ ". Please check your whether your event log is working with " + message + "plug-in");
				}
			} else {
				swimlaneBasedOTMatrixListForSelectedSwimlaneMap.put(swimlane, null);
				swimlaneBasedHoWSocialNetworkList.put(swimlane, null);
				swimlaneBasedWTSocialNetworkList.put(swimlane, null);
				swimlaneBasedSTSocialNetworkList.put(swimlane, null);
			}
		}
		bpmnAnalysisData.setSwimlaneUserListMap(swimlaneUserListMap);
		bpmnAnalysisData.setSwimlaneBasedSTSocialNetworkList(swimlaneBasedSTSocialNetworkList);
		bpmnAnalysisData.setSelectedSwimlaneUTMatrixList(swimlaneBasedOTMatrixListForSelectedSwimlaneMap);
		bpmnAnalysisData.setSwimlaneBasedWTSocialNetworkList(swimlaneBasedWTSocialNetworkList);
		bpmnAnalysisData.setSwimlaneBasedHoWSocialNetworkList(swimlaneBasedHoWSocialNetworkList);
	}

	private boolean doesLogContainAtLeastOneActivityInSwimlane(Swimlane swimlane, List<String> activityNameList) {
		for (ContainableDirectedGraphElement node : swimlane.getChildren()) {
			if (node instanceof Activity) {
				if (activityNameList.contains(((Activity) node).getLabel())) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * @param log
	 * @param eventLabelList
	 * @return
	 */
	private XLog getEventLog(XLog log, Set<String> eventLabelSet) {
		XLog filteredLog = (XLog) log.clone();
		List<XEvent> removalXEventList = new ArrayList<XEvent>();
		List<XTrace> removalXTraceList = new ArrayList<XTrace>();
		for (XTrace trace : filteredLog) {
			removalXEventList.clear();
			for (XEvent xEvent : trace) {
				String eventName = XConceptExtension.instance().extractName(xEvent);
				if (!eventLabelSet.contains(eventName)) {
					removalXEventList.add(xEvent);
				}
			}
			trace.removeAll(removalXEventList);
			if (trace.isEmpty()) {
				removalXTraceList.add(trace);
			}
		}
		filteredLog.removeAll(removalXTraceList);
		return filteredLog;
	}

	private void getPredecessorActivitySet(BPMNDiagram bpmn, BPMNNode node, Set<String> activitySet,
			List<String> activityNameList) {
		for (BPMNEdge<? extends BPMNNode, ? extends BPMNNode> inEdge : bpmn.getInEdges(node)) {
			BPMNNode bpmnNode = inEdge.getSource();
			if (bpmnNode instanceof Activity) {
				if (activityNameList.contains(bpmnNode.getLabel()))
					activitySet.add(((Activity) bpmnNode).getLabel());
			} else if (bpmnNode instanceof Gateway) {
				getPredecessorActivitySet(bpmn, bpmnNode, activitySet, activityNameList);
			}
		}

	}

	private void getSuccessorActivitySet(BPMNDiagram bpmn, BPMNNode node, Set<String> activitySet,
			List<String> activityNameList) {
		for (BPMNEdge<? extends BPMNNode, ? extends BPMNNode> outEdge : bpmn.getOutEdges(node)) {
			BPMNNode bpmnNode = outEdge.getTarget();
			if (bpmnNode instanceof Activity) {
				if (activityNameList.contains(bpmnNode.getLabel()))
					activitySet.add(((Activity) bpmnNode).getLabel());
			} else if (bpmnNode instanceof Gateway) {
				getSuccessorActivitySet(bpmn, bpmnNode, activitySet, activityNameList);
			}
		}

	}

	/**
	 * @param log
	 * @param originalModel
	 * @param userActivityTripleList
	 * @param bpmnAnalysisData
	 */
	private void fillActivityBasedSNAndOTMatricesToBPMNAnalysisData(XLog log, Flex originalModel, BPMNDiagram bpmn,
			List<Triple<String, String, Integer>> userActivityTripleList, BPMNAnalysisData bpmnAnalysisData,
			PluginContext context) {

		@SuppressWarnings("serial")
		Set<String> eventLabelSet = new LinkedHashSet<String>() {
			@Override
			public boolean contains(Object other) {
				boolean b = false;
				String value = null;
				if (other instanceof String) {
					value = (String) other;
					for (String string : this) {
						if (string.contains(value)) {
							b = true;
							break;
						} else {
							continue;
						}
					}
					return b;
				} else {
					return b;
				}
			}
		};

		Map<Activity, SocialNetwork> activityBasedHoWSocialNetworkList = new HashMap<Activity, SocialNetwork>();
		Map<Activity, SocialNetwork> activityBasedWTSocialNetworkList = new HashMap<Activity, SocialNetwork>();
		Map<Activity, SocialNetwork> activityBasedSTSocialNetworkList = new HashMap<Activity, SocialNetwork>();
		Map<DirectedGraphNode, List<Triple<String, String, Integer>>> activityBasedUTMatrixListForSelectedActivityMap = new HashMap<DirectedGraphNode, List<Triple<String, String, Integer>>>();
		Map<String, List<String>> activityUserListMap = new HashMap<String, List<String>>();
		List<String> activityNameList = getActivityClassNames(log);
		//TODO: Activity Flex Event Log Map is needed later on
		for (Activity activity : bpmn.getActivities()) {
			eventLabelSet.clear();
			getPredecessorActivitySet(bpmn, activity, eventLabelSet, activityNameList);
			eventLabelSet.add(activity.getLabel());
			getSuccessorActivitySet(bpmn, activity, eventLabelSet, activityNameList);

			if (activityNameList.contains(activity.getLabel())) {
				//put eventLabelSet for sorting purposes into bpmnAnalysisData
				bpmnAnalysisData.getSelectedNodeUTMatrixSortedHeaderMap().put(activity,
						new LinkedHashSet<String>(eventLabelSet));

				// GET USER ACTIVITY MATRIX 
				find(activityNameList, eventLabelSet, userActivityTripleList, activity, bpmnAnalysisData, bpmn,
						activityBasedUTMatrixListForSelectedActivityMap, activityUserListMap, 5, 3);

				// get Event Log that contains only Events In EventLabelList
				XLog filteredLog = getEventLog(log, eventLabelSet);

				// RUN HAND-OVER OF WORK SOCIAL NETWORK PLUG-IN AND SAVE TO ALL-IN ONE
				getSNForActivity(context, activityBasedHoWSocialNetworkList, activityBasedWTSocialNetworkList,
						activityBasedSTSocialNetworkList, activity, filteredLog);
			} else {
				activityBasedUTMatrixListForSelectedActivityMap.put(activity, null);
				activityBasedHoWSocialNetworkList.put(activity, null);
				activityBasedWTSocialNetworkList.put(activity, null);
				activityBasedSTSocialNetworkList.put(activity, null);
			}
		}
		bpmnAnalysisData.setActivityUserListMap(activityUserListMap);
		bpmnAnalysisData.setActivityBasedHoWSocialNetworkList(activityBasedHoWSocialNetworkList);
		bpmnAnalysisData.setActivityBasedWTSocialNetworkList(activityBasedWTSocialNetworkList);
		bpmnAnalysisData.setActivityBasedSTSocialNetworkList(activityBasedSTSocialNetworkList);
		bpmnAnalysisData.setSelectedActivityUTMatrixList(activityBasedUTMatrixListForSelectedActivityMap);
	}

	private void getSNForActivity(PluginContext context,
			Map<Activity, SocialNetwork> activityBasedHoWSocialNetworkList,
			Map<Activity, SocialNetwork> activityBasedWTSocialNetworkList,
			Map<Activity, SocialNetwork> activityBasedSTSocialNetworkList, Activity activity, XLog filteredLog) {
		String message = "";

		try {
			message = "HoW Social Network";
			activityBasedHoWSocialNetworkList.put(activity, new SNHoWMiner().socialnetwork(context, filteredLog));
			message = "WT Social Network";
			activityBasedWTSocialNetworkList.put(activity, new SNWTMiner().socialnetwork(context, filteredLog));
			// activityBasedSTSocialNetworkList.put(activity, new SNSTMiner().socialnetwork(context, filteredLog));
			// temp
			message = "ST Social Network";
			activityBasedSTSocialNetworkList.put(activity, getSN(filteredLog));
		} catch (Error e) {
			System.out.println("Social Network plug-in has thrown an error while getting " + message
					+ "for the activity: " + activity.getLabel()
					+ ". Please check your whether your event log is working with " + message + "plug-in");
		}
	}

	/**
	 * @param log
	 * @return
	 */
	public List<String> getActivityClassNames(XLog log) {
		List<String> activityNameList = new ArrayList<String>();

		XLogInfo summary = XLogInfoFactory.createLogInfo(log);

		XEventClasses activityClasses = summary.getNameClasses();

		for (int k = 0; k < activityClasses.getClasses().size(); k++) {
			activityNameList.add(activityClasses.getByIndex(k).toString());
		}
		return activityNameList;
	}

	/**
	 * @param log
	 * @param eventLabelList
	 * @return
	 */
	private XLog removeUnnecessaryOriginators(XLog log) {
		XLog filteredLog = (XLog) log.clone();
		List<XEvent> removalXEventList = new ArrayList<XEvent>();
		List<XTrace> removalXTraceList = new ArrayList<XTrace>();
		for (XTrace trace : filteredLog) {
			removalXEventList.clear();
			for (XEvent xEvent : trace) {
				String resource = XOrganizationalExtension.instance().extractResource(xEvent);
				if (UNKNOWN.equals(resource) || ARTIFICIAL_PROM.equals(resource)) {
					removalXEventList.add(xEvent);
				}
			}
			trace.removeAll(removalXEventList);
			if (trace.isEmpty()) {
				removalXTraceList.add(trace);
			}
		}
		filteredLog.removeAll(removalXTraceList);
		return filteredLog;
	}

	/**
	 * @param log
	 * @return
	 */
	private List<Triple<String, String, Integer>> getUserActivityTripleList(XLog log, BPMNAnalysisData bpmnAnalysisData/*
																														 * ,
																														 * boolean
																														 * withTransition
																														 */) {
		XLogInfo summary = XLogInfoFactory.createLogInfo(log);
		List<String> originatorList = new ArrayList<String>();
		XEventClasses originators = summary.getResourceClasses();

		bpmnAnalysisData.setNoOfResources(originators.getClasses().size());

		for (int k = 0; k < originators.getClasses().size(); k++) {
			originatorList.add(originators.getByIndex(k).toString());
		}

		List<String> activityNameList = new ArrayList<String>();
		DoubleMatrix2D otMatrix = null;
		/* if (!withTransition) { */
		XEventClasses activityNameClasses = summary.getNameClasses();
		for (int k = 0; k < activityNameClasses.getClasses().size(); k++) {
			activityNameList.add(activityNameClasses.getByIndex(k).toString());
		}
		int numOriginator = originatorList.size();

		otMatrix = DoubleFactory2D.sparse.make(numOriginator, activityNameList.size(), 0);

		for (XTrace trace : log) {
			for (int k = 0; k < trace.size(); k++) {
				XExtendedEvent xEvent = XExtendedEvent.wrap(trace.get(k));
				int row = originatorList.indexOf(xEvent.getResource());
				if (row == -1) {
					continue;
				}
				int column = activityNameList.indexOf(xEvent.getName());
				otMatrix.set(row, column, otMatrix.get(row, column) + 1.0);
			}
		}
		/*
		 * } else { XEventClasses activityEventClasses =
		 * summary.getEventClasses(); for (int k = 0; k <
		 * activityEventClasses.getClasses().size(); k++) {
		 * activityNameList.add(activityEventClasses.getByIndex(k).toString());
		 * } int numOriginator = originatorList.size();
		 * 
		 * otMatrix = DoubleFactory2D.sparse.make(numOriginator,
		 * activityNameList.size(), 0);
		 * 
		 * for (XTrace trace : log) { for (int k = 0; k < trace.size(); k++) {
		 * XExtendedEvent xEvent = XExtendedEvent.wrap(trace.get(k)); int row =
		 * originatorList.indexOf(xEvent.getResource()); if (row == -1) {
		 * continue; } int column = activityNameList.indexOf(xEvent.getName() +
		 * "+" + xEvent.getTransition()); otMatrix.set(row, column,
		 * otMatrix.get(row, column) + 1.0); } } }
		 */

		List<Triple<String, String, Integer>> userActivityTripleList = new ArrayList<Triple<String, String, Integer>>();
		for (int i = 0; i < otMatrix.rows(); i++) {
			for (int k = 0; k < otMatrix.columns(); k++) {
				String user = originatorList.get(i);
				String activity = activityNameList.get(k);
				Integer occurence = (int) otMatrix.get(i, k);
				Triple<String, String, Integer> userActivityTriple = new Triple<String, String, Integer>(user,
						activity, occurence / 2);
				userActivityTripleList.add(userActivityTriple);
			}
		}
		// sorting according to the number of times it is conducted by users
		Collections.sort(userActivityTripleList, new Comparator<Triple<String, String, Integer>>() {

			@Override
			public int compare(Triple<String, String, Integer> o1, Triple<String, String, Integer> o2) {
				if (o1.getThird().intValue() < o2.getThird().intValue())
					return 1;
				else if (o1.getThird().intValue() > o2.getThird().intValue())
					return -1;
				else
					return 0;
			}

		});
		return userActivityTripleList;
	}

	private void find(List<String> activityNameList, Set<String> eventLabelSet,
			List<Triple<String, String, Integer>> userActivityTripleList, DirectedGraphNode dnode,
			BPMNAnalysisData bpmnAnalysisData, BPMNDiagram bpmn,
			Map<DirectedGraphNode, List<Triple<String, String, Integer>>> UTMatrixListForSelectedNodeMap,
			Map<String, List<String>> userListMap, int noOfUser, int noOfUser2) {

		@SuppressWarnings("serial")
		Set<String> stringLabelSet = new HashSet<String>() {
			@Override
			public boolean contains(Object other) {
				boolean b = false;
				String value = null;
				if (other instanceof String) {
					value = (String) other;
					for (String string : this) {
						if (string.contains(value)) {
							b = true;
							break;
						} else {
							continue;
						}
					}
					return b;
				} else {
					return b;
				}
			}
		};

		// gets the user,activity, no of perform triple list for the events in eventlabel list
		List<Triple<String, String, Integer>> tempTripleList = new ArrayList<Triple<String, String, Integer>>();
		for (Triple<String, String, Integer> triple : userActivityTripleList) {
			if (eventLabelSet.contains(triple.getSecond())) {
				tempTripleList.add(triple);
			}
		}

		List<Triple<String, String, Integer>> resultTripleListForSelectedNode = new ArrayList<Triple<String, String, Integer>>();
		List<String> totalUsersList = new ArrayList<String>();
		List<String> userListForSelectedNodeList = new ArrayList<String>();
		List<String> userListForPredecessorNodeList = new ArrayList<String>();
		List<String> userListForSuccessorNodeList = new ArrayList<String>();

		if (dnode instanceof Swimlane) {
			Swimlane swimlane = (Swimlane) dnode;

			//fill stringLabelSet with the activities in swimlane
			for (ContainableDirectedGraphElement cdge : swimlane.getChildren()) {
				if (cdge instanceof Activity) {
					stringLabelSet.add(((Activity) cdge).getLabel());
				}
			}

			//List of different users which performed the activities
			userListForSelectedNodeList = getDifferentListOfUsers(stringLabelSet, tempTripleList,
					new ArrayList<String>());

			userListMap.put(swimlane.getLabel(), userListForSelectedNodeList);

			if (userListForSelectedNodeList.size() > noOfUser) {
				totalUsersList.addAll(userListForSelectedNodeList.subList(0, noOfUser));
			} else {
				totalUsersList.addAll(userListForSelectedNodeList);
			}

			// set the number of different users who conducted any activity in the selected swimlane
			bpmnAnalysisData.getNumberOfUsersPerSwimlaneMap().put(swimlane, userListForSelectedNodeList.size());

			//get top users
			for (Triple<String, String, Integer> triple : tempTripleList) {
				if (totalUsersList.contains(triple.getFirst())) {
					resultTripleListForSelectedNode.add(triple);
				}
			}

			//clear and fill the string labelSet with the Predecessor activities' labels
			stringLabelSet.clear();
			if (!swimlane.getChildren().isEmpty()) {
				for (ContainableDirectedGraphElement element : swimlane.getChildren()) {
					if (element instanceof Activity) {
						Activity activity = (Activity) element;
						getPredecessorActivitySet(bpmn, activity, stringLabelSet, activityNameList);
					}
				}
			}

			//List of different users which performed the Predecessor activities
			userListForPredecessorNodeList = getDifferentListOfUsers(stringLabelSet, tempTripleList, totalUsersList);

			List<String> temp = new ArrayList<String>();
			if (userListForPredecessorNodeList.size() > noOfUser) {
				totalUsersList.addAll(userListForPredecessorNodeList.subList(0, noOfUser2));
				temp.addAll(userListForPredecessorNodeList.subList(0, noOfUser2));
			} else {
				totalUsersList.addAll(userListForPredecessorNodeList);
				temp.addAll(userListForPredecessorNodeList);
			}

			//get top users
			for (Triple<String, String, Integer> triple : tempTripleList) {
				if (temp.contains(triple.getFirst())) {
					resultTripleListForSelectedNode.add(triple);
				}
			}

			//clear and fill the string labelSet with the Successor activities' labels
			stringLabelSet.clear();
			if (!swimlane.getChildren().isEmpty()) {
				for (ContainableDirectedGraphElement element : swimlane.getChildren()) {
					if (element instanceof Activity) {
						Activity activity = (Activity) element;
						getSuccessorActivitySet(bpmn, activity, stringLabelSet, activityNameList);
					}
				}
			}
			//List of different users which performed the Successor activities
			userListForSuccessorNodeList = getDifferentListOfUsers(stringLabelSet, tempTripleList, totalUsersList);

			temp.clear();
			if (userListForSuccessorNodeList.size() > noOfUser) {
				totalUsersList.addAll(userListForSuccessorNodeList.subList(0, noOfUser2));
				temp.addAll(userListForSuccessorNodeList.subList(0, noOfUser2));
			} else {
				totalUsersList.addAll(userListForSuccessorNodeList);
				temp.addAll(userListForSuccessorNodeList);
			}

			//get top users
			for (Triple<String, String, Integer> triple : tempTripleList) {
				if (temp.contains(triple.getFirst())) {
					resultTripleListForSelectedNode.add(triple);
				}
			}

			//fill to the map
			UTMatrixListForSelectedNodeMap.put(swimlane, resultTripleListForSelectedNode);

		} else if (dnode instanceof Activity) {
			Activity activity = (Activity) dnode;

			//fill stringLabelSet with the activity
			stringLabelSet.add(activity.getLabel());

			//List of different users which performed the activity
			userListForSelectedNodeList = getDifferentListOfUsers(stringLabelSet, tempTripleList,
					new ArrayList<String>());

			userListMap.put(activity.getLabel(), userListForSelectedNodeList);

			if (userListForSelectedNodeList.size() > noOfUser) {
				totalUsersList.addAll(userListForSelectedNodeList.subList(0, noOfUser));
			} else {
				totalUsersList.addAll(userListForSelectedNodeList);
			}

			// set the number of different users who conducted selected activity
			bpmnAnalysisData.getNumberOfUsersPerTaskMap().put(activity, userListForSelectedNodeList.size());

			//get top users
			for (Triple<String, String, Integer> triple : tempTripleList) {
				if (totalUsersList.contains(triple.getFirst())) {
					resultTripleListForSelectedNode.add(triple);
				}
			}

			//clear and fill the string labelSet with the Predecessor activities' labels
			stringLabelSet.clear();
			getPredecessorActivitySet(bpmn, activity, stringLabelSet, activityNameList);

			//List of different users which performed the Predecessor activities
			userListForPredecessorNodeList = getDifferentListOfUsers(stringLabelSet, tempTripleList, totalUsersList);

			List<String> temp = new ArrayList<String>();
			if (userListForPredecessorNodeList.size() > noOfUser) {
				totalUsersList.addAll(userListForPredecessorNodeList.subList(0, noOfUser2));
				temp.addAll(userListForPredecessorNodeList.subList(0, noOfUser2));
			} else {
				totalUsersList.addAll(userListForPredecessorNodeList);
				temp.addAll(userListForPredecessorNodeList);
			}

			//get top users
			for (Triple<String, String, Integer> triple : tempTripleList) {
				if (temp.contains(triple.getFirst())) {
					resultTripleListForSelectedNode.add(triple);
				}
			}

			//clear and fill the string labelSet with the Successor activities' labels
			stringLabelSet.clear();
			getSuccessorActivitySet(bpmn, activity, stringLabelSet, activityNameList);

			//List of different users which performed the Successor activities
			userListForSuccessorNodeList = getDifferentListOfUsers(stringLabelSet, tempTripleList, totalUsersList);

			temp.clear();
			if (userListForSuccessorNodeList.size() > noOfUser) {
				totalUsersList.addAll(userListForSuccessorNodeList.subList(0, noOfUser2));
				temp.addAll(userListForSuccessorNodeList.subList(0, noOfUser2));
			} else {
				totalUsersList.addAll(userListForSuccessorNodeList);
				temp.addAll(userListForSuccessorNodeList);
			}

			//get top users
			for (Triple<String, String, Integer> triple : tempTripleList) {
				if (temp.contains(triple.getFirst())) {
					resultTripleListForSelectedNode.add(triple);
				}
			}

			//fill to the map
			UTMatrixListForSelectedNodeMap.put(activity, resultTripleListForSelectedNode);
		}

	}

	private List<String> getDifferentListOfUsers(Set<String> stringLabelSet,
			List<Triple<String, String, Integer>> tempTripleList, List<String> existingUsersList) {
		List<String> usersList = new ArrayList<String>();
		for (Triple<String, String, Integer> triple : tempTripleList) {
			if (triple.getThird().intValue() != 0 && (stringLabelSet.contains(triple.getSecond()))) {
				if (!usersList.contains(triple.getFirst())) {
					usersList.add(triple.getFirst());
				}
			}
		}
		usersList.removeAll(existingUsersList);

		return usersList;
	}
	
	private Map<FlexNode, XEventClass> getNewFlexNodeEventClassMap(Map<FlexNode, XEventClass> mappings,
			XLog filteredLog, Flex originalModel) {
		XEventClassifier stdClassifier2 = XLogInfoImpl.NAME_CLASSIFIER;
		XLogInfo summaryName2 = XLogInfoFactory.createLogInfo(filteredLog, stdClassifier2);
		XEventClasses nameEventClasses2 = summaryName2.getEventClasses();
		Map<FlexNode, XEventClass> newMappings = new HashMap<FlexNode, XEventClass>();

		List<XEventClass> classes = new ArrayList<XEventClass>(nameEventClasses2.getClasses());
		if (!classes.isEmpty()) {
			for (FlexNode node : originalModel.getNodes()) {
				int index = classes.indexOf(mappings.get(node));
				if (index == -1) {
					newMappings.put(node, SimpleMappingStep.DUMMYEVENTCLASS);
				} else {
					newMappings.put(node, classes.get(index));
				}
			}
		}
		return newMappings;
	}

	private XLog filterEventlog(XLog log, SortedSet<Integer> caseIDIndexList, int count) {
		XLog filteredLog = (XLog) log.clone();
		filteredLog.clear();
		
		for (Integer traceindex : caseIDIndexList) {
			filteredLog.add(log.get(traceindex));
		}
		
		XConceptExtension.instance().assignName(filteredLog, "Filtered event log for Replay Result " + count);
		return filteredLog;
	}
}