package org.processmining.plugins.bpmn.analysis;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.Progress;
import org.processmining.models.flexiblemodel.Flex;
import org.processmining.plugins.log.logfilters.LogFilter;
import org.processmining.plugins.log.logfilters.XEventCondition;

public abstract class BPMNAnalysisPlugin_CallMiner extends BPMNAnalysis {

	private XLog filterLog(final PluginContext context, XLog log) throws Exception {
		Progress progress = context.getProgress();
		
		
		XLogInfo summaryName = XLogInfoFactory.createLogInfo(log);
		List<XEventClass> transitionClassList = new ArrayList<XEventClass>(summaryName.getTransitionClasses()
				.getClasses());

		List<String> transitionStrList = new ArrayList<String>();

		for (XEventClass xEventClass : transitionClassList) {
			transitionStrList.add(xEventClass.toString());
		}
		
		XLog filteredLog = log;
		final String selectedLifecycle;

		// multiple states: prioritize
		if (transitionStrList.contains("complete")) {
			selectedLifecycle = "complete";
		} else if (transitionStrList.contains("start")) {
			selectedLifecycle = "start";
		} else {
			String cancelMessage =
				  "Failed to identify a standard life-cycle transitions (start or complete):\n"
				+ "Please obtain a feasible log by filtering. The log should contain one life-cycle transition (start or complete) for each event.\n";
			throw new Exception(cancelMessage);
		}
	
		String message = "Filter log to obtain event log with events which have" + selectedLifecycle + "lifecycle";
		progress.setCaption(message);
		progress.setIndeterminate(false);
		progress.setMinimum(0);
		progress.setMaximum(log.size() + 2000);

		filteredLog = LogFilter.filter(context.getProgress(), 100, log,
				XLogInfoFactory.createLogInfo(log), new XEventCondition() {

					@Override
					public boolean keepEvent(XEvent event) {
						if (selectedLifecycle.equals(XLifecycleExtension.instance().extractTransition(event))) {
							return true;
						} else
							return false;
					}
				});
		progress.setValue(2000);

		Set<String> startEventSet = new HashSet<String>();
		Set<String> endEventSet = new HashSet<String>();
		for (XTrace xTrace : filteredLog) {
			for (XEvent xEvent : xTrace) {
				if (xTrace.indexOf(xEvent) == 0)
					startEventSet.add(XConceptExtension.instance().extractName(xEvent));
				else if (xTrace.indexOf(xEvent) == (xTrace.size() - 1))
					endEventSet.add(XConceptExtension.instance().extractName(xEvent));
			}
		}
		
		if (startEventSet.size() > 1) {
			String cancelMessage = 
				  "Log has multiple start events:\n"
				+ "Please filter the log to obtain a unique start event.\n";
			throw new Exception(cancelMessage);
		}
		if (endEventSet.size() > 1) {
			String cancelMessage = 
				  "Log has multiple end events:\n"
				+ "Please filter the log to obtain a unique end event.\n";
			throw new Exception(cancelMessage);
		}
		
		return filteredLog;
	}
	
	public abstract Flex mineFlexModel(final PluginContext context, XLog log) throws ConnectionCannotBeObtained, NoSuchMethodException, Exception ;
	
	public Object[] discover_and_analyze(final PluginContext context, XLog log) {

		Progress progress = context.getProgress();
			
		XLog filteredLog;
		try {
			filteredLog = filterLog(context, log);
			if (progress.isCancelled()) return null;
		} catch (Exception e) {
			return BPMNAnalysis.cancelExecution(context, e.getMessage(), "Failure During Log Filtering");
		}

		Flex flex;
		try {
			progress.setValue(log.size() + 2000);
			flex = mineFlexModel(context, filteredLog);
			if (progress.isCancelled()) return null;
			progress.setValue(2000);
		} catch (Exception e) {
			return BPMNAnalysis.cancelExecution(context, "Cannot discover Causal Net from log\n"+e.getMessage(), "Failure During Discovery");
		}
			
		try {
			if (context instanceof UIPluginContext)
				return bpmnAnalysis((UIPluginContext)context, log, flex);
			else
				return bpmnAnalysis(context, log, flex);
			
		} catch (Exception e) {
			e.printStackTrace();
			return BPMNAnalysis.cancelExecution(context, "Can't create and annotate BPMN model\n"+ e.getMessage(), "BPMN Analysis Error");
		}
	}
	
}
