package org.processmining.plugins.bpmn.analysis;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.flexiblemodel.Flex;
import org.processmining.models.graphbased.directed.bpmn.BPMNDiagram;
import org.processmining.plugins.bpmn.analysis.data.BPMNAnalysisDataList;

@Plugin(name = "BPMN Analysis (using Causal Net Miner)",
		parameterLabels = { "Log" },
		returnLabels = { "BPMN Model without Swimlanes", "BPMN Model with Swimlanes", "BPMN Analysis Data List" }, 
		returnTypes = { BPMNDiagram.class, BPMNDiagram.class, BPMNAnalysisDataList.class }, 
		userAccessible = true, 
		help = "BPMN Analysis (using Causal Net Miner)")
public class BPMNAnalysisPlugin_CausalNetMiner extends BPMNAnalysisPlugin_CallMiner {
	
	@UITopiaVariant(uiLabel = "BPMN Analysis (using Causal Net Miner)", 
					affiliation = UITopiaVariant.EHV, 
					author = "I. Bayraktar", 
					email = "ib@processuniverse.net", 
					website = "http://www.processuniverse.net")
	@PluginVariant(variantLabel = "BPMN Analysis (using Causal Net Miner)", requiredParameterLabels = { 0 })
	public Object[] bpmnAnalysisWrapperWithCausalNetMiner(final UIPluginContext context, XLog log) {
		return discover_and_analyze(context, log);
	}

	@PluginVariant(variantLabel = "BPMN Analysis (using Causal Net Miner)", requiredParameterLabels = { 0 })
	public Object[] bpmnAnalysisWrapperWithCausalNetMiner(final PluginContext context, XLog log) {
		return discover_and_analyze(context, log);
	}
	
	public Flex mineFlexModel(PluginContext context, XLog log) throws ConnectionCannotBeObtained, NoSuchMethodException, Exception {


		Flex flex2 = context.tryToFindOrConstructFirstNamedObject(Flex.class, "Mine for a Causal Net using Heuristics Miner (BPMNAnalysis)", 
				null, null, log);
		return flex2;
		
		/*
		// explicitly run the local copy of flexible heuristics miner plugin 
		PluginDescriptor cnetMiner = context.getPluginManager().getPlugin("org.processmining.plugins.bpmn.analysis.miners.FlexibleHeuristicsMinerCNetPlugin");
		List<PluginParameterBinding> cnetMinerBindings = PluginParameterBinding.Factory.tryToBind(context.getPluginManager(), cnetMiner, 0, true, true, XLog.class);
		
		if (cnetMinerBindings.isEmpty()) 
			throw new NoSuchMethodException("Could not find FlexibleHeuristicsMinerCNetPlugin");
		
		PluginParameterBinding cnetMinerBinding = cnetMinerBindings.get(0);

		PluginContext ilp_context = context.createChildContext("CNet mining");
		context.getPluginLifeCycleEventListeners().firePluginCreated(ilp_context);
		PluginExecutionResult pluginResult = cnetMinerBinding.invoke(ilp_context, log);
		pluginResult.synchronize();
		
		Flex flex = pluginResult.<Flex> getResult(0);
		return flex;
		*/
	}
}
