package org.processmining.plugins.bpmn.analysis;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.Progress;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.bpmn.FlexHNConnection;
import org.processmining.models.flexiblemodel.Flex;
import org.processmining.models.graphbased.directed.bpmn.BPMNDiagram;
import org.processmining.plugins.bpmn.analysis.data.BPMNAnalysisDataList;
import org.processmining.plugins.bpmn.analysis.miners.FlexibleHeuristicsMinerPlugin;
import org.processmining.plugins.heuristicsnet.AnnotatedHeuristicsNet;

@Plugin(name = "BPMN Analysis (using Heuristics Miner)",
		parameterLabels = { "Log" },
		returnLabels = { "BPMN Model without Swimlanes", "BPMN Model with Swimlanes", "BPMN Analysis Data List" }, 
		returnTypes = { BPMNDiagram.class, BPMNDiagram.class, BPMNAnalysisDataList.class }, 
		userAccessible = true, 
		help = "BPMN Analysis (using Heuristics Miner)")
public class BPMNAnalysisPlugin_HeuristicsMiner extends BPMNAnalysisPlugin_CallMiner {
	
	@UITopiaVariant(uiLabel = "BPMN Analysis (using Heuristics Miner)",
					affiliation = UITopiaVariant.EHV,
					author = "I. Bayraktar",
					email = "ib@processuniverse.net", 
					website = "http://www.processuniverse.net")
	@PluginVariant(variantLabel = "BPMN Analysis (using Heuristics Miner)", requiredParameterLabels = { 0 })
	public Object[] bpmnAnalysisWrapperWithHeuristicsNetMiner(final UIPluginContext context, XLog log) {
		return discover_and_analyze(context, log);
	}
	
	@PluginVariant(variantLabel = "BPMN Analysis (using Heuristics Miner)", requiredParameterLabels = { 0 })
	public Object[] bpmnAnalysisWrapperWithHeuristicsNetMiner(final PluginContext context, XLog log) {
		return discover_and_analyze(context, log);
	}

	public Flex mineFlexModel(final PluginContext context, XLog log) throws ConnectionCannotBeObtained {
		String message = "Mine for a Heuristics Net using Heuristics Miner";
		Progress progress = context.getProgress();
		progress.setCaption(message);

		/*
		 * HeuristicsNet HNModel =
		 * context.tryToFindOrConstructFirstNamedObject
		 * (HeuristicsNet.class, "Flexible Heuristics Miner", null,
		 * null, filteredXlog);
		 */
		
		AnnotatedHeuristicsNet ahn = (AnnotatedHeuristicsNet) FlexibleHeuristicsMinerPlugin.run(
				context.createChildContext(message), log);

		message = "HeuristicsNet to C-Net Converter";
		progress.setCaption(message);
		Flex flex = context.tryToFindOrConstructFirstObject(Flex.class, FlexHNConnection.class,
				FlexHNConnection.FLEX, ahn);
		
		return flex;
	}
}
