/**
 * 
 */
package org.processmining.plugins.bpmn.analysis.decorators;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.text.NumberFormat;

import org.processmining.models.graphbased.directed.bpmn.elements.IGraphElementDecoration;
import org.processmining.plugins.bpmn.analysis.utils.PerformanceUtils;

/**
 * @author ibayraktar
 * 
 */
public class ActivityDecorator implements IGraphElementDecoration {
	/**
	 * GUI RELATED VARIABLES
	 */

	public static final int PADDINGFROMBOX = 5;

	public static final int VERTICALBOXWIDTH = 20;
	public static final int VERTICALBOXHEIGHT = 60;
	
	public static final int HORIZONTALBOXWIDTH = 120;
	public static final double MINWIDTHRATIO = 0.25;
	public static final int HORIZONTALBOXHEIGHT = 10;

	public static final int PADDINGFROMBOXTOTEXT = 3;
	public static final int VERTICALPADDINGFROMBOXTOTEXT = 7;
	public static final int TEXTHEIGHT = 5;
	public static final int BRANCHINGBOXWIDTH = 15;
	public static final int MINHEIGHT = 60;
	public static final int MAXLABELCHAR = 21;

	// utility
	private final NumberFormat nf;

	// visualization data
	ActivityDecoratorData decor = null;

	// for this node, no information can be obtained by replaying
	public ActivityDecorator() {
		nf = null;
	}

	public ActivityDecorator(ActivityDecoratorData decor) {
		// init utility
		nf = NumberFormat.getInstance();
		nf.setMaximumFractionDigits(2);

		// init data
		this.decor = decor;
	}

	public void decorate(Graphics2D g2d, double x, double y, double width, double height) {
		/**
		 * Label frequency and also throughput time
		 */

		g2d.setFont(new Font(g2d.getFont().getFamily(), g2d.getFont().getStyle(), 10));
		FontMetrics fm = g2d.getFontMetrics();

		// fill in the semantic box
		GeneralPath path = new GeneralPath();

		// general stroke
		BasicStroke stroke = new BasicStroke((float) 1.0);
		g2d.setStroke(stroke);

		/**
		 * PERF Boxes
		 */
		// Synch Box
		g2d.setColor(new Color(50, 50, 50));
		double xForSynchTimeBox = x;
		double yForSynchTimeBox = y;
		boolean isSynchTimeAvailable = Double.compare(Double.NaN, decor.getAvgSynchTime()) != 0 && Double.compare(0.0, decor.getAvgSynchTime()) < 0;
		if (isSynchTimeAvailable) {
			xForSynchTimeBox = x + PADDINGFROMBOXTOTEXT;
			yForSynchTimeBox = y + (2 * VERTICALPADDINGFROMBOXTOTEXT);
			java.awt.Shape synchTimeShape = new Rectangle2D.Double(xForSynchTimeBox, yForSynchTimeBox, VERTICALBOXWIDTH, VERTICALBOXHEIGHT);
			path.append(synchTimeShape, false);
			g2d.fill(path);

			g2d.setColor(decor.getSyncColor());
			java.awt.Shape synchTimeSemanticShape = new Rectangle2D.Double(xForSynchTimeBox, yForSynchTimeBox + VERTICALBOXHEIGHT * (1 - decor.getAvgSynchTimeRatio()),
					VERTICALBOXWIDTH, VERTICALBOXHEIGHT * decor.getAvgSynchTimeRatio());
			path.reset();
			path.append(synchTimeSemanticShape, false);
			g2d.fill(path);

			g2d.setColor(new Color(50, 50, 50));

			String text = PerformanceUtils.getSuitableTimeStr(decor.getAvgSynchTime());
			g2d.drawString(text, Math.round(xForSynchTimeBox), Math.round(yForSynchTimeBox + VERTICALBOXHEIGHT * (1 - decor.getAvgSynchTimeRatio()) + 10) - 1);

		}
		// Coordinates for total, working, waiting time boxes
		double xForTotalTimeBox = (xForSynchTimeBox == x) ? xForSynchTimeBox + PADDINGFROMBOX : (xForSynchTimeBox + PADDINGFROMBOX + VERTICALBOXWIDTH);
		double yForTotalTimeBox = y + height - 2 * HORIZONTALBOXHEIGHT - PADDINGFROMBOX;
		double totalTimeShapeWidth = width - ((xForSynchTimeBox == x) ? (xForSynchTimeBox + 2 *PADDINGFROMBOX) : (xForSynchTimeBox + 2*PADDINGFROMBOX + VERTICALBOXWIDTH));
		double waitingTimeShapeWidth = 0;
		if ((Double.compare(Double.NaN, decor.getAvgTotalTime()) != 0 && Double.compare(0.0, decor.getAvgTotalTime()) != 0)) {
			if ((Double.compare(Double.NaN, decor.getAvgWaitingTime()) != 0 && Double.compare(0.0, decor.getAvgWaitingTime()) != 0)) {
				waitingTimeShapeWidth = ((totalTimeShapeWidth / decor.getAvgTotalTime()) * decor.getAvgWaitingTime());
				waitingTimeShapeWidth = (waitingTimeShapeWidth <= width*MINWIDTHRATIO) ? width*MINWIDTHRATIO : waitingTimeShapeWidth;
			}
			else {
				waitingTimeShapeWidth = 30;
			}
		}
		else {
			waitingTimeShapeWidth = totalTimeShapeWidth / 2;
		}
		double workingTimeShapeWidth = totalTimeShapeWidth - waitingTimeShapeWidth;
		
		workingTimeShapeWidth=(workingTimeShapeWidth<30)?30:workingTimeShapeWidth;
		waitingTimeShapeWidth=totalTimeShapeWidth -workingTimeShapeWidth;
		
		// total time box
		g2d.setColor(new Color(50, 50, 50));
		java.awt.Shape totalTimeShape = new Rectangle2D.Double(xForTotalTimeBox, yForTotalTimeBox, totalTimeShapeWidth, HORIZONTALBOXHEIGHT);
		path.reset();
		path.append(totalTimeShape, false);
		g2d.fill(path);

		// waiting time box
		g2d.setColor(new Color(50, 50, 50));
		java.awt.Shape waitingTimeShape = new Rectangle2D.Double(xForTotalTimeBox, yForTotalTimeBox + HORIZONTALBOXHEIGHT, waitingTimeShapeWidth, HORIZONTALBOXHEIGHT);
		path.reset();
		path.append(waitingTimeShape, false);
		g2d.fill(path);

		// working time box
		g2d.setColor(new Color(50, 50, 50));
		java.awt.Shape workingTimeShape = new Rectangle2D.Double(xForTotalTimeBox + waitingTimeShapeWidth, yForTotalTimeBox + HORIZONTALBOXHEIGHT, workingTimeShapeWidth,
				HORIZONTALBOXHEIGHT);
		path.reset();
		path.append(workingTimeShape, false);
		g2d.fill(path);

		// Total Time semantics
		if (Double.compare(Double.NaN, decor.getAvgTotalTime()) != 0 && Double.compare(0.0, decor.getAvgTotalTime()) != 0) {

			g2d.setColor(decor.getTotalColor());
			java.awt.Shape totalTimeSemanticShape = new Rectangle2D.Double(xForTotalTimeBox, yForTotalTimeBox, totalTimeShapeWidth, HORIZONTALBOXHEIGHT - 1);
			path.reset();
			path.append(totalTimeSemanticShape, false);
			g2d.fill(path);
		}
		g2d.setColor(new Color(50, 50, 50));
		String text = PerformanceUtils.getSuitableTimeStr(decor.getAvgTotalTime());
		g2d.drawString(text, Math.round(xForTotalTimeBox + width -2*PADDINGFROMBOX - fm.stringWidth(text)-(isSynchTimeAvailable?VERTICALBOXWIDTH:0)), Math.round(yForTotalTimeBox) - 1);

		// Waiting Time semantics
		if ((Double.compare(Double.NaN, decor.getAvgWaitingTime()) != 0 && Double.compare(0.0, decor.getAvgWaitingTime()) != 0)) {
			g2d.setColor(decor.getWaitColor());
			java.awt.Shape waitingTimeSemanticShape = new Rectangle2D.Double(xForTotalTimeBox, yForTotalTimeBox + HORIZONTALBOXHEIGHT + 1, waitingTimeShapeWidth - 1,
					HORIZONTALBOXHEIGHT);
			path.reset();
			path.append(waitingTimeSemanticShape, false);
			g2d.fill(path);
		}

		// Working Time semantics
		if ((Double.compare(Double.NaN, decor.getAvgWorkingTime()) != 0 && Double.compare(0.0, decor.getAvgWorkingTime()) != 0)) {
			g2d.setColor(decor.getWorkingColor());
			java.awt.Shape workingTimeSemanticShape = new Rectangle2D.Double(xForTotalTimeBox + waitingTimeShapeWidth + 1, yForTotalTimeBox + HORIZONTALBOXHEIGHT + 1,
					workingTimeShapeWidth-1, HORIZONTALBOXHEIGHT);
			path.reset();
			path.append(workingTimeSemanticShape, false);
			g2d.fill(path);
		}

		if ((Double.compare(Double.NaN, decor.getAvgWaitingTime()) != 0 && Double.compare(0.0, decor.getAvgWaitingTime()) != 0)) g2d.setColor(new Color(50, 50, 50));
		else g2d.setColor(Color.WHITE);

		text = PerformanceUtils.getSuitableTimeStr(decor.getAvgWaitingTime());
		g2d.drawString(text, Math.round(xForTotalTimeBox), Math.round(yForTotalTimeBox + 2 * HORIZONTALBOXHEIGHT) - 1);

		if ((Double.compare(Double.NaN, decor.getAvgWorkingTime()) != 0 && Double.compare(0.0, decor.getAvgWorkingTime()) != 0)) g2d.setColor(new Color(50, 50, 50));
		else g2d.setColor(Color.WHITE);

		text = PerformanceUtils.getSuitableTimeStr(decor.getAvgWorkingTime());
		g2d.drawString(text, Math.round(xForTotalTimeBox + width -2*PADDINGFROMBOX- fm.stringWidth(text)-(isSynchTimeAvailable?VERTICALBOXWIDTH:0)), Math.round(yForTotalTimeBox + 2 * HORIZONTALBOXHEIGHT) - 1);

		/**
		 * Draw frequency of the node
		 */
		g2d.setColor(Color.BLACK);

		text = "#" + String.valueOf(decor.getFrequency() + decor.getUnfinishedActivityFrequency());
		g2d.drawString(text, Math.round(x + width - 2* PADDINGFROMBOX - fm.stringWidth(text)), Math.round(y + 15));

		/**
		 * Draw percentage/probablity freq of the node
		 */
		g2d.setColor(Color.BLACK);
		text = "%" + String.valueOf(nf.format(decor.getCaseInvolvementRatio() * 100));
		if (Double.compare(Double.NaN, decor.getAvgSynchTime()) != 0 && Double.compare(0.0, decor.getAvgSynchTime()) != 0) {
			g2d.drawString(text, Math.round(x + 2 * PADDINGFROMBOX + VERTICALBOXWIDTH), Math.round(y + 15));
		}
		else {
			g2d.drawString(text, Math.round(x + PADDINGFROMBOX), Math.round(y + 15));
		}

	}

	public ActivityDecoratorData getDecor() {
		return decor;
	}

	public void setDecor(ActivityDecoratorData decor) {
		this.decor = decor;
	}

}
