package org.processmining.plugins.bpmn.analysis.pn.visualization;


/**
 * @author ibayraktar
 * 
 */
public class BPMNPerformanceProjectionPanel /*extends JPanel implements ViewInteractionPanel*/ {
	/*private static final long serialVersionUID = 3172287032216250110L;

	// for formatting purpose
	private static final Border EMPTYBORDER = BorderFactory.createEmptyBorder(2, 5, 2, 5);
	private NumberFormat nf = NumberFormat.getInstance();

	// for hover
	protected int visualizationSelection = -1;

	// VISUALIZATION PARAMETER
	// NODES
	public final static int NODEWIDTH = 140;
	public final static int NODEHEIGHT = 80;
	public final static int MAXNODEADDITIONALWIDTH = 30;
	public final static int UNVISITEDNODEWIDTH = 140;
	public final static int UNVISITEDHEIGHT = 80;

	// EDGES
	public final static float NORMALLINEWIDTH = 1f;
	public final static float MINARCWIDTH = 1f;
	public final static float MAXARCADDITIONWIDTH = 3f;

	// PANELS TO VIEW AND CONTROL

	// performance/conformance info panel
	private PerfConfInteractionPanel confPerfAnalysisPanel = null;
	private JPanel perfPanel = new JPanel();
	private JPanel confPanel = new JPanel();

	// social network panels
	private OrganizationalInteractionPanel HandOverOfWorkSNPanel = null;
	private OrganizationalInteractionPanel WorkingTogetherSNPanel = null;
	private OrganizationalInteractionPanel DoingSimilarTasksSNPanel = null;

	// user-activity matrices for activities and swimlanes
	private OrganizationalInteractionPanel activitybasedUAPanel = null;
	private OrganizationalInteractionPanel swimlanebasedUAPanel = null;

	// to control performance projection
	boolean isProjected = false;

	// PROJECTION VIEW INTERACTION PANEL
	protected final String title;
	private ProMJGraphPanel graphPanel;
	protected ProMJGraph graph;
	protected final JScrollPane scroll;
	protected final JList component;
	protected SlickerFactory factory = SlickerFactory.instance();
	protected SlickerDecorator decorator = SlickerDecorator.instance();
	protected ViewSpecificAttributeMap originalSpecificMap;
	protected ViewSpecificAttributeMap viewSpecificMap;

	private JSlider swimlaneSlider;
	private JLabel sliderMinValue, sliderMaxValue, sliderValue;

	private String name;
	private JLabel frequencyJLabel, arrivalRateJLabel, avgThroughputJLabel, minThroughputJLabel, maxThroughputJLabel,
			stdDevThroughputJLabel, fitnessLabel, minFitnessJLabel, maxFitnessJLabel;

	// reference to model and between models
	private BPMNDiagram bpmn = null;
	private Flex flex = null;
	private Map<FlexNode, BPMNNode> flexBPMNNodeMapping = new HashMap<FlexNode, BPMNNode>();
	private Map<BPMNNode, FlexNode> bpmnFlexNodeMapping = new HashMap<BPMNNode, FlexNode>();
	private Map<FlexEdge<? extends FlexNode, ? extends FlexNode>, BPMNEdge<BPMNNode, BPMNNode>> flexBPMNEdgeMapping = new HashMap<FlexEdge<? extends FlexNode, ? extends FlexNode>, BPMNEdge<BPMNNode, BPMNNode>>();
	private Map<BPMNEdge<BPMNNode, BPMNNode>, FlexEdge<? extends FlexNode, ? extends FlexNode>> bpmnFlexEdgeMapping = new HashMap<BPMNEdge<BPMNNode, BPMNNode>, FlexEdge<? extends FlexNode, ? extends FlexNode>>();

	private FlexCodec codec = null;

	// reference to BPMNAnalysis Data information
	private BPMNAnalysisDataList BPMNAnalysisDataList;
	private BPMNAnalysisData selectedBPMNAnalysisData;

	public BPMNPerformanceProjectionPanel(String name, String title, PluginContext context, BPMNDiagram bpmn,
			FlexCodec codec, Flex flex, BPMNAnalysisDataList BPMNAnalysisDataList, ProMJGraphPanel graphPanel) {
		super(new BorderLayout());

		this.name = name;
		this.title = title;
		this.component = new JList();
		this.component.setOpaque(false);
		this.graphPanel = graphPanel;
		this.BPMNAnalysisDataList = BPMNAnalysisDataList;
		// add the title
		JLabel panelTitle = factory.createLabel(title);
		panelTitle.setHorizontalTextPosition(SwingConstants.CENTER);
		panelTitle.setVerticalTextPosition(SwingConstants.CENTER);
		panelTitle.setForeground(Color.WHITE);
		panelTitle.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 16));
		panelTitle.setOpaque(false);
		this.add(panelTitle, BorderLayout.NORTH);
		this.setOpaque(false);

		this.scroll = new JScrollPane(this.component);
		decorator.decorate(scroll, Color.WHITE, Color.GRAY, Color.DARK_GRAY);
		scroll.getViewport().setOpaque(false);
		scroll.setOpaque(false);
		scroll.setBorder(BorderFactory.createEmptyBorder());
		scroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		scroll.getVerticalScrollBar().setOpaque(false);

		this.add(scroll, BorderLayout.CENTER);

		ListCellRenderer renderer = new ViewListCellRenderer();
		this.component.setCellRenderer(renderer);
		this.component.setSelectionModel(new ToggleSelectionModel());

		component.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		// add reference to original model and other necessary things
		this.setBpmn(bpmn);
		this.flex = flex;
		this.codec = codec;
		this.selectedBPMNAnalysisData = BPMNAnalysisDataList.get(0);

		fillFlexBPMNNodeEdgeMappings(bpmn, flex);

		// formatting numbers
		nf.setMinimumFractionDigits(3);
		nf.setMaximumFractionDigits(3);

		// visualization from JGraph
		setScalableComponent(graphPanel.getGraph());
		
		// implement selection listener, it is interactive
		ListSelectionListener listSelectionListener = new ListSelectionListener() {

			public void valueChanged(ListSelectionEvent e) {
				if (e.getValueIsAdjusting()) {
					return;
				}
				ListSelectionModel lsm = component.getSelectionModel();
				int selectedIndex = lsm.getMinSelectionIndex();
				if (selectedIndex < 0) {
					component.clearSelection();
					uncolorElements();
					isProjected = false;
					return;
				}

				selectedBPMNAnalysisData = getBPMNAnalysisDataList().get(selectedIndex);
				updateConfPerfAnalysisPanel();

				HandOverOfWorkSNPanel.setBPMNAnalysisData(selectedBPMNAnalysisData);
				WorkingTogetherSNPanel.setBPMNAnalysisData(selectedBPMNAnalysisData);
				DoingSimilarTasksSNPanel.setBPMNAnalysisData(selectedBPMNAnalysisData);
				activitybasedUAPanel.setBPMNAnalysisData(selectedBPMNAnalysisData);
				HandOverOfWorkSNPanel.repaintAll(false);
				WorkingTogetherSNPanel.repaintAll(false);
				DoingSimilarTasksSNPanel.repaintAll(false);
				activitybasedUAPanel.repaintAll(false);

				isProjected = true;
				uncolorElements();
				fillMapAndRefreshEdges(Color.LIGHT_GRAY);

				colorElements();
				visualizationSelection = selectedIndex;
			}

		};
		component.addListSelectionListener(listSelectionListener);
		component.setModel(new DefaultListModel());
		
		
		for (BPMNAnalysisData bpmnAnalysisData : BPMNAnalysisDataList) {
			((DefaultListModel) component.getModel()).addElement(new Pair<Integer, String>(getBPMNAnalysisDataList().indexOf(
					bpmnAnalysisData), bpmnAnalysisData.getDescription()));
		}
		// initSwimlanePanel();
		initConfPerfAnalysisPanel();

		HandOverOfWorkSNPanel = OrganizationalInteractionPanel.instance();
		WorkingTogetherSNPanel = OrganizationalInteractionPanel.instance();
		DoingSimilarTasksSNPanel = OrganizationalInteractionPanel.instance();
		activitybasedUAPanel = OrganizationalInteractionPanel.instance();

		activitybasedUAPanel.createPanel(context, graphPanel, "Users & Tasks Table", getSelectedBPMNAnalysisData(),
				OrganizationalInteractionPanel.RIGHT, 1, 600, 350, OrganizationalInteractionPanel.A_OTM);
		HandOverOfWorkSNPanel.createPanel(context, graphPanel, "Handover of work", getSelectedBPMNAnalysisData(),
				OrganizationalInteractionPanel.BOTTOM, 1, 400, 500, OrganizationalInteractionPanel.HoWSN);
		DoingSimilarTasksSNPanel.createPanel(context, graphPanel, "Doing Similar Tasks", getSelectedBPMNAnalysisData(),
				OrganizationalInteractionPanel.BOTTOM, 2, 400, 500, OrganizationalInteractionPanel.STSN);
		WorkingTogetherSNPanel.createPanel(context, graphPanel, "Working together", getSelectedBPMNAnalysisData(),
				OrganizationalInteractionPanel.BOTTOM, 3, 400, 500, OrganizationalInteractionPanel.WTSN);
	}

	*//**
	 * Performance Conformance Information Panel
	 *//*
	private void initConfPerfAnalysisPanel() {
		ConformanceInfo confResult = getSelectedBPMNAnalysisData().getConformanceInfo();
		FlexPerfRepInfo perfResult = getSelectedBPMNAnalysisData().getPerfRepInfo();
		CaseInstance caseInstance = perfResult.getCaseInstance();

		perfPanel.setLayout(new GridLayout(0, 1));
		perfPanel.setSize(300, 135);
		perfPanel.setLocation(10, 10);
		perfPanel.setBounds(10, 10, 300, 135);
		JLabel perfLabel = new JLabel("Performance Related Information");
		perfLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 14));
		perfLabel.setForeground(Color.WHITE);
		perfLabel.setHorizontalAlignment(JLabel.LEFT);

		frequencyJLabel = new JLabel("Num. of cases : " + caseInstance.getFrequency());
		arrivalRateJLabel = new JLabel("Arrival rate : " + getSelectedBPMNAnalysisData().getArrivalRate());
		avgThroughputJLabel = new JLabel("Avg. throughput time: "
				+ PerformanceUtils.getSuitableTimeStr(caseInstance.getAvgTimeSpan()));
		minThroughputJLabel = new JLabel("Min. throughput time : "
				+ PerformanceUtils.getSuitableTimeStr(caseInstance.getMinTimeSpan()));
		maxThroughputJLabel = new JLabel("Max. throughput time : "
				+ PerformanceUtils.getSuitableTimeStr(caseInstance.getMaxTimeSpan()));
		stdDevThroughputJLabel = new JLabel("Std. deviation throughput time : "
				+ PerformanceUtils.getSuitableTimeStr(caseInstance.getStdDevTimeSpan()));

		frequencyJLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));
		arrivalRateJLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));
		avgThroughputJLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));
		minThroughputJLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));
		maxThroughputJLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));
		stdDevThroughputJLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));

		frequencyJLabel.setBorder(EMPTYBORDER);
		arrivalRateJLabel.setBorder(EMPTYBORDER);
		avgThroughputJLabel.setBorder(EMPTYBORDER);
		minThroughputJLabel.setBorder(EMPTYBORDER);
		maxThroughputJLabel.setBorder(EMPTYBORDER);
		stdDevThroughputJLabel.setBorder(EMPTYBORDER);

		frequencyJLabel.setOpaque(false);
		arrivalRateJLabel.setOpaque(false);
		avgThroughputJLabel.setOpaque(false);
		minThroughputJLabel.setOpaque(false);
		maxThroughputJLabel.setOpaque(false);
		stdDevThroughputJLabel.setOpaque(false);

		SlickerDecorator.instance().decorate(frequencyJLabel);
		SlickerDecorator.instance().decorate(arrivalRateJLabel);
		SlickerDecorator.instance().decorate(avgThroughputJLabel);
		SlickerDecorator.instance().decorate(minThroughputJLabel);
		SlickerDecorator.instance().decorate(maxThroughputJLabel);
		SlickerDecorator.instance().decorate(stdDevThroughputJLabel);

		perfPanel.add(perfLabel);
		perfPanel.add(frequencyJLabel);
		perfPanel.add(arrivalRateJLabel);
		perfPanel.add(avgThroughputJLabel);
		perfPanel.add(minThroughputJLabel);
		perfPanel.add(maxThroughputJLabel);
		perfPanel.add(stdDevThroughputJLabel);

		if (confResult!=null) {
			confPanel.setLayout(new GridLayout(0, 1));
			confPanel.setSize(300, 80);
			confPanel.setLocation(10, 150);
			confPanel.setBounds(10, 150, 300, 80);
			JLabel confLabel = new JLabel("Conformance Related Information");
			confLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 14));
			confLabel.setForeground(Color.WHITE);
			confLabel.setHorizontalAlignment(JLabel.LEFT);
			confPanel.add(confLabel);
			fitnessLabel = new JLabel("Avg. Fitness (out of all cases) : %" + nf.format(confResult.getFitness() * 100));
			fitnessLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));
			fitnessLabel.setBorder(EMPTYBORDER);
			SlickerDecorator.instance().decorate(fitnessLabel);
			confPanel.add(fitnessLabel);
			if (confResult.getMinFitness() != 0) {
				minFitnessJLabel = new JLabel("Min. Fitness (out of all paths) : %"
						+ nf.format(confResult.getMinFitness()));
				maxFitnessJLabel = new JLabel("Max. Fitness (out of all paths) : %"
						+ nf.format(confResult.getMaxFitness()));

				minFitnessJLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));
				maxFitnessJLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));

				minFitnessJLabel.setBorder(EMPTYBORDER);
				maxFitnessJLabel.setBorder(EMPTYBORDER);

				minFitnessJLabel.setOpaque(false);
				maxFitnessJLabel.setOpaque(false);

				SlickerDecorator.instance().decorate(minFitnessJLabel);
				SlickerDecorator.instance().decorate(maxFitnessJLabel);

				confPanel.add(minFitnessJLabel);
				confPanel.add(maxFitnessJLabel);
			}
		}
		confPerfAnalysisPanel = new PerfConfInteractionPanel(getGraphPanel(), "Performance/Conformance", confPanel,
				perfPanel, 320, 230, PerfConfInteractionPanel.RIGHT_UP);

	}

	*//**
	 * 
	 *//*
	@SuppressWarnings("unused")
	private void initSwimlanePanel() {
		JPanel sliderPanel = new JPanel();
		sliderPanel.setLayout(null);
		sliderPanel.setSize(150, 365);
		sliderPanel.setLocation(0, 0);
		JLabel sliderInfo = new JLabel("Number of Executions by Users Threshold");
		sliderInfo.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));
		sliderInfo.setForeground(Color.DARK_GRAY);
		sliderInfo.setUI(new VerticalLabelUI(false));
		sliderInfo.setHorizontalTextPosition(SwingConstants.CENTER);
		sliderInfo.setVerticalTextPosition(SwingConstants.CENTER);

		swimlaneSlider = SlickerFactory.instance().createSlider(1);

		swimlaneSlider.setMinimum(0);
		swimlaneSlider.setValue(0);
		swimlaneSlider.setMaximum(100);

		swimlaneSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				updateBPMNModel(true);
			}
		});

		sliderMinValue = SlickerFactory.instance().createLabel("%0");
		sliderMaxValue = SlickerFactory.instance().createLabel("%100");

		sliderValue = SlickerFactory.instance().createLabel("%0");

		sliderMinValue.setHorizontalAlignment(SwingConstants.CENTER);
		sliderMaxValue.setHorizontalAlignment(SwingConstants.CENTER);
		sliderValue.setHorizontalAlignment(SwingConstants.LEFT);

		sliderMinValue.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));
		sliderMaxValue.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));
		sliderValue.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));

		sliderMinValue.setForeground(Color.DARK_GRAY);
		sliderMaxValue.setForeground(Color.DARK_GRAY);
		sliderValue.setForeground(Color.DARK_GRAY);

		int height = 365;
		int sliderHeight = height - 65;

		sliderInfo.setBounds(20, 45, 20, sliderHeight - 60);
		this.swimlaneSlider.setBounds(75, 25, 20, sliderHeight);
		this.sliderMaxValue.setBounds(65, 10, 60, 20);
		this.sliderMinValue.setBounds(65, height - 40, 40, 20);

		int span = this.swimlaneSlider.getMaximum() - this.swimlaneSlider.getMinimum();
		int position = 33 + (int) ((float) (this.swimlaneSlider.getMaximum() - 50) / (float) span * (sliderHeight - 28));
		this.sliderValue.setBounds(95, position, 60, 20);

		sliderPanel.setBackground(Color.LIGHT_GRAY);
		sliderPanel.add(sliderInfo);
		sliderPanel.add(swimlaneSlider);
		sliderPanel.add(sliderMinValue);
		sliderPanel.add(sliderMaxValue);
		sliderPanel.add(sliderValue);

		JPanel applyPanel = new JPanel();
		applyPanel.setLocation(0, 365);
		applyPanel.setLayout(null);
		applyPanel.setSize(150, 35);
		JButton applyButton = new JButton("Apply Swimlane");
		applyButton.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));
		applyButton.setForeground(Color.DARK_GRAY);
		applyButton.setBackground(new Color(238, 238, 238));
		applyButton.setHorizontalAlignment(SwingConstants.CENTER);
		applyButton.setVerticalAlignment(SwingConstants.CENTER);
		applyButton.setSize(130, 20);
		applyButton.setLocation(5, 5);
		applyButton.setBounds(5, 5, 130, 20);
		applyButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				updateBPMNModel(false);

			}
		});
		applyPanel.add(applyButton);

		confPerfAnalysisPanel = new PerfConfInteractionPanel(getGraphPanel(), "Swimlane", sliderPanel, applyPanel, 150,
				400, PerfConfInteractionPanel.LEFT_BOTTOM);

	}

	public void updateBPMNModel(boolean b) {
		@SuppressWarnings("unused")
		Map<String, Map<String, Integer>> userActivityPercentageMap = getSelectedBPMNAnalysisData()
				.getUserActivityPercentageMap();

		if (b) {
			int value = this.swimlaneSlider.getValue();
			this.sliderValue.setText("%" + value);

		} else {
			// removeSwimlanes(bpmn);

			Swimlane s = bpmn.addSwimlane("Test", null);
			for (BPMNNode bpmnNode : bpmn.getNodes()) {
				if (!s.equals(bpmnNode)) {
					bpmnNode.setParentSwimlane(s);
				} else {
					System.out.println("");
				}
			}

			
			 * graphPanel =
			 * ProMJGraphVisualizer.instance().visualizeGraph(context, bpmn);
			 * graph = graphPanel.getGraph(); graphPanel.getGraph().setModel(new
			 * ProMGraphModel(bpmn)); graph.update(bpmn.getNodes());
			 * graph.repaint();
			 * 
			 * 
			 * JComponent c = graph.getComponent(); Component[] c2 =
			 * c.getComponents(); ProMJGraphPanel proMJGraphPanel =
			 * ProMJGraphVisualizer.instance().visualizeGraph(context, bpmn);
			 * graphPanel.getGraph().removeAll();
			 * graphPanel.getGraph().add(proMJGraphPanel.getGraph());
			 * graph.repositionToOrigin(); graphPanel.repaint();
			 * graph.update(bpmn.getNodes()); graph.update(bpmn.getEdges());
			 

		}

	}

	private void fillFlexBPMNNodeEdgeMappings(BPMNDiagram bpmn, Flex flex) {

		for (Activity activity : bpmn.getActivities()) {
			for (FlexNode flexNode : flex.getNodes()) {
				String flexLabel = flexNode.getLabel();
				int index = flexLabel.indexOf("+");
				if (index != -1) {
					flexLabel = flexLabel.substring(0, index);

				}
				if (flexLabel.equals(activity.getLabel())) {
					flexBPMNNodeMapping.put(flexNode, activity);
					bpmnFlexNodeMapping.put(activity, flexNode);
					break;
				}
			}
		}

		for (FlexEdge<? extends FlexNode, ? extends FlexNode> flexEdge : flex.getEdges()) {
			for (Flow flow : bpmn.getFlows()) {
				FlexNode tempTargetFlexNode = flexEdge.getTarget();
				FlexNode tempSourceFlexNode = flexEdge.getSource();
				BPMNNode tempTargetBPMNNode = flexBPMNNodeMapping.get(tempTargetFlexNode);
				BPMNNode tempSourceBPMNNode = flexBPMNNodeMapping.get(tempSourceFlexNode);
				BPMNNode tempTargetBPMNNode2 = flow.getTarget();
				BPMNNode tempSourceBPMNNode2 = flow.getSource();
				if (tempTargetBPMNNode.equals(tempTargetBPMNNode2) && tempSourceBPMNNode.equals(tempSourceBPMNNode2)) {
					flexBPMNEdgeMapping.put(flexEdge, flow);
					bpmnFlexEdgeMapping.put(flow, flexEdge);
					break;
				}
			}
		}

	}

	public void updated() {
		// ignore (as in ListSelectionPanel)
	}

	public String getPanelName() {
		return name;
	}

	public JComponent getComponent() {
		return this;
	}

	public void setScalableComponent(ScalableComponent scalable) {
		if (scalable instanceof ProMJGraph) {
			this.graph = (ProMJGraph) scalable;
			this.viewSpecificMap = graph.getViewSpecificAttributes();

			originalSpecificMap = new ViewSpecificAttributeMap();
			for (AttributeMapOwner o : viewSpecificMap.keySet()) {
				for (String key : viewSpecificMap.keySet(o)) {
					originalSpecificMap.putViewSpecific(o, key, viewSpecificMap.get(o, key));
				}
			}
		} else {
			throw new IllegalArgumentException("Scalable needs to be instance of ProMJGraph");
		}
	}

	public void setParent(ScalableViewPanel viewPanel) {
		// ignore (as in ListSelectionPanel)
	}

	public double getHeightInView() {
		return 140;
	}

	public double getWidthInView() {
		return 250;
	}

	public void willChangeVisibility(boolean to) {
		// do nothing if selection becomes empty
	}

	public void colorElements() {
		// DECOR ACTIVITIES
		Map<Activity, ActivityDecoratorData> activityDecorationMap = decorActivities();

		// DECOR FLOWS
		decorFlows(activityDecorationMap);

		// DECOR EVENTS
		Map<Event, EventDecoratorData> eventDecorationMap = decorEvents(activityDecorationMap);

		// DECOR GATEWAYS
		decorGateways(activityDecorationMap, eventDecorationMap);

		// refresh the figure
		graph.update(bpmn.getNodes());
		graph.update(bpmn.getFlows());
	}

	private Map<Activity, ActivityDecoratorData> decorActivities() {

		FlexPerfRepInfo perfResult = getSelectedBPMNAnalysisData().getPerfRepInfo();

		// calculate performance values
		Map<Activity, Double> workingTime = new HashMap<Activity, Double>(); // also store temporary frequency
		Map<Activity, Double> waitingTime = new HashMap<Activity, Double>();
		Map<Activity, Double> syncTime = new HashMap<Activity, Double>();

		Map<Activity, Double> sumWorkingTime = new HashMap<Activity, Double>();
		Map<Activity, Double> sumWaitingTime = new HashMap<Activity, Double>();
		Map<Activity, Double> sumSyncTime = new HashMap<Activity, Double>();

		Map<Activity, ActivityDecoratorData> activityDecorationMap = new HashMap<Activity, ActivityDecoratorData>();

		// abuse flexBindingInstance to store node performance info

		Map<Activity, Long> minWaitingTime = new HashMap<Activity, Long>();
		Map<Activity, Long> maxWaitingTime = new HashMap<Activity, Long>();
		Map<Activity, Long> minWorkingTime = new HashMap<Activity, Long>();
		Map<Activity, Long> maxWorkingTime = new HashMap<Activity, Long>();
		Map<Activity, Long> minSyncTime = new HashMap<Activity, Long>();
		Map<Activity, Long> maxSyncTime = new HashMap<Activity, Long>();

		Map<Short, Set<FlexBindingInstance>> nodeInstancesMap = perfResult.getNodeInstancesMap();
		for (Short encodedFlexNode : nodeInstancesMap.keySet()) {
			for (FlexBindingInstance bindingInstance : nodeInstancesMap.get(encodedFlexNode)) {
				FlexBinding binding = codec.getFlexBindingFor(bindingInstance.getEncodedBinding());
				short bindingEncodedNode = binding.getEncodedNode();
				FlexNode flexNode = codec.decode(bindingEncodedNode);
				Activity activityNode = (Activity) flexBPMNNodeMapping.get(flexNode);

				ActivityDecoratorData decor = activityDecorationMap.get(activityNode);
				if (decor == null) {
					decor = new ActivityDecoratorData();
					activityDecorationMap.put(activityNode, decor);
				}
				decor.setFrequency(decor.getFrequency() + bindingInstance.getFrequency());
				updateDecorSemantic(flexNode, decor, binding, decor.getFrequency());

				decor.setCancelledFrequency(perfResult.getCancellationFrequency(bindingEncodedNode));

				if (sumWorkingTime.get(activityNode) != null) {
					// working time
					sumWorkingTime.put(
							activityNode,
							sumWorkingTime.get(activityNode)
									+ (bindingInstance.getAvgWorkTime() * bindingInstance.getFrequency()));
					workingTime.put(activityNode, bindingInstance.getFrequency() + workingTime.get(activityNode));

					if (Double.compare(minWorkingTime.get(activityNode), bindingInstance.getMinWorkTime()) > 0) {
						minWorkingTime.put(activityNode, bindingInstance.getMinWorkTime());
					}
					if (Double.compare(maxWorkingTime.get(activityNode), bindingInstance.getMaxWorkTime()) < 0) {
						maxWorkingTime.put(activityNode, bindingInstance.getMaxWorkTime());
					}

				} else {
					// working time for the first time
					sumWorkingTime.put(activityNode, bindingInstance.getAvgWorkTime() * bindingInstance.getFrequency());
					workingTime.put(activityNode, (double) bindingInstance.getFrequency());
					minWorkingTime.put(activityNode, bindingInstance.getMinWorkTime());
					maxWorkingTime.put(activityNode, bindingInstance.getMaxWorkTime());

				}

				if (sumWaitingTime.get(activityNode) != null) {
					// waiting time
					sumWaitingTime.put(
							activityNode,
							sumWaitingTime.get(activityNode)
									+ (bindingInstance.getAvgWaitTime() * bindingInstance.getFrequency()));
					waitingTime.put(activityNode, bindingInstance.getFrequency() + waitingTime.get(activityNode));
					if (Double.compare(minWaitingTime.get(activityNode), bindingInstance.getMinWaitTime()) > 0) {
						minWaitingTime.put(activityNode, bindingInstance.getMinWaitTime());
					}
					if (Double.compare(maxWaitingTime.get(activityNode), bindingInstance.getMaxWaitTime()) < 0) {
						maxWaitingTime.put(activityNode, bindingInstance.getMaxWaitTime());
					}
				} else {
					// waiting time for the first time
					sumWaitingTime.put(activityNode, bindingInstance.getAvgWaitTime() * bindingInstance.getFrequency());
					waitingTime.put(activityNode, (double) bindingInstance.getFrequency());
					minWaitingTime.put(activityNode, bindingInstance.getMinWaitTime());
					maxWaitingTime.put(activityNode, bindingInstance.getMaxWaitTime());
				}

				if (sumSyncTime.get(activityNode) != null) {
					// sync time
					sumSyncTime.put(activityNode, sumSyncTime.get(activityNode)
							+ (bindingInstance.getAvgSyncTime() * bindingInstance.getFrequency()));
					syncTime.put(activityNode, bindingInstance.getFrequency() + syncTime.get(activityNode));
					if (Double.compare(minSyncTime.get(activityNode), bindingInstance.getMinSyncTime()) > 0) {
						minSyncTime.put(activityNode, bindingInstance.getMinSyncTime());
					}
					if (Double.compare(maxSyncTime.get(activityNode), bindingInstance.getMaxSyncTime()) < 0) {
						maxSyncTime.put(activityNode, bindingInstance.getMaxSyncTime());
					}

				} else {
					// sync time for the first time
					sumSyncTime.put(activityNode, bindingInstance.getAvgSyncTime() * bindingInstance.getFrequency());
					syncTime.put(activityNode, (double) bindingInstance.getFrequency());
					minSyncTime.put(activityNode, bindingInstance.getMinSyncTime());
					maxSyncTime.put(activityNode, bindingInstance.getMaxSyncTime());
				}

			}
		}

		Set<FlexNode> setAllNodes = codec.getFlexNodes();

		Set<Double> sumNodeAvgWorkingTime = new HashSet<Double>();
		Set<Double> sumNodeAvgWaitingTime = new HashSet<Double>();
		Set<Double> sumNodeAvgTotalTime = new HashSet<Double>();
		Set<Double> sumNodeAvgSyncTime = new HashSet<Double>();

		int maxNodeFreq = Integer.MIN_VALUE;
		int minNodeFreq = Integer.MAX_VALUE;
		for (FlexNode flexNode : setAllNodes) {
			Activity activityNode = (Activity) flexBPMNNodeMapping.get(flexNode);

			if (workingTime.get(activityNode) != null) {
				double freq = workingTime.get(activityNode);
				double avg = sumWorkingTime.get(activityNode) / freq;

				if (maxNodeFreq < activityDecorationMap.get(activityNode).getFrequency()) {
					maxNodeFreq = activityDecorationMap.get(activityNode).getFrequency();
				}
				if (minNodeFreq > activityDecorationMap.get(activityNode).getFrequency()) {
					minNodeFreq = activityDecorationMap.get(activityNode).getFrequency();
				}

				workingTime.put(activityNode, avg);
				sumNodeAvgWorkingTime.add(avg);
			} else {
				workingTime.put(activityNode, Double.NaN);
			}

			if (waitingTime.get(activityNode) != null) {
				double avg = sumWaitingTime.get(activityNode) / waitingTime.get(activityNode);
				waitingTime.put(activityNode, avg);
				sumNodeAvgWaitingTime.add(avg);
			} else {
				waitingTime.put(activityNode, Double.NaN);
			}

			if (workingTime.get(activityNode) != Double.NaN && waitingTime.get(activityNode) != Double.NaN) {
				sumNodeAvgTotalTime.add(waitingTime.get(activityNode) + workingTime.get(activityNode));
			}

			if (syncTime.get(activityNode) != null) {
				double avg = sumSyncTime.get(activityNode) / syncTime.get(activityNode);
				syncTime.put(activityNode, avg);
				sumNodeAvgSyncTime.add(avg);
			} else {
				syncTime.put(activityNode, Double.NaN);
			}
		}

		// prepare color for sync, etc
		// bound for working time
		sumNodeAvgWorkingTime.remove(Double.NaN);
		double lowBoundAvgWorking = Double.POSITIVE_INFINITY;
		double upBoundAvgWorking = Double.POSITIVE_INFINITY;
		if (sumNodeAvgWorkingTime.size() > 1) {
			double[] arrSumNodeAvgWorking = getSortedArrayFromSet(sumNodeAvgWorkingTime);
			if (arrSumNodeAvgWorking.length >= 3) {
				int indexLow = arrSumNodeAvgWorking.length / 3;
				lowBoundAvgWorking = arrSumNodeAvgWorking[indexLow - 1];
				upBoundAvgWorking = arrSumNodeAvgWorking[(indexLow * 2) - 1];
			} else {
				lowBoundAvgWorking = arrSumNodeAvgWorking[0];
			}
			for (int i = 0; i < arrSumNodeAvgWorking.length; i++) {
				System.out.println(i + ".:" + (int) arrSumNodeAvgWorking[i]);
			}
			System.out.println(lowBoundAvgWorking + ".:" + (int) lowBoundAvgWorking);
			System.out.println(upBoundAvgWorking + ".:" + (int) upBoundAvgWorking);
		}

		// bound for waiting time
		sumNodeAvgWaitingTime.remove(Double.NaN);
		double lowBoundAvgWaiting = Double.POSITIVE_INFINITY;
		double upBoundAvgWaiting = Double.POSITIVE_INFINITY;
		if (sumNodeAvgWaitingTime.size() > 1) {
			double[] arrSumNodeAvgWaiting = getSortedArrayFromSet(sumNodeAvgWaitingTime);
			if (arrSumNodeAvgWaiting.length >= 3) {
				int indexLow = arrSumNodeAvgWaiting.length / 3;
				lowBoundAvgWaiting = arrSumNodeAvgWaiting[indexLow - 1];
				upBoundAvgWaiting = arrSumNodeAvgWaiting[(indexLow * 2) - 1];
			} else {
				lowBoundAvgWaiting = arrSumNodeAvgWaiting[0];
			}
			for (int i = 0; i < arrSumNodeAvgWaiting.length; i++) {
				System.out.println(i + ".:" + (int) arrSumNodeAvgWaiting[i]);
			}
			System.out.println(lowBoundAvgWaiting + ".:" + (int) lowBoundAvgWaiting);
			System.out.println(upBoundAvgWaiting + ".:" + (int) upBoundAvgWaiting);

		}
		// bound for sync time
		sumNodeAvgWorkingTime.remove(Double.NaN);
		double lowBoundAvgSync = Double.POSITIVE_INFINITY;
		double upBoundAvgSync = Double.POSITIVE_INFINITY;
		if (sumNodeAvgSyncTime.size() > 1) {
			double[] arrSumNodeAvgSync = getSortedArrayFromSet(sumNodeAvgSyncTime);
			if (arrSumNodeAvgSync.length >= 3) {
				int indexLow = arrSumNodeAvgSync.length / 3;
				lowBoundAvgSync = arrSumNodeAvgSync[indexLow - 1];
				upBoundAvgSync = arrSumNodeAvgSync[(indexLow * 2) - 1];
			} else {
				lowBoundAvgSync = arrSumNodeAvgSync[0];
			}
			for (int i = 0; i < arrSumNodeAvgSync.length; i++) {
				System.out.println(i + ".:" + (int) arrSumNodeAvgSync[i]);
			}
			System.out.println(lowBoundAvgSync + ".:" + (int) lowBoundAvgSync);
			System.out.println(upBoundAvgSync + ".:" + (int) upBoundAvgSync);

		}
		// prepare color for total, etc
		// bound for total time

		double lowBoundAvgTotal = Double.POSITIVE_INFINITY;
		double upBoundAvgTotal = Double.POSITIVE_INFINITY;
		if (sumNodeAvgWorkingTime.size() > 1) {
			double[] arrSumNodeAvgTotal = getSortedArrayFromSet(sumNodeAvgTotalTime);
			if (arrSumNodeAvgTotal.length >= 3) {
				int indexLow = arrSumNodeAvgTotal.length / 3;
				lowBoundAvgTotal = arrSumNodeAvgTotal[indexLow - 1];
				upBoundAvgTotal = arrSumNodeAvgTotal[(indexLow * 2) - 1];
			} else {
				lowBoundAvgTotal = arrSumNodeAvgTotal[0];
			}
			for (int i = 0; i < arrSumNodeAvgTotal.length; i++) {
				System.out.println(i + ".:" + (int) arrSumNodeAvgTotal[i]);
			}
			System.out.println(lowBoundAvgTotal + ".:" + (int) lowBoundAvgTotal);
			System.out.println(upBoundAvgTotal + ".:" + (int) upBoundAvgTotal);

		}
		// decorate activity nodes
		Map<Short, Integer> caseInvolvement = perfResult.getNodeCaseInvolvement();
		CaseInstance caseInstance = perfResult.getCaseInstance();
		Map<Short, Integer> unfinishedFrequency = perfResult.getUnfinishedFrequency();

		double minMaxNodeDifference = upBoundAvgTotal - lowBoundAvgTotal; // for calculation of height
		double widthRatio = 0;
		if (minMaxNodeDifference != 0) {
			widthRatio = MAXNODEADDITIONALWIDTH / minMaxNodeDifference;
		}
		for (Activity node : bpmn.getActivities()) {

			viewSpecificMap.putViewSpecific(node, AttributeMap.FILLCOLOR, Color.LIGHT_GRAY);
			viewSpecificMap.putViewSpecific(node, AttributeMap.SHOWLABEL, false);

			ActivityDecoratorData nodeDecor = activityDecorationMap.get(node);

			short encodedNode = codec.encode(bpmnFlexNodeMapping.get(node));
			if (nodeDecor != null) {
				nodeDecor.setCaseInvolvementRatio((double) caseInvolvement.get(encodedNode)
						/ (double) caseInstance.getFrequency());
				nodeDecor.setWorkingColor(getColor(workingTime.get(node), lowBoundAvgWorking, upBoundAvgWorking));
				nodeDecor.setWaitColor(getColor(waitingTime.get(node), lowBoundAvgWaiting, upBoundAvgWaiting));
				nodeDecor.setTotalColor(getColor(workingTime.get(node) + waitingTime.get(node), lowBoundAvgTotal,
						upBoundAvgTotal));
				nodeDecor.setSyncColor(getColor(syncTime.get(node), lowBoundAvgSync, upBoundAvgSync));
				nodeDecor.setAvgWorkingTime(workingTime.get(node));
				nodeDecor.setAvgWaitingTime(waitingTime.get(node));
				nodeDecor.setAvgTotalTime(workingTime.get(node) + waitingTime.get(node));
				nodeDecor.setAvgSynchTime(syncTime.get(node));
				nodeDecor.setName(node.getLabel());

				if (unfinishedFrequency.get(encodedNode) != null) {
					nodeDecor.setUnfinishedActivityFrequency(unfinishedFrequency.get(encodedNode));
				}
				nodeDecor.setCancelledFrequency(perfResult.getCancellationFrequency(encodedNode));
				node.setDecorator(new ActivityDecorator(nodeDecor));

				double totalTimeDifference = nodeDecor.getAvgTotalTime() - lowBoundAvgTotal;
				double nodeTimeDifference = (totalTimeDifference - lowBoundAvgTotal) > minMaxNodeDifference ? minMaxNodeDifference
						: (totalTimeDifference - lowBoundAvgTotal);

				viewSpecificMap.removeViewSpecific(node, AttributeMap.SIZE);
				int additionalWidth = (int) (nodeTimeDifference * widthRatio);
				additionalWidth = (additionalWidth > 0 ? additionalWidth : 0);
				viewSpecificMap.putViewSpecific(node, AttributeMap.SIZE, new Dimension(NODEWIDTH + additionalWidth,
						NODEHEIGHT));
				StringBuffer additionalText = new StringBuffer("<html><table border=0>" + "<tr><td colspan=6><b>"
						+ node.getLabel() + "</b><br/></td></tr>" + "<tr>" + "<td colspan=6><b>Waiting time</b></td>"
						+ "</tr>" + "<tr>" + "<td>Avg.</td>" + "<td>"
						+ PerformanceUtils.getSuitableTimeStr(waitingTime.get(node)) + "</td>" + "<td>Min.</td>"
						+ "<td>" + PerformanceUtils.getSuitableTimeStr(minWaitingTime.get(node)) + " </td>"
						+ "<td>Max.</td>" + "<td>" + PerformanceUtils.getSuitableTimeStr(maxWaitingTime.get(node))
						+ "</td>" + "</tr>");
				additionalText.append("<tr>" + "<td colspan=6><b>Working time</b></td></tr>" + "<tr>" + "<td>Avg.</td>"
						+ "<td>" + PerformanceUtils.getSuitableTimeStr(workingTime.get(node)) + "</td>"
						+ "<td>Min.</td>" + "<td>" + PerformanceUtils.getSuitableTimeStr(minWorkingTime.get(node))
						+ "</td>" + "<td>Max.</td>" + "<td>"
						+ PerformanceUtils.getSuitableTimeStr(maxWorkingTime.get(node)) + "</td>" + "</tr>");
				additionalText.append("<tr>" + "<td colspan=6><b>Total time</b></td></tr>" + "<tr>" + "<td>Avg.</td>"
						+ "<td>" + PerformanceUtils.getSuitableTimeStr(waitingTime.get(node) + workingTime.get(node))
						+ "</td>" + "<td>Min.</td>" + "<td>"
						+ PerformanceUtils.getSuitableTimeStr(minWaitingTime.get(node) + minWorkingTime.get(node))
						+ "</td>" + "<td>Max.</td>" + "<td>"
						+ PerformanceUtils.getSuitableTimeStr(maxWaitingTime.get(node) + maxWorkingTime.get(node))
						+ "</td>" + "</tr>");

				if (Double.compare(Double.NaN, nodeDecor.getAvgSynchTime()) != 0
						&& Double.compare(0.0, nodeDecor.getAvgSynchTime()) < 0) {
					additionalText.append("<tr><td colspan=6><b>Sync. time</b></td></tr>" + "<tr>	<td>Avg.</td>"
							+ "<td>" + PerformanceUtils.getSuitableTimeStr(syncTime.get(node)) + "</td>"
							+ "<td>Min.</td>" + "<td>" + PerformanceUtils.getSuitableTimeStr(minSyncTime.get(node))
							+ "</td>" + "<td>Max.</td>" + "<td>"
							+ PerformanceUtils.getSuitableTimeStr(maxSyncTime.get(node)) + "</td>" + "</tr>");

				}

				viewSpecificMap.putViewSpecific(node, AttributeMap.TOOLTIP, additionalText.append("</table></html>")
						.toString());
			} else {
				nodeDecor = new ActivityDecoratorData();
				nodeDecor.setName(node.getLabel());
				if (unfinishedFrequency.get(encodedNode) != null) {
					nodeDecor.setUnfinishedActivityFrequency(unfinishedFrequency.get(encodedNode));
				}
				nodeDecor.setCancelledFrequency(perfResult.getCancellationFrequency(encodedNode));
				node.setDecorator(new ActivityDecorator(nodeDecor));

				viewSpecificMap.removeViewSpecific(node, AttributeMap.SIZE);
				Dimension d = (Dimension) viewSpecificMap.get(node, AttributeMap.SIZE);
				viewSpecificMap.putViewSpecific(node, AttributeMap.SIZE, d);
				// viewSpecificMap.removeViewSpecific(node, AttributeMap.SIZE);
				// viewSpecificMap.putViewSpecific(node, AttributeMap.SIZE, new Dimension(UNVISITEDNODEWIDTH, UNVISITEDHEIGHT));

			}

		}
		return activityDecorationMap;
	}

	private void decorFlows(Map<Activity, ActivityDecoratorData> activityDecorationMap) {
		// find bound for edges (including frequency)
		int minEdgeFreq = Integer.MAX_VALUE;
		int maxEdgeFreq = Integer.MIN_VALUE;
		Map<Pair<Short, Short>, EdgeInstance> instanceMap = getSelectedBPMNAnalysisData().getPerfRepInfo()
				.getEdgeInstancesMap();
		Set<Double> sumEdgeAvgMoveTime = new HashSet<Double>();
		for (Pair<Short, Short> pair : instanceMap.keySet()) {
			EdgeInstance edgeInstance = instanceMap.get(pair);
			if (minEdgeFreq > edgeInstance.getFrequency()) {
				minEdgeFreq = edgeInstance.getFrequency();
			}
			if (maxEdgeFreq < edgeInstance.getFrequency()) {
				maxEdgeFreq = edgeInstance.getFrequency();
			}
			sumEdgeAvgMoveTime.add(edgeInstance.getAvgMoveTime());
		}

		int minMaxEdgeDifference = maxEdgeFreq - minEdgeFreq;

		for (FlexNode flexNode : flex.getNodes()) {
			List<FlexNode> inNodes = PerformanceUtils.getFlexNodeList(flex, flexNode, false);
			List<FlexNode> outNodes = PerformanceUtils.getFlexNodeList(flex, flexNode, true);

			if (outNodes.size() == 0) {
				// end event
				// get correspondant bpmn node
				BPMNNode bpmnNode = flexBPMNNodeMapping.get(flexNode);

				// get edge to the gateway
				BPMNEdge<? extends BPMNNode, ? extends BPMNNode> outEdge = bpmn.getOutEdges(bpmnNode).iterator().next();

				// get correspondent BPMN flow and paint the flow arrow
				Flow flow = (Flow) outEdge;
				ActivityDecoratorData activityDecoratorData = activityDecorationMap.get(bpmnNode);
				int frequency = 0;
				if (activityDecoratorData != null) {
					frequency = activityDecorationMap.get(bpmnNode).getFrequency();
				}
				paintArrows(minEdgeFreq, minMaxEdgeDifference, flow, frequency);

			}
			if (outNodes.size() == 1) {
				FlexNode outNode = outNodes.get(0);

				// get frequency
				Pair<Short, Short> shortRepresentation = new Pair<Short, Short>(codec.encode(flexNode),
						codec.encode(outNode));
				EdgeInstance edgeInstance = instanceMap.get(shortRepresentation);
				int frequency = 0;
				if (edgeInstance != null) {
					frequency = edgeInstance.getFrequency();
				}
				// get correspondent BPMN flow
				FlexEdge<? extends FlexNode, ? extends FlexNode> flexEdge = flex.getOutEdges(flexNode).iterator()
						.next();
				Flow flow = (Flow) flexBPMNEdgeMapping.get(flexEdge);

				paintArrows(minEdgeFreq, minMaxEdgeDifference, flow, frequency);
			}
			if (outNodes.size() > 1) {
				// get correspondant bpmn node
				BPMNNode bpmnNode = flexBPMNNodeMapping.get(flexNode);

				// get edge to the gateway
				BPMNEdge<? extends BPMNNode, ? extends BPMNNode> outEdge = bpmn.getOutEdges(bpmnNode).iterator().next();

				// get correspondent BPMN flow and paint the flow arrow
				Flow flow = (Flow) outEdge;
				ActivityDecoratorData activityDecoratorData = activityDecorationMap.get(bpmnNode);
				int frequency = 0;
				if (activityDecoratorData != null) {
					frequency = activityDecoratorData.getFrequency();
				}
				paintArrows(minEdgeFreq, minMaxEdgeDifference, flow, frequency);

				BPMNNode bpmnNodeGateway = outEdge.getTarget();

				for (BPMNEdge<? extends BPMNNode, ? extends BPMNNode> bpmnNodeGatewayEdge : bpmn
						.getOutEdges(bpmnNodeGateway)) {
					BPMNNode bpmnNode2 = bpmnNodeGatewayEdge.getTarget();
					if (bpmnNode2 instanceof Activity) {
						FlexNode flexNode2 = bpmnFlexNodeMapping.get(bpmnNode2);

						// get frequency
						Pair<Short, Short> shortRepresentation = new Pair<Short, Short>(codec.encode(flexNode),
								codec.encode(flexNode2));
						EdgeInstance edgeInstance = instanceMap.get(shortRepresentation);
						frequency = 0;
						if (edgeInstance != null) {
							frequency = edgeInstance.getFrequency();
						}
						// get correspondent BPMN flow
						Flow flow2 = (Flow) bpmnNodeGatewayEdge;
						paintArrows(minEdgeFreq, minMaxEdgeDifference, flow2, frequency);

						FlowDecoratorData data = new FlowDecoratorData();
						data.setFrequency(frequency);
						if (activityDecoratorData != null) {
							data.setProbability((double) frequency / activityDecoratorData.getFrequency());
						}
						flow2.setDecorator(new FlowDecorator(data));
						paintArrows2(flow2, data.getProbability());

					} else {
						// gateway then
						Collection<BPMNEdge<? extends BPMNNode, ? extends BPMNNode>> bpmnEdges = bpmn
								.getOutEdges(bpmnNode2);
						if (bpmnEdges.size() > 1) {
							// split gateway and it has to be AND gateway according to our implementation in C-net to BPMN conversion

							// get frequency of one of the flexnodes because it has AND gateway
							FlexNode flexNode2 = bpmnFlexNodeMapping.get(bpmnEdges.iterator().next().getTarget());
							Pair<Short, Short> shortRepresentation = new Pair<Short, Short>(codec.encode(flexNode),
									codec.encode(flexNode2));
							EdgeInstance edgeInstance = instanceMap.get(shortRepresentation);
							frequency = 0;
							if (edgeInstance != null) {
								frequency = edgeInstance.getFrequency();
							}
							// get correspondent BPMN flow
							Flow flow2 = (Flow) bpmnNodeGatewayEdge;
							paintArrows(minEdgeFreq, minMaxEdgeDifference, flow2, frequency);

							FlowDecoratorData data = new FlowDecoratorData();
							data.setFrequency(frequency);
							if (activityDecoratorData != null) {
								data.setProbability(frequency / frequency);
							}
							flow2.setDecorator(new FlowDecorator(data));
							paintArrows2(flow2, data.getProbability());
							for (BPMNEdge<? extends BPMNNode, ? extends BPMNNode> bpmnEdge : bpmnEdges) {
								Flow flow3 = (Flow) bpmnEdge;
								paintArrows(minEdgeFreq, minMaxEdgeDifference, flow3, frequency);
								FlowDecoratorData data2 = new FlowDecoratorData();
								data2.setFrequency(frequency);
								if (activityDecoratorData != null) {
									data2.setProbability(frequency / frequency);
								}
								flow3.setDecorator(new FlowDecorator(data2));
								paintArrows2(flow3, data2.getProbability());
							}
						} else {
							// join gateway
							FlexNode flexNode2 = bpmnFlexNodeMapping.get(bpmnEdges.iterator().next().getTarget());

							// get frequency
							Pair<Short, Short> shortRepresentation = new Pair<Short, Short>(codec.encode(flexNode),
									codec.encode(flexNode2));
							EdgeInstance edgeInstance = instanceMap.get(shortRepresentation);
							frequency = 0;
							if (edgeInstance != null) {
								frequency = edgeInstance.getFrequency();
							}

							// get correspondent BPMN flow
							Flow flow2 = (Flow) bpmnNodeGatewayEdge;
							paintArrows(minEdgeFreq, minMaxEdgeDifference, flow2, frequency);

							FlowDecoratorData data = new FlowDecoratorData();
							data.setFrequency(frequency);
							if (activityDecoratorData != null) {
								data.setProbability((double) frequency / activityDecoratorData.getFrequency());
							}
							flow2.setDecorator(new FlowDecorator(data));
							paintArrows2(flow2, data.getProbability());
						}

					}
				}

			}
			if (inNodes.size() == 0) {
				// end event
				// get correspondant bpmn node
				BPMNNode bpmnNode = flexBPMNNodeMapping.get(flexNode);

				// get edge to the gateway
				BPMNEdge<? extends BPMNNode, ? extends BPMNNode> inEdge = bpmn.getInEdges(bpmnNode).iterator().next();

				// get correspondent BPMN flow and paint the flow arrow
				Flow flow = (Flow) inEdge;
				ActivityDecoratorData activityDecoratorData = activityDecorationMap.get(bpmnNode);
				int frequency = 0;
				if (activityDecoratorData != null) {
					frequency = activityDecorationMap.get(bpmnNode).getFrequency();
				}
				if (flow != null) {
					paintArrows(minEdgeFreq, minMaxEdgeDifference, flow, frequency);
				}

			}
			if (inNodes.size() == 1) {
				FlexNode inNode = inNodes.get(0);

				// get frequency
				Pair<Short, Short> shortRepresentation = new Pair<Short, Short>(codec.encode(inNode),
						codec.encode(flexNode));
				EdgeInstance edgeInstance = instanceMap.get(shortRepresentation);
				int frequency = 0;
				if (edgeInstance != null) {
					frequency = edgeInstance.getFrequency();
				}
				// get correspondent BPMN flow
				FlexEdge<? extends FlexNode, ? extends FlexNode> flexEdge = flex.getInEdges(flexNode).iterator().next();
				Flow flow = (Flow) flexBPMNEdgeMapping.get(flexEdge);
				if (flow != null) {
					paintArrows(minEdgeFreq, minMaxEdgeDifference, flow, frequency);
				}

			}
			if (inNodes.size() > 1) {
				// get correspondant bpmn node
				BPMNNode bpmnNode = flexBPMNNodeMapping.get(flexNode);
				BPMNEdge<? extends BPMNNode, ? extends BPMNNode> bpmnEdge = bpmn.getInEdges(bpmnNode).iterator().next();

				// get correspondent BPMN flow and paint the flow arrow
				Flow flow = (Flow) bpmnEdge;
				ActivityDecoratorData activityDecoratorData = activityDecorationMap.get(bpmnNode);
				int frequency = 0;
				if (activityDecoratorData != null) {
					frequency = activityDecorationMap.get(bpmnNode).getFrequency();
				}
				paintArrows(minEdgeFreq, minMaxEdgeDifference, flow, frequency);

				BPMNNode bpmnNodeGateway = bpmnEdge.getSource();
				for (BPMNEdge<? extends BPMNNode, ? extends BPMNNode> bpmnNodeGatewayEdge : bpmn
						.getInEdges(bpmnNodeGateway)) {
					BPMNNode bpmnNode2 = bpmnNodeGatewayEdge.getSource();
					if (bpmnNode2 instanceof Activity) {
						FlexNode flexNode2 = bpmnFlexNodeMapping.get(bpmnNode2);

						// get frequency
						Pair<Short, Short> shortRepresentation = new Pair<Short, Short>(codec.encode(flexNode2),
								codec.encode(flexNode));
						EdgeInstance edgeInstance = instanceMap.get(shortRepresentation);
						frequency = 0;
						if (edgeInstance != null) {
							frequency = edgeInstance.getFrequency();
						}

						// get correspondent BPMN flow
						Flow flow2 = (Flow) bpmnNodeGatewayEdge;
						if (flow2 != null) {
							paintArrows(minEdgeFreq, minMaxEdgeDifference, flow2, frequency);
						}
					} else {
						// bpmnNode2 gateway then
						Collection<BPMNEdge<? extends BPMNNode, ? extends BPMNNode>> bpmnEdges = bpmn
								.getInEdges(bpmnNode2);
						if (bpmnEdges.size() > 1) {
							// join gateway and it has to be AND gateway according to our implementation in C-net to BPMN conversion

							// get frequency of one of the flexnodes because it has AND gateway
							FlexNode flexNode2 = bpmnFlexNodeMapping.get(bpmnEdges.iterator().next().getSource());
							Pair<Short, Short> shortRepresentation = new Pair<Short, Short>(codec.encode(flexNode),
									codec.encode(flexNode2));
							EdgeInstance edgeInstance = instanceMap.get(shortRepresentation);
							frequency = 0;
							if (edgeInstance != null) {
								frequency = edgeInstance.getFrequency();
							}
							// get correspondent BPMN flow
							Flow flow2 = (Flow) bpmnNodeGatewayEdge;
							if (flow2 != null) {
								paintArrows(minEdgeFreq, minMaxEdgeDifference, flow2, frequency);
							}
							for (BPMNEdge<? extends BPMNNode, ? extends BPMNNode> bpmnEdge2 : bpmnEdges) {
								Flow flow3 = (Flow) bpmnEdge2;
								paintArrows(minEdgeFreq, minMaxEdgeDifference, flow3, frequency);
							}
						} else {
							// then it is a split gateway so skip this part because we take thi into account in above second condition
						}

					}
				}

			}

		}
	}

	private Map<Event, EventDecoratorData> decorEvents(Map<Activity, ActivityDecoratorData> activityDecorationMap) {
		Map<Event, EventDecoratorData> eventDecorationMap = new HashMap<Event, EventDecoratorData>();
		for (Event event : bpmn.getEvents()) {
			viewSpecificMap.putViewSpecific(event, AttributeMap.FILLCOLOR, Color.LIGHT_GRAY);
			if (event.getEventType().equals(EventType.START)) {
				BPMNEdge<? extends BPMNNode, ? extends BPMNNode> bpmnEdge = bpmn.getOutEdges(event).iterator().next();
				Flow flow = (Flow) bpmnEdge;
				BPMNNode bpmnNode = flow.getTarget();
				ActivityDecoratorData data = activityDecorationMap.get(bpmnNode);
				// SET EVENT DECORATION
				EventDecoratorData eventDecoratorData = new EventDecoratorData();
				eventDecoratorData.setFrequency(data.getFrequency());
				eventDecorationMap.put(event, eventDecoratorData);
				event.setDecorator(new EventDecorator(eventDecoratorData));

			} else if (event.getEventType().equals(EventType.END)) {
				ActivityDecoratorData data = activityDecorationMap.get(bpmn.getInEdges(event).iterator().next()
						.getSource());
				EventDecoratorData eventDecoratorData = new EventDecoratorData();
				eventDecoratorData.setFrequency(data.getFrequency());
				eventDecorationMap.put(event, eventDecoratorData);
				event.setDecorator(new EventDecorator(eventDecoratorData));
			}
		}
		return eventDecorationMap;
	}

	private void paintArrows(int minEdgeFreq, int minMaxEdgeDifference, Flow bpmnFlow, int frequency) {

		if (frequency == 0) {
			viewSpecificMap.putViewSpecific(bpmnFlow, AttributeMap.EDGECOLOR, Color.LIGHT_GRAY);
			viewSpecificMap.putViewSpecific(bpmnFlow, AttributeMap.LINEWIDTH, MINARCWIDTH);
		} else {
			viewSpecificMap.removeViewSpecific(bpmnFlow, AttributeMap.SHOWLABEL);
			viewSpecificMap.removeViewSpecific(bpmnFlow, AttributeMap.LABEL);
			viewSpecificMap.removeViewSpecific(bpmnFlow, AttributeMap.LABELCOLOR);
			viewSpecificMap.putViewSpecific(bpmnFlow, AttributeMap.LABELCOLOR, Color.BLACK);
			viewSpecificMap.putViewSpecific(bpmnFlow, AttributeMap.SHOWLABEL, true);
			viewSpecificMap.putViewSpecific(bpmnFlow, AttributeMap.LABEL, "" + frequency);
			viewSpecificMap.putViewSpecific(bpmnFlow, AttributeMap.EDGECOLOR, Color.BLACK);

			StringBuffer additionalText = new StringBuffer("<html><table border=0>"
					+ "<tr><td><b>Frequency:</b><br/></td><td>" + frequency + "</td></tr>");
			viewSpecificMap.putViewSpecific(bpmnFlow, AttributeMap.TOOLTIP, additionalText.append("</table></html>")
					.toString());

			float newSize = 0;
			if (minMaxEdgeDifference != 0) {
				newSize = MINARCWIDTH + (((frequency - minEdgeFreq) * MAXARCADDITIONWIDTH) / minMaxEdgeDifference);
			} else {
				newSize = MAXARCADDITIONWIDTH;
			}
			viewSpecificMap.putViewSpecific(bpmnFlow, AttributeMap.LINEWIDTH, newSize);
		}
	}

	private void paintArrows2(Flow bpmnFlow, double probability) {
		String string = (String) viewSpecificMap.get(bpmnFlow, AttributeMap.TOOLTIP);
		if (string != null) {
			if (string.indexOf("</table></html>") > 0) {
				StringBuffer additionalText = new StringBuffer(string.substring(0, string.indexOf("</html>")));
				additionalText.append("<tr><td><b>Probability:</b><br/></td><td>" + probability + "</td></tr>");
				viewSpecificMap.putViewSpecific(bpmnFlow, AttributeMap.TOOLTIP, additionalText
						.append("</table></html>").toString());
			}
		}
	}

	*//**
	 * @param activityDecorationMap
	 * @param eventDecorationMap
	 *//*
	private void decorGateways(Map<Activity, ActivityDecoratorData> activityDecorationMap,
			Map<Event, EventDecoratorData> eventDecorationMap) {
		FlexPerfRepInfo perfResult = getSelectedBPMNAnalysisData().getPerfRepInfo();

		// Decoring Gateways
		for (Gateway gateway : bpmn.getGateways()) {
			viewSpecificMap.putViewSpecific(gateway, AttributeMap.FILLCOLOR, Color.LIGHT_GRAY);
			Collection<BPMNEdge<? extends BPMNNode, ? extends BPMNNode>> inEdgesCollection = bpmn.getInEdges(gateway);
			Collection<BPMNEdge<? extends BPMNNode, ? extends BPMNNode>> outEdgesCollection = bpmn.getOutEdges(gateway);
			if (inEdgesCollection.size() == 1) {
				BPMNNode bpmnNode = inEdgesCollection.iterator().next().getSource();

				if (bpmnNode instanceof Gateway) {
					Collection<BPMNEdge<? extends BPMNNode, ? extends BPMNNode>> inEdgesCollection2 = bpmn
							.getInEdges(bpmnNode);
					BPMNNode bpmnNode2 = inEdgesCollection2.iterator().next().getSource();
					// no more iteration between gateways
					if (bpmnNode2 instanceof Event) {
						EventDecoratorData eventDecoratorData = eventDecorationMap.get(bpmnNode2);
						GatewayDecoratorData gatewayDecoratorData = new GatewayDecoratorData();
						gatewayDecoratorData.setFrequency(eventDecoratorData.getFrequency());
						gatewayDecoratorData.setCaseInvolvementRatio(gatewayDecoratorData.getFrequency()
								/ perfResult.getCaseInstance().getFrequency());
						gateway.setDecorator(new GatewayDecorator(gatewayDecoratorData));
					} else if (bpmnNode2 instanceof Activity) {
						Activity activity = (Activity) bpmnNode2;
						ActivityDecoratorData data = activityDecorationMap.get(activity);
						GatewayDecoratorData gatewayDecoratorData = new GatewayDecoratorData();
						gatewayDecoratorData.setFrequency((data != null) ? data.getFrequency() : 0);
						gatewayDecoratorData.setCaseInvolvementRatio(gatewayDecoratorData.getFrequency()
								/ perfResult.getCaseInstance().getFrequency());
						gateway.setDecorator(new GatewayDecorator(gatewayDecoratorData));
					}
				} else if (bpmnNode instanceof Event) {
					EventDecoratorData eventDecoratorData = eventDecorationMap.get(bpmnNode);
					GatewayDecoratorData gatewayDecoratorData = new GatewayDecoratorData();
					gatewayDecoratorData.setFrequency(eventDecoratorData.getFrequency());
					gatewayDecoratorData.setCaseInvolvementRatio(gatewayDecoratorData.getFrequency()
							/ perfResult.getCaseInstance().getFrequency());
					gateway.setDecorator(new GatewayDecorator(gatewayDecoratorData));
				} else if (bpmnNode instanceof Activity) {
					Activity activity = (Activity) bpmnNode;
					ActivityDecoratorData data = activityDecorationMap.get(activity);
					GatewayDecoratorData gatewayDecoratorData = new GatewayDecoratorData();
					gatewayDecoratorData.setFrequency((data != null) ? data.getFrequency() : 0);
					gatewayDecoratorData.setCaseInvolvementRatio(gatewayDecoratorData.getFrequency()
							/ perfResult.getCaseInstance().getFrequency());
					gateway.setDecorator(new GatewayDecorator(gatewayDecoratorData));
				}

			}
			if (outEdgesCollection.size() == 1) {
				BPMNNode bpmnNode = outEdgesCollection.iterator().next().getTarget();
				if (bpmnNode instanceof Gateway) {
					Collection<BPMNEdge<? extends BPMNNode, ? extends BPMNNode>> outEdgesCollection2 = bpmn
							.getOutEdges(bpmnNode);
					BPMNNode bpmnNode2 = outEdgesCollection2.iterator().next().getSource();
					// no more iteration between gateways
					if (bpmnNode2 instanceof Event) {
						EventDecoratorData eventDecoratorData = eventDecorationMap.get(bpmnNode2);
						GatewayDecoratorData gatewayDecoratorData = new GatewayDecoratorData();
						gatewayDecoratorData.setFrequency(eventDecoratorData.getFrequency());
						gatewayDecoratorData.setCaseInvolvementRatio(gatewayDecoratorData.getFrequency()
								/ perfResult.getCaseInstance().getFrequency());
						gateway.setDecorator(new GatewayDecorator(gatewayDecoratorData));
					} else if (bpmnNode2 instanceof Activity) {
						Activity activity = (Activity) bpmnNode2;
						ActivityDecoratorData data = activityDecorationMap.get(activity);
						GatewayDecoratorData gatewayDecoratorData = new GatewayDecoratorData();
						gatewayDecoratorData.setFrequency((data != null) ? data.getFrequency() : 0);
						gatewayDecoratorData.setCaseInvolvementRatio(gatewayDecoratorData.getFrequency()
								/ perfResult.getCaseInstance().getFrequency());
						gateway.setDecorator(new GatewayDecorator(gatewayDecoratorData));
					}
				} else if (bpmnNode instanceof Event) {
					EventDecoratorData eventDecoratorData = eventDecorationMap.get(bpmnNode);
					GatewayDecoratorData gatewayDecoratorData = new GatewayDecoratorData();
					gatewayDecoratorData.setFrequency(eventDecoratorData.getFrequency());
					gatewayDecoratorData.setCaseInvolvementRatio(gatewayDecoratorData.getFrequency()
							/ perfResult.getCaseInstance().getFrequency());
					gateway.setDecorator(new GatewayDecorator(gatewayDecoratorData));
				} else if (bpmnNode instanceof Activity) {
					Activity activity = (Activity) bpmnNode;
					ActivityDecoratorData data = activityDecorationMap.get(activity);
					GatewayDecoratorData gatewayDecoratorData = new GatewayDecoratorData();
					gatewayDecoratorData.setFrequency((data != null) ? data.getFrequency() : 0);
					gatewayDecoratorData.setCaseInvolvementRatio(gatewayDecoratorData.getFrequency()
							/ perfResult.getCaseInstance().getFrequency());
					gateway.setDecorator(new GatewayDecorator(gatewayDecoratorData));
				}
			}
		}
	}

	private Color getColor(Double val, double lowBound, double upBound) {
		if (Double.compare(Double.NaN, val) == 0) {
			return Color.GREEN;
		} else {
			if (Double.compare(val, lowBound) <= 0) {
				return Color.GREEN;
			} else if (Double.compare(val, upBound) <= 0) {
				return Color.YELLOW;
			} else {
				return Color.RED;
			}
		}
	}

	private double[] getSortedArrayFromSet(Set<Double> sumNodeAvgWorkingTime) {
		double[] arrSumNodeAvgWorking = new double[sumNodeAvgWorkingTime.size()];
		int counter = 0;
		for (double value : sumNodeAvgWorkingTime) {
			arrSumNodeAvgWorking[counter] = value;
			counter++;
		}
		Arrays.sort(arrSumNodeAvgWorking);
		return arrSumNodeAvgWorking;
	}

	private void updateDecorSemantic(FlexNode node, ActivityDecoratorData decor, FlexBinding binding, int frequency) {
		if ((binding.getEncodedInputBinding() != FlexCodec.BLANK)
				&& (binding.getEncodedInputBinding() != FlexCodec.EMPTYSET)) {
			int sizeInpBind = codec.getIOBindingsFor(binding.getEncodedInputBinding()).size();
			int sizeInp = bpmn.getInEdges(node).size();

			if (!(sizeInp == sizeInpBind || sizeInpBind == 1)) {
				decor.setOrJoinFreq(decor.getOrJoinFreq() + frequency);
			}

		}

		if ((binding.getEncodedOutputBinding() != FlexCodec.BLANK)
				&& (binding.getEncodedOutputBinding() != FlexCodec.EMPTYSET)) {
			int sizeOutBind = codec.getIOBindingsFor(binding.getEncodedOutputBinding()).size();
			int sizeOut = bpmn.getOutEdges(node).size();

			if (!(sizeOut == sizeOutBind || sizeOutBind == 1)) {
				decor.setOrSplitFreq(decor.getOrSplitFreq() + frequency);
			}
			
			 * if (sizeOut == sizeOutBind) {
			 * decor.setAndSplitFreq(decor.getAndSplitFreq() + frequency); }
			 * else if (sizeOutBind == 1) {
			 * decor.setXorSplitFreq(decor.getXorSplitFreq() + frequency); }
			 * else { decor.setOrSplitFreq(decor.getOrSplitFreq() + frequency);
			 * }
			 
		}
	}

	private void uncolorElements() {
		for (Activity activityNode : bpmn.getActivities()) {
			activityNode.setDecorator(null);

			// set the attributes
			viewSpecificMap.putViewSpecific(activityNode, AttributeMap.FILLCOLOR, Color.WHITE);
			viewSpecificMap.putViewSpecific(activityNode, AttributeMap.SHOWLABEL, false);
			viewSpecificMap.putViewSpecific(activityNode, AttributeMap.SIZE, new Dimension(NODEWIDTH, NODEHEIGHT));

		}
		for (Gateway gatewayNode : bpmn.getGateways()) {
			gatewayNode.setDecorator(null);

			// set the attributes
			viewSpecificMap.putViewSpecific(gatewayNode, AttributeMap.FILLCOLOR, Color.WHITE);
			viewSpecificMap.putViewSpecific(gatewayNode, AttributeMap.SHOWLABEL, false);
			viewSpecificMap.putViewSpecific(gatewayNode, AttributeMap.SIZE, new Dimension((NODEWIDTH + NODEHEIGHT) / 2,
					(NODEWIDTH + NODEHEIGHT) / 2));

		}
		for (Event eventNode : bpmn.getEvents()) {
			eventNode.setDecorator(null);

			// set the attributes
			viewSpecificMap.putViewSpecific(eventNode, AttributeMap.FILLCOLOR, Color.WHITE);
			viewSpecificMap.putViewSpecific(eventNode, AttributeMap.SHOWLABEL, false);
			viewSpecificMap.putViewSpecific(eventNode, AttributeMap.SIZE, new Dimension(NODEHEIGHT, NODEHEIGHT));

		}
		fillMapAndRefreshEdges(Color.BLACK);

		// refresh the figure
		graph.update(bpmn.getNodes());

	}

	private void fillMapAndRefreshEdges(Color color) {
		for (Flow flow : bpmn.getFlows()) {
			flow.setDecorator(null);
			viewSpecificMap.removeViewSpecific(flow, AttributeMap.SHOWLABEL);
			viewSpecificMap.putViewSpecific(flow, AttributeMap.SHOWLABEL, false);
			// set the attributes
			viewSpecificMap.putViewSpecific(flow, AttributeMap.EDGECOLOR, color);
			viewSpecificMap.putViewSpecific(flow, AttributeMap.LINEWIDTH, NORMALLINEWIDTH);
			viewSpecificMap.removeViewSpecific(flow, AttributeMap.TOOLTIP);
		}
		graph.update(bpmn.getEdges());
	}

	public void addElementCollection(Map<FlexNode, Double> workingTime, String label) {
		Pair<Map<FlexNode, Double>, String> pair = new Pair<Map<FlexNode, Double>, String>(workingTime, label);

		if (!component.isEnabled()) {
			((DefaultListModel) component.getModel()).removeAllElements();
			component.setEnabled(true);
		}
		if (!((DefaultListModel) component.getModel()).contains(pair)) {
			((DefaultListModel) component.getModel()).addElement(pair);
		}

	}

	public void setBpmn(BPMNDiagram bpmn) {
		this.bpmn = bpmn;
	}

	public BPMNDiagram getBpmn() {
		return bpmn;
	}

	public ProMJGraphPanel getGraphPanel() {
		return graphPanel;
	}

	public void setGraphPanel(ProMJGraphPanel graphPanel) {
		this.graphPanel = graphPanel;
	}

	public BPMNAnalysisDataList getBPMNAnalysisDataList() {
		return BPMNAnalysisDataList;
	}

	public void setBPMNAnalysisDataList(BPMNAnalysisDataList BPMNAnalysisDataList) {
		this.BPMNAnalysisDataList = BPMNAnalysisDataList;
	}

	public BPMNAnalysisData getSelectedBPMNAnalysisData() {
		return selectedBPMNAnalysisData;
	}

	public void setSelectedBPMNAnalysisData(BPMNAnalysisData selectedBPMNAnalysisData) {
		this.selectedBPMNAnalysisData = selectedBPMNAnalysisData;
	}

	*//**
	 * Update performance conformance information panel
	 *//*
	private void updateConfPerfAnalysisPanel() {
		CaseInstance caseInstance = getSelectedBPMNAnalysisData().getPerfRepInfo().getCaseInstance();
		ConformanceInfo confResult = getSelectedBPMNAnalysisData().getConformanceInfo();
		arrivalRateJLabel.setText("Arrival rate : " + getSelectedBPMNAnalysisData().getArrivalRate());
		frequencyJLabel.setText("Num. of cases : " + caseInstance.getFrequency());
		avgThroughputJLabel.setText("Avg. throughput time: "
				+ PerformanceUtils.getSuitableTimeStr(caseInstance.getAvgTimeSpan()));
		minThroughputJLabel.setText("Min. throughput time : "
				+ PerformanceUtils.getSuitableTimeStr(caseInstance.getMinTimeSpan()));
		maxThroughputJLabel.setText("Max. throughput time : "
				+ PerformanceUtils.getSuitableTimeStr(caseInstance.getMaxTimeSpan()));
		stdDevThroughputJLabel.setText("Std. deviation throughput time : "
				+ PerformanceUtils.getSuitableTimeStr(caseInstance.getStdDevTimeSpan()));

		if (confResult!=null) {
			fitnessLabel.setText("Avg. Fitness (out of all cases) : %" + nf.format(confResult.getFitness() * 100));
			if (confResult.getMinFitness() != 0) {
				minFitnessJLabel.setVisible(true);
				maxFitnessJLabel.setVisible(true);
				minFitnessJLabel.setText("Min. Fitness (out of all paths) : %" + nf.format(confResult.getMinFitness()));
				maxFitnessJLabel.setText("Max. Fitness (out of all paths) : %" + nf.format(confResult.getMaxFitness()));
			} else {
				minFitnessJLabel.setVisible(false);
				maxFitnessJLabel.setVisible(false);
			}
		}
		perfPanel.repaint();
		confPanel.repaint();
		confPerfAnalysisPanel.repaint();
	}
}

*//**
 * This DefaultSelectionModel subclass enables SINGLE_SELECTION mode and
 * overrides setSelectionInterval so that the first selection update in a
 * gesture (like mouse press, drag, release) toggles the current selection
 * state. A "gesture" starts when the first update to the selection model
 * occurs, and the gesture ends when the isAdjusting ListSelectionModel property
 * is set to false.
 *//*
class ToggleSelectionModel extends DefaultListSelectionModel {
	private static final long serialVersionUID = -8595521046515667658L;
	boolean gestureStarted = false;

	public ToggleSelectionModel() {
		super();
		this.setSelectionMode(SINGLE_SELECTION);
	}

	public void setSelectionInterval(int index0, int index1) {
		if (isSelectedIndex(index0) && !gestureStarted) {
			super.removeSelectionInterval(index0, index1);
		} else {
			super.setSelectionInterval(index0, index1);
		}
		gestureStarted = true;
	}

	public void setValueIsAdjusting(boolean isAdjusting) {
		if (isAdjusting == false) {
			gestureStarted = false;
		}
	}
}

class ViewListCellRenderer implements ListCellRenderer {
	private static final CompoundBorder BORDER = BorderFactory.createCompoundBorder(
			BorderFactory.createRaisedBevelBorder(), BorderFactory.createEmptyBorder(2, 5, 2, 5));;
	private static final CompoundBorder SELBORDER = BorderFactory.createCompoundBorder(
			BorderFactory.createLoweredBevelBorder(), BorderFactory.createEmptyBorder(2, 5, 2, 5));

	private final static JLabel LABEL = SlickerFactory.instance().createLabel("test");
	private static final Border EMPTYBORDER = BorderFactory.createEmptyBorder(2, 5, 2, 5);

	public ViewListCellRenderer() {
		LABEL.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));

		LABEL.setBorder(EMPTYBORDER);
		LABEL.setOpaque(false);
		SlickerDecorator.instance().decorate(LABEL);
	}

	@SuppressWarnings("unchecked")
	public Component getListCellRendererComponent(JList component, Object value, int index, boolean isSelected,
			boolean cellHasFocus) {

		Pair<Map<FlexNode, Double>, String> pair = (Pair<Map<FlexNode, Double>, String>) value;

		String label = pair.getSecond();
		LABEL.setText(fitLabelToWidth(LABEL, label, component.getWidth()));
		LABEL.setToolTipText(label);
		LABEL.setForeground(isSelected ? Color.WHITE : Color.BLACK);
		LABEL.setBorder(isSelected ? SELBORDER : BORDER);

		return LABEL;
	}

	public static String fitLabelToWidth(JComponent c, String label, int maxWidth) {
		FontMetrics metrics = c.getFontMetrics(c.getFont());
		Graphics g = c.getGraphics();
		boolean abbreviated = false;
		int width = Integer.MAX_VALUE;
		while (true) {
			if (label.length() < 2) {
				break;
			}
			String test = label;
			if (abbreviated) {
				test += "...";
			}
			Rectangle2D stringBounds = metrics.getStringBounds(test, g);
			width = (int) stringBounds.getWidth();
			if (width > maxWidth) {
				label = label.substring(0, label.length() - 1);
				if (!abbreviated) {
					abbreviated = true;
				}
			} else {
				break;
			}
		}
		if (abbreviated) {
			label += "...";
		}
		return label;
	}*/
}