package org.processmining.plugins.bpmn.analysis.pn.visualization;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.List;

import javax.swing.JComponent;

/**
 * @author ibayraktar This class is modification of class ProcessInstanceView
 *         package org.processmining.plugins.log.ui.logdialog made by Christian
 *         W. Guenther (christian@deckfour.org) similar to Arya's implementation
 *         in replayer package
 * 
 */
public class ProcessInstanceConformanceView extends JComponent implements MouseListener, MouseMotionListener {

	private static final long serialVersionUID = -3966399549433794592L;
	protected static Color colorAttenuationDark = new Color(0, 0, 0, 160);
	protected static Color colorAttenuationBright = new Color(0, 0, 0, 80);
	protected static Color colorBgInstanceflag = new Color(70, 70, 70, 210);
	protected static Color colorBgEventFlag = new Color(30, 30, 30, 200);

	protected static DecimalFormat format = new DecimalFormat("##0.00%");
	protected static DateFormat dateFormat = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss.SSS");

	public static int trackPadding = 150;
	protected static int trackY = 40;
	protected static int trackHeight = 35;
	public static int elementWidth = 5;
	protected static int elementTriOffset = 20;
	protected String traceLabel;
	protected int maxOccurrenceCount;
	protected boolean mouseOver = false;
	protected int mouseX;
	protected int mouseY;
	protected int frequency;

	// modification
	protected List<Object> nodeInstance = null;
	protected List<org.processmining.plugins.petrinet.replayresult.StepTypes> stepTypes = null;

	// coloring
	public static Color RED = new Color(255, 0, 0);
	public static Color GREEN = new Color(0, 255, 0);
	public static Color GRAY = new Color(100, 100, 100);
	public static Color YELLOW = new Color(255, 255, 0);
	public static Color PURPLE = new Color(205, 106, 205);

	public ProcessInstanceConformanceView(List<Object> nodeInstance, List<org.processmining.plugins.petrinet.replayresult.StepTypes> stepTypes, int confViewWidth,
			int frequency) {
		this.traceLabel = "Replayed";
		this.nodeInstance = nodeInstance;
		this.stepTypes = stepTypes;
		this.frequency = frequency;
		this.setSize(confViewWidth - 100, 100);
		this.setPreferredSize(new Dimension(confViewWidth - 100, 100));
		this.setMinimumSize(new Dimension(confViewWidth - 100, 100));
		this.setMaximumSize(new Dimension(confViewWidth - 100, 100));

		addMouseListener(this);
		addMouseMotionListener(this);
		setDoubleBuffered(true);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	@Override
	protected void paintComponent(Graphics g) {
		Rectangle clip = getVisibleRect();//g.getClipBounds();
		Graphics2D g2d = (Graphics2D) g;
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		// draw background
		g2d.setColor(new Color(30, 30, 30));
		g2d.fillRect(clip.x, clip.y, clip.width, clip.height);
		// determine active event
		int activeEvent = -1;
		if (mouseOver == true) {
			activeEvent = mapEventIndex(mouseX, mouseY);
		}
		// draw events
		int clipRightX = clip.x + clip.width;
		int trackRightX;

		trackRightX = trackPadding + (nodeInstance.size() * elementWidth);

		int startX = clip.x - (clip.x % elementWidth); // shift to left if necessary
		if (startX < trackPadding) {
			startX = trackPadding;
		}
		int eventIndex = (startX - trackPadding) / elementWidth;
		for (int x = startX; ((x < clipRightX) && (x < trackRightX)); x += elementWidth + 200) {
			drawEvent(g2d, eventIndex, (eventIndex == activeEvent), x, trackY, elementWidth, trackHeight);
			eventIndex++;
		}
		if (clip.x <= trackRightX) {
			// draw instance flag
			drawInstanceFlag(g2d, clip.x, 25, 60); //trackY, trackHeight);
			// draw event flag
			if (activeEvent >= 0) {
				int eventX = trackPadding + (activeEvent * elementWidth);
				drawEventFlag(g2d, activeEvent, eventX, 5, 30);
			}
		}

	}

	protected int mapEventIndex(int x, int y) {
		if ((y >= trackY) && (y <= (trackY + trackHeight))) {
			// y-coordinate matches, remap x to index
			x -= trackPadding;
			x /= elementWidth;
			if ((x >= 0) && (x < nodeInstance.size())) {
				return x;
			} else {
				return -1;
			}
		} else {
			return -1;
		}

	}

	protected void drawInstanceFlag(Graphics2D g2d, int x, int y, int height) {
		String size;
		size = nodeInstance.size() + " activities";

		String processInstanceStr = frequency + " process ";
		String processInstanceStr2 = "instances with";

		// calculate width
		g2d.setFont(g2d.getFont().deriveFont(11f));
		FontMetrics fm = g2d.getFontMetrics();
		int longest = 0;
		int nameWidth = fm.stringWidth(traceLabel);
		longest = nameWidth;
		int sizeWidth = fm.stringWidth(size);
		longest = (longest > sizeWidth) ? longest : sizeWidth;
		int processInstanceStrWidth = fm.stringWidth(processInstanceStr);
		longest = (longest > processInstanceStrWidth) ? longest : processInstanceStrWidth;
		int processInstanceStrWidth2 = fm.stringWidth(processInstanceStr2);
		longest = (longest > processInstanceStrWidth2) ? longest : processInstanceStrWidth2;

		int width = longest + 10;
		width = Math.max(width, trackPadding - 10);
		// draw flag shadow
		int shadowOffset = 4;
		int[] xSCoords = new int[] { x, x + width - elementTriOffset + shadowOffset, x + width + shadowOffset,
				x + width - elementTriOffset + shadowOffset, x };
		int[] ySCoords = new int[] { y + shadowOffset, y + shadowOffset, y + (height / 2) + shadowOffset,
				y + height + shadowOffset, y + height + shadowOffset };
		g2d.setColor(new Color(0, 0, 0, 100));
		g2d.fillPolygon(xSCoords, ySCoords, 5);
		// draw flag background
		g2d.setColor(colorBgInstanceflag);
		int[] xCoords = new int[] { x, x + width - elementTriOffset, x + width, x + width - elementTriOffset, x };
		int[] yCoords = new int[] { y, y, y + (height / 2), y + height, y + height };
		g2d.fillPolygon(xCoords, yCoords, 5);
		// draw string
		int fontHeight = fm.getHeight();
		int fontOffset = (height - 4 * fontHeight) / 5;
		g2d.setColor(new Color(220, 220, 220));
		g2d.drawString(traceLabel, x + 5, y + fontOffset + fontHeight - 2);
		g2d.setColor(new Color(200, 200, 200));
		g2d.drawString(processInstanceStr, x + 5, y + 2 * (fontOffset) + 2 * fontHeight - 2);
		g2d.setColor(new Color(200, 200, 200));
		g2d.drawString(processInstanceStr2, x + 5, y + 3 * (fontOffset) + 3 * fontHeight - 2);
		g2d.setColor(new Color(200, 200, 200));
		g2d.drawString(size, x + 5, y + height - fontOffset - 2);
	}

	protected void drawEventFlag(Graphics2D g2d, int index, int x, int y, int height) throws IndexOutOfBoundsException {
		String name = nodeInstance.get(index).toString();

		// calculate width
		g2d.setFont(g2d.getFont().deriveFont(9f));
		FontMetrics fm = g2d.getFontMetrics();
		int width = fm.stringWidth(name) + 10;

		// draw background
		g2d.setColor(colorBgEventFlag);
		g2d.fillRect(x, y, width, height);
		// set color
		Color color = getColor(stepTypes.get(index));
		g2d.setColor(color);
		// draw anchor line
		g2d.drawLine(x, y+10, x, y + height);
		// draw strings
		int fontHeight = fm.getHeight();
		int fontOffset = (height - fontHeight - fontHeight - fontHeight) / 4;
		g2d.drawString(name, 5, y + fontOffset + fontHeight - 2);
	}

	private Color getColor(org.processmining.plugins.petrinet.replayresult.StepTypes stepTypes) {
		switch (stepTypes) {
			case L :
				return ProcessInstanceConformanceView.YELLOW;
			case MINVI :
				return ProcessInstanceConformanceView.GRAY;
			case MREAL :
				return ProcessInstanceConformanceView.PURPLE;
			case LMNOGOOD :
				return ProcessInstanceConformanceView.RED;
			case LMGOOD :
				return ProcessInstanceConformanceView.GREEN;
			default :
				return Color.YELLOW;
		}
	}

	protected void drawEvent(Graphics2D g2d, int index, boolean active, int x, int y, int width, int height) {
		Color color;
		if (nodeInstance != null) {
			// set correct color for event
			color = getColorOfIndex(index);
		} else {
			// set correct color for event
			// XEventClass eventClass = info.getEventClasses().getClassOf(instance.get(index));
			// int occurrence = (eventClass != null ? eventClass.size() : 0);
			// double frequency = (maxOccurrenceCount == 0 ? 0.0 : (double) occurrence / (double) maxOccurrenceCount);
			color = ProcessInstanceConformanceView.GREEN;
		}

		if (active == false) {
			color = attenuateColor(color);
		}
		g2d.setColor(color);
		// draw triangularish shape
		int midPointBX = x + elementTriOffset;
		int midPointAX = x + width + elementTriOffset + 50;
		int midPointY = y + (height / 2);
		int[] xCoords = new int[] { x, x + width + 50, midPointAX, x + width + 50, x, midPointBX };
		int[] yCoords = new int[] { y, y, midPointY, y + height, y + height, midPointY };
		if (active == true) {
			for (int i = 0; i < xCoords.length; i++) {
				xCoords[i] -= 1;
				yCoords[i] -= 3;
			}
		}
		g2d.fillPolygon(xCoords, yCoords, 6);
		// draw attenuations for 3d-effect
		if (active == true) {
			g2d.setColor(colorAttenuationDark);
			g2d.drawPolyline(new int[] { x - 3, midPointBX - 3, x - 3 }, new int[] { y - 3, midPointY - 3,
					y + height - 3 }, 3);
			g2d.setColor(colorAttenuationBright);
			g2d.drawPolyline(new int[] { x - 2, midPointBX - 2, x - 2 }, new int[] { y - 3, midPointY - 3,
					y + height - 3 }, 3);
		} else {
			g2d.setColor(colorAttenuationDark);
			g2d.drawPolyline(new int[] { x, midPointBX, x }, new int[] { y, midPointY, y + height }, 3);
			g2d.setColor(colorAttenuationBright);
			g2d.drawPolyline(new int[] { x + 1, midPointBX + 1, x + 1 }, new int[] { y, midPointY, y + height }, 3);
		}
	}

	private Color getColorOfIndex(int index) {
		return getColor(stepTypes.get(index));
	}

	protected Color attenuateColor(Color color) {
		int red = (int) (color.getRed() * 0.5);
		int green = (int) (color.getGreen() * 0.5);
		int blue = (int) (color.getBlue() * 0.5);
		return new Color(red, green, blue);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent
	 * )
	 */
	public void mouseDragged(MouseEvent evt) {
		mouseX = evt.getX();
		mouseY = evt.getY();
		repaint();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
	 */
	public void mouseMoved(MouseEvent evt) {
		mouseX = evt.getX();
		mouseY = evt.getY();
		repaint();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent arg0) {
		// ignore
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent arg0) {
		mouseOver = true;
		repaint();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent arg0) {
		mouseOver = false;
		repaint();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent arg0) {
		// ignore
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent arg0) {
		// ignore
	}
}
