package org.processmining.plugins.bpmn.analysis.settings;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.info.impl.XLogInfoImpl;
import org.deckfour.xes.model.XLog;
import org.processmining.framework.util.LevenshteinDistance;
import org.processmining.models.flexiblemodel.Flex;
import org.processmining.models.flexiblemodel.FlexNode;

/**
 * @author arya
 * @email arya.adriansyah@gmail.com
 * @version Dec 1, 2009
 */
public class ModelToLogConnection {

	public static final XEventClass DUMMYEVENTCLASS = new XEventClass("DUMMY",Integer.MIN_VALUE);

	/**
	 * Internal data
	 */
	private FlexNode[] flexNodes;
	private final Map<FlexNode, XEventClass> selectedEvent = new HashMap<FlexNode, XEventClass>();
	private XEventClasses eventClasses;

	public ModelToLogConnection(Flex flex, XLog log) {
		findMapping(flex, log);
	}

	private void findMapping(Flex flex, XLog log) {

		// number of flex nodes
		List<FlexNode> listFlexNodes = new LinkedList<FlexNode>();
		for (FlexNode node : flex.getNodes()){
			if (!node.isInvisible()){
				listFlexNodes.add(node);
			}
		}
		flexNodes = listFlexNodes.toArray(new FlexNode[listFlexNodes.size()]);
		Arrays.sort(flexNodes);

		// create array of available event classes
		XEventClassifier nameClassifier = XLogInfoImpl.NAME_CLASSIFIER;
		XLogInfo summary = XLogInfoFactory.createLogInfo(log, nameClassifier);
		
		// init event classes 
		eventClasses = summary.getEventClasses();

		Collection<XEventClass> colEventClasses = eventClasses.getClasses();
		XEventClass[] arrEvClasses = colEventClasses.toArray(new XEventClass[colEventClasses.size()]);
		Arrays.sort(arrEvClasses);

		XEventClass[] availEventClasses = new XEventClass[arrEvClasses.length + 1];
		availEventClasses[0] = DUMMYEVENTCLASS;

		for (int i = 0; i < arrEvClasses.length; i++) {
			availEventClasses[i + 1] = arrEvClasses[i];
		}

		// insert correct item
		LevenshteinDistance lDist = new LevenshteinDistance();
		
		for (int i = 0; i < flexNodes.length; i++) {
			String currEvNodeStr = (flexNodes[i]).getLabel();
			
			// find the most suitable index
			int suitableIndex = 0;
			int bestDistance = Integer.MAX_VALUE;
			int calcDistance = Integer.MAX_VALUE;
			for (int j = 0; j < availEventClasses.length; j++) {
				calcDistance = lDist.getLevenshteinDistanceLinearSpace(currEvNodeStr, availEventClasses[j].toString());
				if (calcDistance < bestDistance){
					suitableIndex = j;
					bestDistance = calcDistance;
				}
			}
			selectedEvent.put(flexNodes[i], availEventClasses[suitableIndex]);
		}
	}

	public Map<FlexNode, XEventClass> getMapping() {
		Map<FlexNode, XEventClass> result = new HashMap<FlexNode, XEventClass>();
		for (int i = 0; i < flexNodes.length; i++) {
			result.put(flexNodes[i], selectedEvent.get(flexNodes[i]));
			System.out.println(flexNodes[i]+" -> "+selectedEvent.get(flexNodes[i]));
		}
		return result;
	}

	public XEventClasses getEventClasses() {
		return eventClasses;
	}
}
