package org.processmining.plugins.bpmn.analysis.settings.ui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import org.processmining.plugins.bpmn.analysis.BPMNAnalysis;
import org.processmining.plugins.bpmn.analysis.BPMNAnalysis.BPMNAnalysisSettings;

import com.fluxicon.slickerbox.factory.SlickerDecorator;
import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * @author ibayraktar
 * 
 */
public class BPMNAnalysisSettingsGUI extends JPanel implements ActionListener {
	private static final long serialVersionUID = -2608532190536549047L;
	private JPanel projectionDetailsPanel;
	private JLabel projectionDetailsPanelTitle, mostFrequentPathLabel, mostTimeConsumingPathLabel, leastFrequentPathLabel, leastTimeConsumingPathLabel;
	private SlickerFactory factory = SlickerFactory.instance();
	private SlickerDecorator decorator = SlickerDecorator.instance();

	private int mostFrequent;
	private int leastFrequent;
	private int mostTimeConsuming;
	private int leastTimeConsuming;

	private JTextField mostFrequentPathTextField, mostTimeConsumingPathTextField, leastFrequentPathTextField, leastTimeConsumingPathTextField;

	public BPMNAnalysisSettingsGUI(BPMNAnalysisSettings initialSettings) {
		setLeastFrequent(initialSettings.noLeastFrequentPaths);
		setMostFrequent(initialSettings.noMostFrequentPaths);
		setLeastTimeConsuming(initialSettings.noLeastTimeConsumingPaths);
		setMostTimeConsuming(initialSettings.noMostTimeConsumingPaths);

		this.init();
	}

	private void init() {

		this.projectionDetailsPanel = factory.createRoundedPanel(15, Color.LIGHT_GRAY);
		this.projectionDetailsPanel.setBounds(0, 0, 770, 250);

		this.projectionDetailsPanelTitle = factory.createLabel("Projection Details");
		this.projectionDetailsPanelTitle.setHorizontalAlignment(SwingConstants.CENTER);
		this.projectionDetailsPanelTitle.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 18));
		this.projectionDetailsPanelTitle.setForeground(new Color(40, 40, 40));
		this.projectionDetailsPanelTitle.setBounds(10, 10, 750, 30);

		this.mostFrequentPathLabel = factory.createLabel("Number of Most Frequent Path :");
		this.mostFrequentPathLabel.setForeground(Color.DARK_GRAY);
		this.mostFrequentPathLabel.setBounds(20, 60, 325, 30);
		decorator.decorate(mostFrequentPathLabel);
		projectionDetailsPanel.add(mostFrequentPathLabel);
		this.mostFrequentPathLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 14));

		this.mostFrequentPathTextField = new JTextField("" + mostFrequent);
		this.mostFrequentPathTextField.setPreferredSize(new Dimension(200, 30));
		this.mostFrequentPathTextField.setBounds(350, 60, 50, 25);
		this.mostFrequentPathTextField.addActionListener(this);
		projectionDetailsPanel.add(mostFrequentPathTextField);

		this.mostTimeConsumingPathLabel = factory.createLabel("Number of Most Time Consuming Path :");
		this.mostTimeConsumingPathLabel.setForeground(Color.DARK_GRAY);
		this.mostTimeConsumingPathLabel.setBounds(20, 100, 325, 30);
		decorator.decorate(mostTimeConsumingPathLabel);
		projectionDetailsPanel.add(mostTimeConsumingPathLabel);
		this.mostTimeConsumingPathLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 14));

		this.mostTimeConsumingPathTextField = new JTextField("" + mostTimeConsuming);
		this.mostTimeConsumingPathTextField.setPreferredSize(new Dimension(200, 30));
		this.mostTimeConsumingPathTextField.setBounds(350, 100, 50, 25);
		this.mostFrequentPathTextField.addActionListener(this);
		projectionDetailsPanel.add(mostTimeConsumingPathTextField);

		this.leastFrequentPathLabel = factory.createLabel("Number of Least Frequent Path :");
		this.leastFrequentPathLabel.setForeground(Color.DARK_GRAY);
		this.leastFrequentPathLabel.setBounds(20, 140, 325, 30);
		decorator.decorate(leastFrequentPathLabel);
		projectionDetailsPanel.add(leastFrequentPathLabel);
		this.leastFrequentPathLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 14));

		this.leastFrequentPathTextField = new JTextField("" + leastFrequent);
		this.leastFrequentPathTextField.setPreferredSize(new Dimension(200, 30));
		this.leastFrequentPathTextField.setBounds(350, 140, 50, 25);
		this.mostFrequentPathTextField.addActionListener(this);
		projectionDetailsPanel.add(leastFrequentPathTextField);

		this.leastTimeConsumingPathLabel = factory.createLabel("Number of Least Time Consuming Path :");
		this.leastTimeConsumingPathLabel.setForeground(Color.DARK_GRAY);
		this.leastTimeConsumingPathLabel.setBounds(20, 180, 325, 30);
		decorator.decorate(leastTimeConsumingPathLabel);
		projectionDetailsPanel.add(leastTimeConsumingPathLabel);
		this.leastTimeConsumingPathLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 14));

		this.leastTimeConsumingPathTextField = new JTextField("" + leastTimeConsuming);
		this.leastTimeConsumingPathTextField.setPreferredSize(new Dimension(200, 30));
		this.leastTimeConsumingPathTextField.setBounds(350, 180, 50, 25);
		this.mostFrequentPathTextField.addActionListener(this);
		projectionDetailsPanel.add(leastTimeConsumingPathTextField);

		this.projectionDetailsPanel.setLayout(null);
		this.projectionDetailsPanel.add(this.projectionDetailsPanelTitle);

		this.setLayout(null);
		this.add(this.projectionDetailsPanel);
		this.validate();
		this.repaint();
	}

	
	public void actionPerformed(ActionEvent e) {

		mostFrequent = Integer.parseInt(this.mostFrequentPathTextField.getText());
		leastFrequent = Integer.parseInt(this.leastFrequentPathTextField.getText());
		mostTimeConsuming = Integer.parseInt(this.mostTimeConsumingPathTextField.getText());
		leastTimeConsuming = Integer.parseInt(this.leastTimeConsumingPathTextField.getText());

	}

	public int getMostFrequent() {
		return Integer.parseInt(this.mostFrequentPathTextField.getText());
	}

	public void setMostFrequent(int mostFrequent) {
		this.mostFrequent = mostFrequent;
	}

	public int getLeastFrequent() {
		return Integer.parseInt(this.leastFrequentPathTextField.getText());
	}

	public void setLeastFrequent(int leastFrequent) {
		this.leastFrequent = leastFrequent;
	}

	public int getMostTimeConsuming() {
		return Integer.parseInt(this.mostTimeConsumingPathTextField.getText());
	}

	public void setMostTimeConsuming(int mostTimeConsuming) {
		this.mostTimeConsuming = mostTimeConsuming;
	}

	public int getLeastTimeConsuming() {
		return Integer.parseInt(this.leastTimeConsumingPathTextField.getText());
	}

	public void setLeastTimeConsuming(int leastTimeConsuming) {
		this.leastTimeConsuming = leastTimeConsuming;
	}

	public BPMNAnalysis.BPMNAnalysisSettings getSettings() {
		BPMNAnalysisSettings s = new BPMNAnalysisSettings();
		s.noLeastFrequentPaths = getLeastFrequent();
		s.noMostFrequentPaths = getMostFrequent();
		s.noLeastTimeConsumingPaths = getLeastTimeConsuming();
		s.noMostTimeConsumingPaths = getMostTimeConsuming();
		return s;
	}

}
