package org.processmining.plugins.bpmn.analysis.settings.ui;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.JOptionPane;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.info.impl.XLogInfoImpl;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.models.connections.flexiblemodel.FlexLogConnection;
import org.processmining.models.flexiblemodel.Flex;
import org.processmining.models.flexiblemodel.FlexNode;
import org.processmining.plugins.flex.replayer.algorithms.CancellationAwareAStarAlgorithm;
import org.processmining.plugins.flex.replayer.algorithms.ExtendedCostBasedAStarLogReplayAlgorithm;
import org.processmining.plugins.flex.replayer.performance.ui.SimpleMappingStep;
import org.processmining.plugins.flex.replayer.performance.util.FlexSpecialNodes;
import org.processmining.plugins.flex.replayer.performance.util.LifecycleIdentifier;
import org.processmining.plugins.flex.replayer.ui.AlgorithmStep;
import org.processmining.plugins.flex.replayer.ui.IntroductionStep;
import org.processmining.plugins.flex.replayer.ui.ParamSettingCancellationAwareAStarAlgorithm;
import org.processmining.plugins.flex.replayer.ui.ParamSettingExtendedCostBasedReplay;
import org.processmining.plugins.flex.replayer.ui.ParamSettingStep;
import org.processmining.plugins.flex.replayer.ui.ReplayStep;
import org.processmining.plugins.replayer.converter.lifecycletransition.FullLifecycleTransition;
import org.processmining.plugins.replayer.converter.lifecycletransition.OnlyCompleteLifecycleTransition;
import org.processmining.plugins.replayer.converter.lifecycletransition.StartCompleteLifecycleTransition;

/**
 * @author arya
 * @email arya.adriansyah@gmail.com
 * @version Dec 1, 2009
 */
public class ReplayerSettingsUI {
	// reference variable
	private final UIPluginContext context;

	public static final int MAPPING = 0;
	public static final int ALGORITHM = 1;
	public static final int PARAMETERS = 2;
	
	@SuppressWarnings("unused")
	private boolean isRequirementSatisfied=true;
	
	// steps 
	private int nofSteps;
	private int currentStep;
	private int introductionStep;
	private int mappingStep;
	private int algorithmStep;
	private int testingParamStep;

	// gui for each steps
	private ReplayStep[] replaySteps;

	private XEventClasses eventClassesName, eventClassesStd;

	// number of params returned
	private int[] paramNum;

	public ReplayerSettingsUI(final UIPluginContext context) {
		this.context = context;
	}
	
	public Object[] getConfiguration(Flex flex, XLog log, boolean isTestingMode) {
		// init local parameter
		FlexLogConnection conn = null;

		// init steps and gui
		nofSteps = 0;
		introductionStep = nofSteps++;

		// check connection in order to determine whether mapping step is needed of not
		try {
			// connection is found, no need for mapping step
			conn = context.getConnectionManager().getFirstConnection(FlexLogConnection.class, context, flex, log);
			mappingStep = -1;
		} catch (Exception e) {
			// connection is not found, ask user to map each nodes in causal net to event classes
			mappingStep = nofSteps++;
		}

		// other steps
		algorithmStep = nofSteps++;
		testingParamStep = nofSteps++;

		// init gui for each step
		replaySteps = new ReplayStep[nofSteps];
		replaySteps[introductionStep] = new IntroductionStep();
		replaySteps[algorithmStep] = new AlgorithmStep();
		if (mappingStep > 0) { // mapping step is needed
			XEventClassifier nameClassifier = XLogInfoImpl.NAME_CLASSIFIER;
			XLogInfo summary = XLogInfoFactory.createLogInfo(log, nameClassifier);
			eventClassesName = summary.getEventClasses();

			XEventClassifier stdClassifier = XLogInfoImpl.STANDARD_CLASSIFIER;
			XLogInfo summaryStd = XLogInfoFactory.createLogInfo(log, stdClassifier);
			eventClassesStd = summaryStd.getEventClasses();

			replaySteps[mappingStep] = new SimpleMappingStep(flex, new FlexSpecialNodes(), eventClassesName);
		}

		// set current step
		currentStep = introductionStep;

		int[] configIndexes = new int[1];
		configIndexes[0] = testingParamStep;

		return showConfiguration(isTestingMode, log, flex, conn, configIndexes);
	}

	private Object[] showConfiguration(boolean isTestingMode, XLog log, Flex flex, FlexLogConnection conn,
			int[] configIndexes) {
		// init result variable
		InteractionResult result = InteractionResult.NEXT;

		// configure interaction with user
		while (true) {
			if (currentStep < 0) {
				currentStep = 0;
			}
			if (currentStep >= nofSteps) {
				currentStep = nofSteps - 1;
			}

			result = context.showWizard("Replay Log in Causal Net", currentStep == 0, currentStep == nofSteps - 1,
					replaySteps[currentStep]);
			switch (result) {
				case NEXT :
					// special checking if going after mapping
					if (currentStep == mappingStep) {
						// smartly create mapping
						LifecycleIdentifier li = new LifecycleIdentifier();
						li.addSupportedLifecycle(new FullLifecycleTransition());
						li.addSupportedLifecycle(new OnlyCompleteLifecycleTransition());
						li.addSupportedLifecycle(new StartCompleteLifecycleTransition());

						// identify lifecycle of each activity
						Set<XEventClass> unmappedEC = li.identifyLifecycleForEachActivity(eventClassesName,
								eventClassesStd);
						if (unmappedEC != null) {
							// requirement is not satisfied
							String message = "";
							String limiter = "";
							for (XEventClass evClass : unmappedEC) {
								message += limiter;
								message += evClass.toString();
								limiter = "<br />- ";
							}
							isRequirementSatisfied=false;
							JOptionPane.showMessageDialog(null, "<html>The following event class: <br/> - " + message
									+ "<br/> " + "do not have appropriate lifecycle transition "
									+ "as the ones supported. Filter out event classes with lifecycle types other "
									+ "than 'start','complete','suspend', and 'resume'." + "</html>",
									"Unsatisfied Replay Requirement", JOptionPane.ERROR_MESSAGE);
						} else {

							// check mapping: all event class must be mapped to at least one node
							SimpleMappingStep mappingStepGUI = (SimpleMappingStep) replaySteps[mappingStep];
							Map<FlexNode, XEventClass> map = mappingStepGUI.getConfiguration();
							Collection<XEventClass> colEventClasses = eventClassesName.getClasses();
							colEventClasses.removeAll(map.values());

							if (!colEventClasses.isEmpty()) {
								// requirement is not satisfied
								String message = "";
								String limiter = "";
								for (XEventClass evClass : colEventClasses) {
									message += limiter;
									message += evClass.toString();
									limiter = "<br />- ";
								}
								isRequirementSatisfied=false;
								
								JOptionPane
										.showMessageDialog(
												null,
												"<html>The following event class: <br/> - "
														+ message
														+ "<br/> are not mapped to any nodes. Map them to nodes, or filter them out from the log and try again.</html>",
												"Unsatisfied Replay Requirement", JOptionPane.ERROR_MESSAGE);
							} else {
								replaySteps[currentStep].readSettings(((AlgorithmStep) replaySteps[algorithmStep])
										.getAlgorithm());
								go(1, isTestingMode);
							}
						}
					} else {
						replaySteps[currentStep].readSettings(((AlgorithmStep) replaySteps[algorithmStep])
								.getAlgorithm());
						go(1, isTestingMode);
					}
					break;
				case PREV :
					go(-1, isTestingMode);
					break;
				case FINISHED :
					// mapping variable
					SimpleMappingStep mappingStepGUI = (SimpleMappingStep) replaySteps[mappingStep];
					Map<FlexNode, XEventClass> mapping=mappingStepGUI.getConfiguration();
					

					// collect all parameters
					paramNum = new int[configIndexes.length];
					List<Object> allParameters = new LinkedList<Object>();
					for (int i = 0; i < configIndexes.length; i++) {
						ParamSettingStep testParamGUI = ((ParamSettingStep) replaySteps[configIndexes[i]]);
						Object[] params = testParamGUI.getAllParameters();
						paramNum[i] = params.length;
						for (Object o : params) {
							allParameters.add(o);
						}
					}

					return new Object[] { mapping, ((AlgorithmStep) replaySteps[algorithmStep]).getAlgorithm(),
							allParameters.toArray() };
				default :
					return new Object[] { null };
			}
		}
	}

	public int[] getParamNum() {
		return this.paramNum;
	}

	private int go(int direction, boolean isTestingMode) {
		currentStep += direction;

		// check which algorithm is selected and adjust parameter as necessary
		if (currentStep == testingParamStep) {
			if (((AlgorithmStep) replaySteps[algorithmStep]).getAlgorithm() instanceof ExtendedCostBasedAStarLogReplayAlgorithm) {
				if (isTestingMode) {
					replaySteps[testingParamStep] = new ParamSettingExtendedCostBasedReplay(true);
				} else {
					replaySteps[testingParamStep] = new ParamSettingExtendedCostBasedReplay(false);
				}
			} else if (((AlgorithmStep) replaySteps[algorithmStep]).getAlgorithm() instanceof CancellationAwareAStarAlgorithm) {
				if (isTestingMode) {
					replaySteps[testingParamStep] = new ParamSettingCancellationAwareAStarAlgorithm(true);
				} else {
					replaySteps[testingParamStep] = new ParamSettingCancellationAwareAStarAlgorithm(false);
				}
			}
		}

		if ((currentStep >= 0) && (currentStep < nofSteps)) {
			if (replaySteps[currentStep].precondition()) {
				return currentStep;
			} else {
				return go(direction, isTestingMode);
			}
		}
		return currentStep;
	}
}
