package org.processmining.plugins.bpmn.analysis.utils;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.processmining.models.flexiblemodel.Flex;
import org.processmining.models.flexiblemodel.FlexEdge;
import org.processmining.models.flexiblemodel.FlexNode;

public class PerformanceUtils {

	public static String getSuitableTimeStr(double avgMoveTime) {
		NumberFormat nf = NumberFormat.getInstance();
		nf.setMaximumFractionDigits(2);
		if (Double.compare(avgMoveTime, 1000 * 60 * 60 * 24) > 0) {
			// days...
			return (nf.format(avgMoveTime / (1000 * 60 * 60 * 24)) + " days");
		}
		else if (Double.compare(avgMoveTime, 1000 * 60 * 60) > 0) {
			// hours
			return (nf.format(avgMoveTime / (1000 * 60 * 60)) + " hours");
		}
		else if (Double.compare(avgMoveTime, 1000 * 60) > 0) {
			// minutes
			return (nf.format(avgMoveTime / (1000 * 60)) + " min");
		}
		else if (Double.compare(avgMoveTime, 1000) > 0) {
			// seconds
			return (nf.format(avgMoveTime / (1000)) + " sec");
		}
		return (nf.format(avgMoveTime));
	}

	/**
	 * @param isOut
	 *            : false for Inedges, true for Outedges
	 * @return List<FlexNode>
	 */
	public static List<FlexNode> getFlexNodeList(Flex flex, FlexNode entry, boolean isOut) {
		List<FlexNode> targetNodeList = new ArrayList<FlexNode>();
		Collection<FlexEdge<? extends FlexNode, ? extends FlexNode>> flexEdgesCollection = null;
		if (isOut) {
			flexEdgesCollection = flex.getOutEdges(entry);
			if (flexEdgesCollection != null) {
				for (FlexEdge<? extends FlexNode, ? extends FlexNode> flexEdge : flexEdgesCollection) {
					targetNodeList.add(flexEdge.getTarget());
				}
			}

		}
		else {
			flexEdgesCollection = flex.getInEdges(entry);
			if (flexEdgesCollection != null) {
				for (FlexEdge<? extends FlexNode, ? extends FlexNode> flexEdge : flexEdgesCollection) {
					targetNodeList.add(flexEdge.getSource());
				}
			}
		}

		return targetNodeList;
	}
}
