package org.processmining.plugins.bpmn.analysis.visualization;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;
import javax.swing.JViewport;
import javax.swing.SwingConstants;
import javax.swing.border.LineBorder;

import org.jgraph.event.GraphSelectionEvent;
import org.jgraph.event.GraphSelectionListener;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.util.ui.scalableview.VerticalLabelUI;
import org.processmining.models.graphbased.directed.DirectedGraphNode;
import org.processmining.models.graphbased.directed.bpmn.elements.Activity;
import org.processmining.models.graphbased.directed.bpmn.elements.Swimlane;
import org.processmining.models.graphbased.directed.socialnetwork.SocialNetwork;
import org.processmining.models.jgraph.visualization.ProMJGraphPanel;
import org.processmining.plugins.bpmn.analysis.data.BPMNAnalysisData;
import org.processmining.plugins.bpmn.analysis.utils.Triple;

import com.fluxicon.slickerbox.factory.SlickerDecorator;
import com.fluxicon.slickerbox.factory.SlickerFactory;

public class OrganizationalInteractionPanel extends JPanel {

	public static final int HoWSN = 1;
	public static final int WTSN = 2;
	public static final int STSN = 3;
	public static final int A_OTM = 4;
	public static final int S_OTM = 5;

	private static final long serialVersionUID = 5563202305263696868L;
	public static final int RIGHT = 0;
	public static final int BOTTOM = 1;
	public static int TAB_HEIGHT = 30;
	public static int TAB_WIDTH = 200;

	protected SlickerFactory factory = SlickerFactory.instance();
	protected SlickerDecorator decorator = SlickerDecorator.instance();

	private ProMJGraphPanel panel;
	private JViewport parentScroll;
	private int panelType = 1;
	private String panelName;
	private BPMNAnalysisData BPMNAnalysisData;
	private boolean activityClicked = false;
	private boolean swimlaneClicked = false;
	private JPanel panelOn, panelOff, infoPanel;
	private JLabel infoLabel, noOfUsersLabel, infoHeadLabel;
	private int location = 0;
	private int count;

	private int height;
	private int width;
	private JPanel sn = new JPanel();
	private PluginContext context;

	public static OrganizationalInteractionPanel instance() {
		return new OrganizationalInteractionPanel();
	}

	public OrganizationalInteractionPanel() {
		// Auto-generated constructor stub
	}

	public OrganizationalInteractionPanel createPanel(PluginContext context, ProMJGraphPanel panel, String panelName,
			BPMNAnalysisData bpmnAnalysisData, int location, int count, int width, int height, int panelType) {
		this.setLayout(new BorderLayout());
		this.context = context;
		this.parentScroll = panel.getViewport();
		this.panel = panel;
		this.panelName = panelName;
		this.panelType = panelType;
		this.location = location;
		this.count = count;
		this.width = width;
		this.height = height;
		this.BPMNAnalysisData = bpmnAnalysisData;

		sn = initMainPanel(context, panelType);
		sn.setOpaque(false);
		this.add(sn, BorderLayout.CENTER);
		this.setOpaque(false);

		initPanelOff();
		initPanelOn();
		initInfoPanel();

		this.setBorder(new LineBorder(Color.DARK_GRAY));
		this.setOpaque(true);
		this.setSize(new Dimension(width, height));

		initListeners();
		return this;
	}

	/**
	 * 
	 */
	private void initInfoPanel() {
		infoPanel = factory.createRoundedPanel();
		infoPanel.setLayout(new GridLayout(0, 1));
		infoPanel.setBorder(BorderFactory.createEmptyBorder());
		infoPanel.setBackground(new Color(238, 238, 238));
		infoHeadLabel = factory.createLabel("");
		infoHeadLabel.setForeground(Color.DARK_GRAY);
		infoHeadLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));
		infoPanel.add(infoHeadLabel);
		infoLabel = factory.createLabel("");
		infoLabel.setBorder(BorderFactory.createTitledBorder("INFO"));
		infoLabel.setForeground(Color.DARK_GRAY);
		infoLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));
		infoPanel.add(infoLabel);

		noOfUsersLabel = factory.createLabel("# of Users in Process : " + BPMNAnalysisData.getNoOfResources());
		noOfUsersLabel.setForeground(Color.DARK_GRAY);
		noOfUsersLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 14));
		updateInfoPanel();

		this.add(infoPanel, BorderLayout.NORTH);
	}

	public void updateInfoPanel() {
		switch (panelType) {
			case A_OTM : {
				sn.setLayout(new GridLayout(1, 1));
				sn.setBackground(new Color(238, 238, 238));
				infoHeadLabel.setText("Table for Number of Times Tasks Performed by Whom");
				infoLabel.setText("Please Select An Activity or A Swimlane To Display");

				noOfUsersLabel.setText("# of Users in Process : " + BPMNAnalysisData.getNoOfResources());
				infoPanel.add(noOfUsersLabel);
			}
				break;
			case HoWSN : {
				infoHeadLabel.setText("Social Network for Handover of Work Demonstration");
				infoLabel.setText("All Process Instances with All Activities considered");
			}
				break;
			case WTSN : {
				infoHeadLabel.setText("Social Network for Working Together Demonstration");
				infoLabel.setText("All Process Instances with All Activities considered");
			}
				break;
			case STSN : {
				infoHeadLabel.setText("Social Network for Performing Similar Task Demonstration");
				infoLabel.setText("All Process Instances with All Activities considered");
			}
				break;
		}
	}

	/**
	 * 
	 */
	private void initPanelOn() {
		this.panelOn = factory.createRoundedPanel(15, new Color(238, 238, 238));

		panelOn.setLayout(null);
		panelOn.setVisible(false);
		panelOn.setEnabled(false);
		panelOn.add(this);
		this.panel.add(panelOn, JLayeredPane.PALETTE_LAYER);

	}

	/**
	 * 
	 */
	private void initPanelOff() {
		this.panelOff = factory.createRoundedPanel(15, Color.DARK_GRAY);
		panelOff.setVisible(true);
		panelOff.setEnabled(true);
		panelOff.setLayout(null);
		JLabel panelTitle = factory.createLabel(this.getPanelName());
		panelTitle.setHorizontalTextPosition(SwingConstants.CENTER);
		panelTitle.setVerticalTextPosition(SwingConstants.CENTER);
		panelTitle.setForeground(Color.WHITE);
		panelTitle.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 16));
		panelOff.add(panelTitle);

		switch (this.location) {
			case RIGHT :
				panelTitle.setBounds(0, 5, TAB_HEIGHT, TAB_WIDTH);
				panelTitle.setUI(new VerticalLabelUI(true));
				break;
			case BOTTOM :
				panelTitle.setBounds(5, 0, TAB_WIDTH, TAB_HEIGHT);
				break;
		}

		this.panel.add(panelOff, JLayeredPane.PALETTE_LAYER);
	}

	/**
	 * @param context
	 * @param panelType
	 * @return
	 */
	private JPanel initMainPanel(PluginContext context, int panelType) {
		switch (panelType) {
			case HoWSN : {
				sn.removeAll();
				if (getBPMNAnalysisData().getHoW4CompleteLog() != null)
					sn = new BPMNAnalysisSocialNetworkUI(context, getBPMNAnalysisData().getHoW4CompleteLog());
			}
				break;
			case WTSN : {
				sn.removeAll();
				if (getBPMNAnalysisData().getWT4CompleteLog() != null)
					sn = new BPMNAnalysisSocialNetworkUI(context, getBPMNAnalysisData().getWT4CompleteLog());
			}
				break;
			case STSN : {
				sn.removeAll();
				if (getBPMNAnalysisData().getST4CompleteLog() != null)
					sn = new BPMNAnalysisSocialNetworkUI(context, getBPMNAnalysisData().getST4CompleteLog());
			}
				break;
			case A_OTM : {
				sn.removeAll();
			}
				break;

		}
		return sn;
	}

	public void initListeners() {

		this.panel.getGraph().addGraphSelectionListener(new GraphSelectionListener() {

			public void valueChanged(GraphSelectionEvent e) {
				repaintAll(true);

			}

		});
		this.panel.addComponentListener(new java.awt.event.ComponentListener() {
			public void componentHidden(ComponentEvent e) {
			}

			public void componentMoved(ComponentEvent e) {
			}

			public void componentShown(ComponentEvent e) {
			}

			public void componentResized(ComponentEvent e) {
				resize();
			}
		});
		MouseMotionListener mouseMotionListener = new MouseMotionListener() {

			@Override
			public synchronized void mouseMoved(MouseEvent e) {
				Point p = e.getPoint();
				if (e.getComponent() == getComponent()) {
					Point p2 = parentScroll.getViewPosition();
					p.setLocation(p.getX() - p2.getX(), p.getY() - p2.getY());
				}

				boolean isInPanelOff = panelOff.getBounds().contains(p);
				if (!activityClicked && isInPanelOff && panelOff.isVisible()) {
					displayPanel(isInPanelOff);
				}
				if (!swimlaneClicked && isInPanelOff && panelOff.isVisible()) {
					displayPanel(isInPanelOff);
				}

				boolean isInPanelOn = panelOn.getBounds().contains(p);
				if (!activityClicked && !isInPanelOn && !panelOff.isVisible()) {
					displayPanel(isInPanelOn);
				}
				if (!swimlaneClicked && !isInPanelOn && !panelOff.isVisible()) {
					displayPanel(isInPanelOn);
				}
			}

			/**
			 * @param b
			 */
			private void displayPanel(boolean b) {
				willChangeVisibility(b);
				panelOn.setVisible(b);
				panelOn.setEnabled(b);
				panelOff.setVisible(!b);
				panelOff.setEnabled(!b);
			}

			@Override
			public void mouseDragged(MouseEvent e) {
				// Auto-generated method stub

			}
		};
		this.panel.addMouseMotionListener(mouseMotionListener);
		this.panel.getGraph().addMouseMotionListener(mouseMotionListener);
	}

	private int getValue(String user, String task, List<Triple<String, String, Integer>> activityBasedOTMatrix) {
		for (Triple<String, String, Integer> triple : activityBasedOTMatrix) {
			if (triple.getFirst().equals(user) && triple.getSecond().equals(task)) {
				return triple.getThird().intValue();
			}
		}
		return 0;
	}

	public JComponent getComponent() {
		return this;
	}

	public int getPosition() {
		return SwingConstants.NORTH;
	}

	public String getPanelName() {
		return panelName;
	}

	public void setPanelName(String name) {
		this.panelName = name;
	}

	public int getPanelType() {
		return panelType;
	}

	public void setPanelType(int panelType) {
		this.panelType = panelType;
	}

	public void updated() {

	}

	public void willChangeVisibility(boolean to) {

	}

	/**
	 * 
	 */
	private void resize() {
		switch (location) {
			case RIGHT : {
				this.panelOn.setLocation(this.panel.getWidth() - width, this.panel.getHeight() - height - TAB_HEIGHT);
				this.panelOn.setBounds(this.panel.getWidth() - width, this.panel.getHeight() - height - TAB_HEIGHT,
						width, height);
				this.panelOff.setLocation(this.panel.getWidth() - TAB_HEIGHT, this.panel.getHeight() - TAB_HEIGHT
						- count * (10 + TAB_WIDTH));
				this.panelOff.setBounds(this.panel.getWidth() - TAB_HEIGHT, this.panel.getHeight() - TAB_HEIGHT - count
						* (10 + TAB_WIDTH), TAB_HEIGHT, TAB_WIDTH);
			}
				break;
			case BOTTOM : {
				this.panelOn.setLocation(
						this.panel.getWidth() - width - TAB_HEIGHT - ((count - 1) * (+10 + TAB_WIDTH)),
						this.panel.getHeight() - height);
				this.panelOn.setBounds(this.panel.getWidth() - width - TAB_HEIGHT - ((count - 1) * (10 + TAB_WIDTH)),
						this.panel.getHeight() - height, width, height);
				this.panelOff.setLocation(this.panel.getWidth() - TAB_HEIGHT - (count * (10 + TAB_WIDTH)),
						this.panel.getHeight() - TAB_HEIGHT);
				this.panelOff.setBounds(this.panel.getWidth() - TAB_HEIGHT - (count * (10 + TAB_WIDTH)),
						this.panel.getHeight() - TAB_HEIGHT, TAB_WIDTH, TAB_HEIGHT);
			}
				break;
		}

	}

	public BPMNAnalysisData getBPMNAnalysisData() {
		return BPMNAnalysisData;
	}

	public void setBPMNAnalysisData(BPMNAnalysisData BPMNAnalysisData) {
		this.BPMNAnalysisData = BPMNAnalysisData;
	}

	public PluginContext getContext() {
		return context;
	}

	public void setContext(PluginContext context) {
		this.context = context;
	}

	/**
	 * @param node
	 * @return
	 */
	public List<Triple<String, String, Integer>> getMatrixTripleList(DirectedGraphNode node) {
		List<Triple<String, String, Integer>> activityBasedOTMatrix = null;

		if (node instanceof Activity) {
			activityBasedOTMatrix = getBPMNAnalysisData().getSelectedActivityUTMatrixList().get(node);

		} else {
			activityBasedOTMatrix = getBPMNAnalysisData().getSelectedActivityUTMatrixList().get(node);
		}
		return activityBasedOTMatrix;
	}

	/**
	 * @param node
	 */
	public void repaintThis(DirectedGraphNode node) {
		sn.removeAll();

		List<Triple<String, String, Integer>> UTMatrixForSelectedNode = getMatrixTripleList(node);
		Set<String> taskSet = getTaskSet(node);

		if (UTMatrixForSelectedNode != null) {
			infoLabel.setText("Process Instances from " + getBPMNAnalysisData().getDescription()
					+ " only with activities related " + node.getLabel());
			infoLabel.setToolTipText(infoLabel.getText());

			noOfUsersLabel.setText("# of Users performed " + node.getLabel() + " : "
					+ BPMNAnalysisData.getNumberOfUsersPerTaskMap().get(node));
			noOfUsersLabel.setToolTipText(noOfUsersLabel.getText());

			Set<String> userSet = new LinkedHashSet<String>();

			for (Triple<String, String, Integer> triple : UTMatrixForSelectedNode) {
				userSet.add(triple.getFirst());
			}

			sn.setBorder(BorderFactory.createEmptyBorder());
			sn.setBackground(new Color(238, 238, 238));

			Dimension userD = new Dimension(80, 15);
			Dimension rest = new Dimension((this.width - 80) / taskSet.size(), 15);
			sn.setLayout(new GridLayout(0, taskSet.size() + 1));

			JLabel userLabel = new JLabel("");
			userLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));
			userLabel.setSize(userD);
			userLabel.setForeground(Color.DARK_GRAY);
			userLabel.setBorder(BorderFactory.createEmptyBorder(2, 5, 2, 5));
			// SlickerDecorator.instance().decorate(userLabel);
			sn.add(userLabel);

			for (String task : taskSet) {
				JLabel taskLabel = new JLabel(task);
				if (task.equals(node.getLabel())) {
					taskLabel.setForeground(Color.RED);
				} else {
					taskLabel.setForeground(Color.DARK_GRAY);
				}
				taskLabel.setSize(rest);
				taskLabel.setToolTipText(task);
				taskLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));
				taskLabel.setBorder(BorderFactory.createEmptyBorder(2, 5, 2, 5));
				// SlickerDecorator.instance().decorate(taskLabel);
				sn.add(taskLabel);
			}
			for (String user : userSet) {
				JLabel userLabel2 = new JLabel(user);
				userLabel2.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));
				userLabel2.setSize(userD);
				userLabel2.setForeground(Color.DARK_GRAY);
				userLabel2.setBorder(BorderFactory.createEmptyBorder(2, 5, 2, 5));
				// SlickerDecorator.instance().decorate(userLabel2);
				sn.add(userLabel2);

				for (String task : taskSet) {
					JLabel taskLabel = new JLabel("" + getValue(user, task, UTMatrixForSelectedNode));
					if (task.equals(node.getLabel())) {
						taskLabel.setForeground(Color.RED);
					} else {
						taskLabel.setForeground(Color.DARK_GRAY);
					}
					taskLabel.setSize(rest);
					taskLabel.setToolTipText(task);
					taskLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));
					taskLabel.setBorder(BorderFactory.createEmptyBorder(2, 5, 2, 5));
					// SlickerDecorator.instance().decorate(taskLabel);
					sn.add(taskLabel);

				}
			}
			repaint();
		} else {
			infoLabel.setText("Process Instances from " + getBPMNAnalysisData().getDescription()
					+ " only with activities related " + node.getLabel());
			infoLabel.setToolTipText(infoLabel.getText());

			noOfUsersLabel.setText("# of Users performed " + node.getLabel() + " : N/A");
			noOfUsersLabel.setToolTipText(noOfUsersLabel.getText());

			sn = new JPanel();
			JLabel label = new JLabel("No User Task Matrix information available for this task!");
			label.setToolTipText(label.getText());
			label.setForeground(Color.DARK_GRAY);
			sn.add(label);
			repaint();
		}

	}

	private Set<String> getTaskSet(DirectedGraphNode node) {
		Set<String> taskSet2 = null;

		if (node instanceof Activity) {
			taskSet2 = getBPMNAnalysisData().getSelectedNodeUTMatrixSortedHeaderMap().get(node);
		} else {
			taskSet2 = getBPMNAnalysisData().getSelectedNodeUTMatrixSortedHeaderMap().get(node);
		}
		return taskSet2;
	}

	/**
	 * 
	 */
	public void repaintAll(boolean b) {
		Collection<DirectedGraphNode> collection = panel.getSelectedNodes();
		if (collection.size() == 1) {
			DirectedGraphNode node = collection.iterator().next();
			if (node instanceof Activity) {
				swimlaneClicked = false;
				activityClicked = true;
				switch (panelType) {
					case HoWSN : {
						sn.removeAll();
						SocialNetwork socialNetwork = getBPMNAnalysisData().getActivityBasedHoWSocialNetworkList().get(
								node);
						infoLabel.setText("Process Instances from " + getBPMNAnalysisData().getDescription()
								+ " only with activities related " + node.getLabel());
						infoLabel.setToolTipText(infoLabel.getText());
						infoPanel.repaint();
						if (socialNetwork != null) {
							sn = new BPMNAnalysisSocialNetworkUI(getContext(), getBPMNAnalysisData()
									.getActivityBasedHoWSocialNetworkList().get(node));
							repaint();
							if (b) {
								panelOff.setVisible(false);
								panelOff.setEnabled(false);
								panelOn.setVisible(true);
								panelOn.setEnabled(true);
							}
						} else {
							sn = new JPanel();
							JLabel label = new JLabel("No Social Network available for this node!");
							label.setToolTipText(label.getText());
							label.setForeground(Color.DARK_GRAY);
							sn.add(label);
							repaint();
						}
					}
						break;
					case WTSN : {
						sn.removeAll();
						SocialNetwork socialNetwork = getBPMNAnalysisData().getActivityBasedWTSocialNetworkList().get(
								node);
						infoLabel.setText("Process Instances from " + getBPMNAnalysisData().getDescription()
								+ " only with activities related " + node.getLabel());
						infoLabel.setToolTipText(infoLabel.getText());
						infoPanel.repaint();
						if (socialNetwork != null) {
							sn = new BPMNAnalysisSocialNetworkUI(getContext(), getBPMNAnalysisData()
									.getActivityBasedWTSocialNetworkList().get(node));
							repaint();
						} else {
							sn = new JPanel();
							JLabel label = new JLabel("No Social Network available for this node!");
							label.setToolTipText(label.getText());
							label.setForeground(Color.DARK_GRAY);
							sn.add(label);
							repaint();
						}
					}
						break;
					case STSN : {
						sn.removeAll();
						SocialNetwork socialNetwork = getBPMNAnalysisData().getActivityBasedSTSocialNetworkList().get(
								node);
						infoLabel.setText("Process Instances from " + getBPMNAnalysisData().getDescription()
								+ " only with activities related " + node.getLabel());
						infoLabel.setToolTipText(infoLabel.getText());
						infoPanel.repaint();
						if (socialNetwork != null) {
							sn = new BPMNAnalysisSocialNetworkUI(getContext(), getBPMNAnalysisData()
									.getActivityBasedSTSocialNetworkList().get(node));
							repaint();
						} else {
							sn = new JPanel();
							JLabel label = new JLabel("No Social Network available for this node!");
							label.setToolTipText(label.getText());
							label.setForeground(Color.DARK_GRAY);
							sn.add(label);
							repaint();
						}

					}
						break;
					case A_OTM : {
						repaintThis(node);
						infoPanel.repaint();
						repaint();
					}
						break;

				}
				sn.setOpaque(false);
				add(sn, BorderLayout.CENTER);
				setOpaque(false);
			} else if (node instanceof Swimlane) {
				swimlaneClicked = true;
				activityClicked = false;

				switch (panelType) {
					case HoWSN : {
						sn.removeAll();
						SocialNetwork socialNetwork = getBPMNAnalysisData().getSwimlaneBasedHoWSocialNetworkList().get(
								node);
						infoLabel.setText("Process Instances from " + getBPMNAnalysisData().getDescription()
								+ " only with activities related " + node.getLabel());
						infoLabel.setToolTipText(infoLabel.getText());
						infoPanel.repaint();
						if (socialNetwork != null) {
							sn = new BPMNAnalysisSocialNetworkUI(getContext(), getBPMNAnalysisData()
									.getSwimlaneBasedHoWSocialNetworkList().get(node));
							repaint();
							if (b) {
								panelOff.setVisible(false);
								panelOff.setEnabled(false);
								panelOn.setVisible(true);
								panelOn.setEnabled(true);
							}
						} else {
							sn = new JPanel();
							JLabel label = new JLabel("No Social Network available for this node!");
							label.setToolTipText(label.getText());
							label.setForeground(Color.DARK_GRAY);
							sn.add(label);
							repaint();
						}
					}
						break;
					case WTSN : {
						sn.removeAll();
						SocialNetwork socialNetwork = getBPMNAnalysisData().getSwimlaneBasedWTSocialNetworkList().get(
								node);
						infoLabel.setText("Process Instances from " + getBPMNAnalysisData().getDescription()
								+ " only with activities related " + node.getLabel());
						infoLabel.setToolTipText(infoLabel.getText());
						infoPanel.repaint();
						if (socialNetwork != null) {
							sn = new BPMNAnalysisSocialNetworkUI(getContext(), getBPMNAnalysisData()
									.getSwimlaneBasedWTSocialNetworkList().get(node));
							repaint();
						} else {
							sn = new JPanel();
							JLabel label = new JLabel("No Social Network available for this node!");
							label.setToolTipText(label.getText());
							label.setForeground(Color.DARK_GRAY);
							sn.add(label);
							repaint();
						}
					}
						break;
					case STSN : {
						sn.removeAll();
						SocialNetwork socialNetwork = getBPMNAnalysisData().getSwimlaneBasedSTSocialNetworkList().get(
								node);
						infoLabel.setText("Process Instances from " + getBPMNAnalysisData().getDescription()
								+ " only with activities related " + node.getLabel());
						infoLabel.setToolTipText(infoLabel.getText());
						infoPanel.repaint();
						if (socialNetwork != null) {
							sn = new BPMNAnalysisSocialNetworkUI(getContext(), getBPMNAnalysisData()
									.getSwimlaneBasedSTSocialNetworkList().get(node));
							repaint();
						} else {
							sn = new JPanel();
							JLabel label = new JLabel("No Social Network available for this node!");
							label.setToolTipText(label.getText());
							label.setForeground(Color.DARK_GRAY);
							sn.add(label);
							repaint();
						}

					}
						break;
					case A_OTM : {
						repaintThis(node);
						infoPanel.repaint();
						repaint();
					}
						break;

				}
				sn.setOpaque(false);
				add(sn, BorderLayout.CENTER);
				setOpaque(false);
			} else {
				swimlaneClicked = false;
				activityClicked = false;
			}
		} else if (collection.size() >= 1) {
			initMainPanel(context, panelType);
			updateInfoPanel();
			sn.setOpaque(false);
			add(sn, BorderLayout.CENTER);
			setOpaque(false);
			panelOff.setVisible(true);
			panelOff.setEnabled(true);
			swimlaneClicked = false;
			activityClicked = false;
			panelOn.setVisible(false);
			panelOn.setEnabled(false);
		} else {

			initMainPanel(context, panelType);
			updateInfoPanel();
			sn.setOpaque(false);
			add(sn, BorderLayout.CENTER);
			setOpaque(false);
			panelOff.setVisible(true);
			panelOff.setEnabled(true);
			swimlaneClicked = false;
			activityClicked = false;
			panelOn.setVisible(false);
			panelOn.setEnabled(false);
		}
	}

}
