package org.processmining.plugins.bpmn.analysis.visualization;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;
import javax.swing.JViewport;
import javax.swing.SwingConstants;

import org.processmining.framework.util.ui.scalableview.ScalableComponent;
import org.processmining.framework.util.ui.scalableview.ScalableViewPanel;
import org.processmining.framework.util.ui.scalableview.VerticalLabelUI;
import org.processmining.models.jgraph.visualization.ProMJGraphPanel;

import com.fluxicon.slickerbox.factory.SlickerDecorator;
import com.fluxicon.slickerbox.factory.SlickerFactory;

public class PerfConfInteractionPanel extends JPanel implements MouseListener, MouseMotionListener {

	private static final long serialVersionUID = 5563202305263696868L;

	public static final int TAB_HEIGHT = 30;
	public static final int TAB_WIDTH = 220;
	public static final int RIGHT_UP = 0;
	public static final int LEFT_BOTTOM = 1;

	private int location = 0;

	private int height;
	private int width;
	// new FlowLayout(FlowLayout.LEADING, 0, 0);

	private JViewport parentScroll;
	private final ProMJGraphPanel panel;
	protected SlickerFactory factory = SlickerFactory.instance();
	protected SlickerDecorator decorator = SlickerDecorator.instance();
	private JComponent component;
	private String panelName;

	private JPanel panelOn, panelOff;
	public PerfConfInteractionPanel(ProMJGraphPanel panel, String panelName, JPanel confPanel, JPanel perfPanel, int width, int height, int location) {
		super(null);

		this.setBorder(BorderFactory.createEmptyBorder());
		this.setOpaque(true);

		this.width = width;
		this.height = height;
		this.location = location;

		this.setSize(new Dimension(width, height));

		this.addMouseMotionListener(this);
		this.addMouseListener(this);

		this.parentScroll = panel.getViewport();
		this.panel = panel;
		this.panelName = panelName;
		paintLegend(confPanel, perfPanel);
		initPanelOff();
		initPanelOn();

		initListeners();
	}

	/**
	 * 
	 */
	private void initPanelOn() {
		this.panelOn = factory.createRoundedPanel(15, Color.LIGHT_GRAY);

		panelOn.setLayout(null);
		panelOn.setVisible(false);
		panelOn.setEnabled(false);
		panelOn.add(this);

		this.panel.add(panelOn, JLayeredPane.PALETTE_LAYER);

	}

	/**
	 * 
	 */
	private void initPanelOff() {
		this.panelOff = factory.createRoundedPanel(15, Color.DARK_GRAY);
		panelOff.setVisible(true);
		panelOff.setEnabled(true);
		panelOff.setLayout(null);
		JLabel panelTitle = factory.createLabel(this.getPanelName());
		panelTitle.setHorizontalTextPosition(SwingConstants.CENTER);
		panelTitle.setVerticalTextPosition(SwingConstants.CENTER);
		panelTitle.setForeground(Color.WHITE);
		panelTitle.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 16));
		panelOff.add(panelTitle);

		panelTitle.setBounds(0, 5, TAB_HEIGHT, TAB_WIDTH);
		panelTitle.setUI(new VerticalLabelUI(true));

		this.panel.add(panelOff, JLayeredPane.PALETTE_LAYER);
	}

	public synchronized void paintLegend(JPanel confPanel, JPanel perfPanel) {
		this.setBackground(new Color(30, 30, 30));
		confPanel.setOpaque(false);

		this.add(perfPanel);
		this.setOpaque(false);

		perfPanel.setOpaque(false);
		this.add(confPanel);
		this.setOpaque(false);
	}

	public double getVisWidth() {
		return component.getSize().getWidth();
	}

	public double getVisHeight() {
		return component.getSize().getHeight();
	}

	@Override
	public void paint(Graphics g) {

		super.paint(g);
	}

	public synchronized void mouseDragged(MouseEvent evt) {
	}

	public void mouseClicked(MouseEvent e) {
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	public void mouseMoved(MouseEvent e) {
	}

	public synchronized void mousePressed(MouseEvent e) {

	}

	public synchronized void mouseReleased(MouseEvent e) {

	}

	public void setScalableComponent(ScalableComponent scalable) {
		this.component = scalable.getComponent();
	}

	public void setParent(ScalableViewPanel parent) {

	}

	public JComponent getComponent() {
		return this;
	}

	public int getPosition() {
		return SwingConstants.NORTH;
	}

	public String getPanelName() {
		return panelName;
	}

	public void setPanelName(String name) {
		this.panelName = name;
	}

	public void updated() {

	}

	public void initListeners() {
		this.panel.addComponentListener(new java.awt.event.ComponentListener() {
			public void componentHidden(ComponentEvent e) {
			}

			public void componentMoved(ComponentEvent e) {
			}

			public void componentShown(ComponentEvent e) {
			}

			public void componentResized(ComponentEvent e) {
				resize();
			}
		});
		MouseMotionListener mouseMotionListener = new MouseMotionListener() {

			@Override
			public synchronized void mouseMoved(MouseEvent e) {
				Point p = e.getPoint();
				if (e.getComponent() == getComponent()) {
					Point p2 = parentScroll.getViewPosition();
					p.setLocation(p.getX() - p2.getX(), p.getY() - p2.getY());
				}

				boolean isInPanelOff = panelOff.getBounds().contains(p);
				if (isInPanelOff) {
					displayPanel(isInPanelOff);
				}

				boolean isInPanelOn = panelOn.getBounds().contains(p);
				if (!isInPanelOn) {
					displayPanel(isInPanelOn);
				}
			}

			/**
			 * @param b
			 */
			private void displayPanel(boolean b) {
				panelOn.setVisible(b);
				panelOn.setEnabled(b);
				panelOff.setVisible(!b);
				panelOff.setEnabled(!b);
			}

			@Override
			public void mouseDragged(MouseEvent e) {
				// Auto-generated method stub

			}
		};
		this.panel.addMouseMotionListener(mouseMotionListener);
		this.panel.getGraph().addMouseMotionListener(mouseMotionListener);
	}

	/**
	 * 
	 */
	private void resize() {

		switch (location) {
			case RIGHT_UP: {
				this.panelOn.setLocation(this.panel.getWidth() - width, TAB_HEIGHT);
				this.panelOn.setBounds(this.panel.getWidth() - width, TAB_HEIGHT, width, height);
				this.panelOff.setLocation(this.panel.getWidth() - TAB_HEIGHT, TAB_HEIGHT);
				this.panelOff.setBounds(this.panel.getWidth() - TAB_HEIGHT, TAB_HEIGHT, TAB_HEIGHT, TAB_WIDTH);
			}
				break;
			case LEFT_BOTTOM: {
				this.panelOn.setLocation(0, this.panel.getHeight() - height - TAB_HEIGHT);
				this.panelOn.setBounds(0, this.panel.getHeight() - height - TAB_HEIGHT, width, height);
				this.panelOff.setLocation(0, this.panel.getHeight() - TAB_WIDTH - TAB_HEIGHT);
				this.panelOff.setBounds(0, this.panel.getHeight() - TAB_WIDTH - TAB_HEIGHT, TAB_HEIGHT, TAB_WIDTH);
			}
				break;
		}
	}

}
