package org.processmining.plugins.bpmn.analysis.xpdl.export;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.text.NumberFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UIExportPlugin;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.connections.ConnectionManager;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.bpmn.analysis.BPMNAnalysisConnection;
import org.processmining.models.graphbased.directed.bpmn.BPMNDiagram;
import org.processmining.models.graphbased.directed.bpmn.elements.Activity;
import org.processmining.models.graphbased.directed.bpmn.elements.Event;
import org.processmining.models.graphbased.directed.bpmn.elements.Event.EventType;
import org.processmining.models.graphbased.directed.bpmn.elements.Flow;
import org.processmining.models.graphbased.directed.bpmn.elements.IGraphElementDecoration;
import org.processmining.plugins.bpmn.analysis.data.BPMNAnalysisData;
import org.processmining.plugins.bpmn.analysis.data.BPMNAnalysisDataList;
import org.processmining.plugins.bpmn.analysis.decorators.ActivityDecorator;
import org.processmining.plugins.bpmn.analysis.decorators.ActivityDecoratorData;
import org.processmining.plugins.bpmn.analysis.decorators.EventDecorator;
import org.processmining.plugins.bpmn.analysis.decorators.EventDecoratorData;
import org.processmining.plugins.bpmn.analysis.decorators.FlowDecorator;
import org.processmining.plugins.bpmn.analysis.decorators.FlowDecoratorData;
import org.processmining.plugins.xpdl.Xpdl;
import org.processmining.plugins.xpdl.XpdlExtendedAttribute;
import org.processmining.plugins.xpdl.XpdlProcessHeader;
import org.processmining.plugins.xpdl.XpdlSimulationInformation;
import org.processmining.plugins.xpdl.XpdlStartEvent;
import org.processmining.plugins.xpdl.XpdlTimeEstimation;
import org.processmining.plugins.xpdl.XpdlTriggerTimer;
import org.processmining.plugins.xpdl.collections.XpdlExtendedAttributes;
import org.processmining.plugins.xpdl.collections.XpdlLanes;
import org.processmining.plugins.xpdl.collections.XpdlPerformers;
import org.processmining.plugins.xpdl.converter.BPMN2XPDLConversion;
import org.processmining.plugins.xpdl.expressiontype.XpdlExpressionType;
import org.processmining.plugins.xpdl.idname.XpdlActivity;
import org.processmining.plugins.xpdl.idname.XpdlLane;
import org.processmining.plugins.xpdl.idname.XpdlPool;
import org.processmining.plugins.xpdl.idname.XpdlTransition;
import org.processmining.plugins.xpdl.text.XpdlDuration;
import org.processmining.plugins.xpdl.text.XpdlPerformer;
import org.processmining.plugins.xpdl.text.XpdlText;
import org.processmining.plugins.xpdl.text.XpdlWaitingTime;
import org.processmining.plugins.xpdl.text.XpdlWorkingTime;

@Plugin(name = "BPMN Analysis XPDL 2.0 export", returnLabels = {}, returnTypes = {}, parameterLabels = {
		"BPMN Analysis XPDL 2.0 export", "File" }, userAccessible = true)
@UIExportPlugin(description = "BPMN Analysis XPDL 2.0 export", extension = "xpdl")
public class BPMNAnalysisXPDLExport {

	@PluginVariant(variantLabel = "BPMN Analysis XPDL 2.0 export", requiredParameterLabels = { 0, 1 })
	public void export(UIPluginContext context, BPMNDiagram bpmn, File file) throws IOException {

		BPMN2XPDLConversion xpdlConversion = new BPMN2XPDLConversion(bpmn);
		Xpdl xpdl = xpdlConversion.convert2XPDL(context);

		writeXPDLwithAnnotations(context, bpmn, xpdl, file);
	}
	
	@PluginVariant(variantLabel = "BPMN Analysis XPDL 2.0 export", requiredParameterLabels = { 0, 1 })
	public void export(PluginContext context, BPMNDiagram bpmn, File file) throws IOException {

		BPMN2XPDLConversion xpdlConversion = new BPMN2XPDLConversion(bpmn);
		Xpdl xpdl = xpdlConversion.convert2XPDL_noLayout();
		
		writeXPDLwithAnnotations(context, bpmn, xpdl, file);
	}
	
	public void writeXPDLwithAnnotations(PluginContext context, BPMNDiagram bpmn, Xpdl xpdl, File file) throws IOException {

		NumberFormat nf = NumberFormat.getInstance();
		nf.setMaximumFractionDigits(2);

		ConnectionManager conn = context.getConnectionManager();
		BPMNAnalysisDataList BPMNAnalysisList = null;
		try {
			BPMNAnalysisConnection bpmnConnection = conn.getFirstConnection(BPMNAnalysisConnection.class, context, bpmn);
			BPMNAnalysisList = (BPMNAnalysisDataList) bpmnConnection.getObjectWithRole(BPMNAnalysisConnection.BPMNAnalysisList);

		}
		catch (ConnectionCannotBeObtained e) {
			System.out.println("XPDL is exported without BPMN Analysis Information");
		}
		
		if (BPMNAnalysisList!=null) {
			XpdlProcessHeader header = xpdl.getWorkflowProcesses().getList().get(0).getProcessHeader();// DurationUnit in processs header
			header.setDurationUnit("h");
			Map<String, ActivityDecoratorData> map = new HashMap<String, ActivityDecoratorData>();
			for (Activity a : bpmn.getActivities()) {
				ActivityDecorator actDec = (ActivityDecorator) a.getDecorator();
				if (actDec != null) {
					ActivityDecoratorData actDecorData = actDec.getDecor();
					if (actDecorData!=null) {
						map.put(a.getLabel(), actDecorData);
					}
				} else {
					System.out.println(a.getLabel());
				}
			}
			Map<Integer, Double> map2 = new HashMap<Integer, Double>();
			for (Flow f : bpmn.getFlows()) {
				FlowDecoratorData data = (f.getDecorator() == null) ? null : ((FlowDecorator) f.getDecorator())
						.getDecor();
				if (data != null) {
					double probability = data.getProbability();
					map2.put(f.hashCode(), probability);
				}

				// we can obtain the in and out edges from here and get the frequencies for each edge but we can not set to xpdl there is no attribute for that
				// so we will use extended attibute tag by adding probability extended attibute
			}
			//get Arrival rate
			XpdlTriggerTimer triggerTimer = new XpdlTriggerTimer("XpdlTriggerTimer");
			XpdlExpressionType timeDateElement = new XpdlExpressionType("TimeDate");
			XpdlText text = new XpdlText("XpdlText");
			text.setText("TIME");
			timeDateElement.setXpdlText(text);
			XpdlExpressionType timeCycleElement = new XpdlExpressionType("TimeCycle");
			XpdlText text2 = new XpdlText("XpdlText");
			text2.setText("" + nf.format(formatArrivalRate(getArrivalRate(bpmn))));// hours
			timeCycleElement.setXpdlText(text2);
			triggerTimer.setTimeDateElement(timeDateElement);
			triggerTimer.setTimeCycleElement(timeCycleElement);
			List<XpdlActivity> activityList = xpdl.getWorkflowProcesses().getList().get(0).getActivities().getList();
			List<XpdlTransition> transitionList = xpdl.getWorkflowProcesses().getList().get(0).getTransitions()
					.getList();
			if (!map.isEmpty()) {
				for (XpdlActivity xpdlActivity : activityList) {
					XpdlStartEvent xpdlStartEvent = xpdlActivity.getEvent() != null ? xpdlActivity.getEvent()
							.getStartEvent() : null;
					if (xpdlStartEvent != null) {
						xpdlStartEvent.setTriggerTimer(triggerTimer);
					}
					if (map.containsKey(xpdlActivity.getName())) {
						ActivityDecoratorData data = map.get(xpdlActivity.getName());
						double avgTotalTime = data.getAvgTotalTime();// activity simulation info-> duration
						double avgWaitingTime = data.getAvgWaitingTime(); // activity simulation info-> waiting time
						double avgWorkingTime = data.getAvgWorkingTime(); // activity simulation info-> working time
						double probability = data.getCaseInvolvementRatio();// we can not set activity probability to the activity in XPDL-there is no field to fit. so we will use
																			// extended
																			// attibute tag by adding probability extended attibute

						XpdlSimulationInformation simulationInformation = new XpdlSimulationInformation(
								"SimulationInformation");
						XpdlTimeEstimation timeEstimation = new XpdlTimeEstimation("TimeEstimation");
						XpdlDuration duration = new XpdlDuration("Duration");
						duration.setText("" + avgTotalTime);
						XpdlWaitingTime waitingTime = new XpdlWaitingTime("WaitingTime");
						waitingTime.setText("" + avgWaitingTime);
						XpdlWorkingTime workingTime = new XpdlWorkingTime("WorkingTime");
						workingTime.setText("" + avgWorkingTime);

						timeEstimation.setDuration(duration);
						timeEstimation.setWaitingTime(waitingTime);
						timeEstimation.setWorkingTime(workingTime);
						simulationInformation.setTimeEstimation(timeEstimation);

						xpdlActivity.setSimulationInformation(simulationInformation);
						XpdlExtendedAttributes extendedAttributes = new XpdlExtendedAttributes("ExtendedAttributes");

						XpdlExtendedAttribute attribute = new XpdlExtendedAttribute("ExtendedAttribute");
						attribute.setName("Probability");
						attribute.setValue("" + probability);

						extendedAttributes.add2List(attribute);

						xpdlActivity.setExtendedAttributes(extendedAttributes);

						if (data.getPerformers()!=null) {
							if (!data.getPerformers().isEmpty()) {
								XpdlPerformers xpdlPerformers = new XpdlPerformers("Performers");
								for (String user : data.getPerformers()) {
									XpdlPerformer performer = new XpdlPerformer("Performer");
									performer.setText(user);
									xpdlPerformers.add2List(performer);
								}
								xpdlActivity.setPerformers(xpdlPerformers);
							}
						}

					}

				}
			}
			if (!map2.isEmpty()) {

				for (XpdlTransition xpdlTransition : transitionList) {
					if (map2.get(Integer.parseInt(xpdlTransition.getId())) != null) {

						XpdlExtendedAttributes extendedAttributes = new XpdlExtendedAttributes("ExtendedAttributes");

						XpdlExtendedAttribute attribute = new XpdlExtendedAttribute("ExtendedAttribute");
						attribute.setName("Probability");

						attribute.setValue("" + map2.get(Integer.parseInt(xpdlTransition.getId())).doubleValue());

						extendedAttributes.add2List(attribute);

						xpdlTransition.setExtendedAttributes(extendedAttributes);
					}
				}
			}
			
			//Swimlanes performers are filled according to the all traces 
			BPMNAnalysisData bpmnAnalysisData = BPMNAnalysisList.get(0);
			
			List<XpdlPool> xpdlPools =xpdl.getPools().getList();
			if (!xpdlPools.isEmpty()) {
				XpdlPool xpdlPool=xpdlPools.get(0);
				XpdlLanes lanes =xpdlPool.getLanes();
				
				if(lanes != null) {
					for (XpdlLane lane : lanes.getList()) {
						XpdlPerformers xpdlPerformers = new XpdlPerformers("Performers");
						if (bpmnAnalysisData.getSwimlaneUserListMap().containsKey(lane.getName())) {
							for (String user : bpmnAnalysisData.getSwimlaneUserListMap().get(lane.getName())) {
								XpdlPerformer performer = new XpdlPerformer("Performer");
								performer.setText(user);
								xpdlPerformers.add2List(performer);
							}
							lane.setPerformers(xpdlPerformers);
						}
					}
				}
			}
		}
		String text3 = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" + xpdl.exportElement();
		BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file)));
		bw.write(text3);
		bw.close();

	}

	private String getArrivalRate(BPMNDiagram bpmn) {
		String arrivalRate = "0.0  process instance(s) per milisecond";
		
		Event startEvent=null;
		for (Event tempEvent : bpmn.getEvents()) {
			if (tempEvent.getEventType().equals(EventType.START)) {
				startEvent=tempEvent;
			}
		}
		if (startEvent!=null) {
			IGraphElementDecoration decoration= startEvent.getDecorator();
			if (decoration!=null) {
				EventDecorator eventDecorator=(EventDecorator)decoration;
				EventDecoratorData eventDecoratorData=eventDecorator.getDecor();
				if (eventDecoratorData!=null) {
					arrivalRate=eventDecoratorData.getArrivalRate();
				}
			}
		}
		return arrivalRate;
	}

	private double formatArrivalRate(String arrivalRate) {
		double arrivalRateDouble;
		// StartEvent TriggerTime setting arrival rate will be converted to hours			
		if (arrivalRate.contains(" process instance(s) per day")) {
			// days...
			String arrivalRateStr = arrivalRate.substring(0, arrivalRate.indexOf(" process instance(s) per day"));
			arrivalRateDouble = Double.parseDouble(arrivalRateStr) / 24;
		} else if (arrivalRate.contains(" process instance(s) per hour")) {
			// hours
			String arrivalRateStr = arrivalRate.substring(0, arrivalRate.indexOf(" process instance(s) per hour"));
			arrivalRateDouble = Double.parseDouble(arrivalRateStr);

		} else if (arrivalRate.contains(" process instance(s) per minute")) {
			// minutes
			String arrivalRateStr = arrivalRate
					.substring(0, arrivalRate.indexOf(" process instance(s) per minute"));
			arrivalRateDouble = Double.parseDouble(arrivalRateStr) * 60;

		} else if (arrivalRate.contains(" process instance(s) per second")) {
			// seconds
			String arrivalRateStr = arrivalRate
					.substring(0, arrivalRate.indexOf(" process instance(s) per second"));
			arrivalRateDouble = Double.parseDouble(arrivalRateStr) * 60 * 60;

		} else {
			String arrivalRateStr = arrivalRate.substring(0,
					arrivalRate.indexOf(" process instance(s) per milisecond"));
			arrivalRateDouble = Double.parseDouble(arrivalRateStr) * 60 * 60 * 1000;

		}
		return arrivalRateDouble;
	}
}
