/**
 * 
 */
package org.processmining.plugins.bpmn.converter;

import java.awt.Dimension;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.NavigableSet;
import java.util.Set;
import java.util.TreeSet;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.bpmn.BPMNDiagram;
import org.processmining.models.graphbased.directed.bpmn.BPMNDiagramFactory;
import org.processmining.models.graphbased.directed.bpmn.BPMNNode;
import org.processmining.models.graphbased.directed.bpmn.elements.Activity;
import org.processmining.models.graphbased.directed.bpmn.elements.Event;
import org.processmining.models.graphbased.directed.bpmn.elements.Event.EventType;
import org.processmining.models.graphbased.directed.bpmn.elements.Event.EventUse;
import org.processmining.models.graphbased.directed.bpmn.elements.Flow;
import org.processmining.models.graphbased.directed.bpmn.elements.Gateway;
import org.processmining.models.graphbased.directed.bpmn.elements.Gateway.GatewayType;
import org.processmining.models.heuristics.HeuristicsNet;
import org.processmining.models.heuristics.impl.HNSubSet;
import org.processmining.plugins.bpmn.converter.ui.CNET2BPMNConverterGUI;
import org.processmining.plugins.heuristicsnet.AnnotatedHeuristicsNet;
import org.processmining.plugins.heuristicsnet.miner.heuristics.miner.operators.Operator;
import org.processmining.plugins.heuristicsnet.miner.heuristics.miner.operators.Stats;

/**
 * @author ibayraktar THIS PLUG_IN IS NOT UPDATED USED PLEASE USE C-NET CONVERTER
 * 
 */
//@Plugin(name = "Heuristics Net to BPMN Conversion", parameterLabels = { "HeuristicsNet" }, returnLabels = { "BPMN Diagram" }, returnTypes = { BPMNDiagram.class }, userAccessible = true, help = "Converts heuristics net (or causal matrices) to BPMN")
public class HeuristicsNet2BPMNConverter {

	private static int aHeight = 80;
	private static int aWidth = 140;

	//@UITopiaVariant(affiliation = "Cordys", author = "I. Bayraktar", email = "i.bayraktar@cordys.com")
	//@PluginVariant(variantLabel = "BPMN 2.0 Conversion", requiredParameterLabels = { 0 })
	public static BPMNDiagram converter(UIPluginContext context, HeuristicsNet hn) {

		//create GUI
		BPMNVendorSettings bpmnSettings = new BPMNVendorSettings();
		CNET2BPMNConverterGUI hn2xpdlConverterGUI = new CNET2BPMNConverterGUI(bpmnSettings);

		InteractionResult result = context.showConfiguration("BPMN Vendor Settings", hn2xpdlConverterGUI);
		if (result.equals(InteractionResult.CANCEL)) {
			context.getFutureResult(0).cancel(true);
		} else {
			hn2xpdlConverterGUI.updateBPMNsettingsFromUI();
		}

		context.getProgress().setCaption("Converting To BPMN");
		context.getProgress().setIndeterminate(false);

		AnnotatedHeuristicsNet ahn = (AnnotatedHeuristicsNet) hn;
		HashMap<String, String> keys = ahn.getInvertedKeys();
		
		/*
		 * Initialize the BPMN diagram.
		 */
		BPMNDiagram bpmn = BPMNDiagramFactory.newBPMNDiagram("BPMN Diagram");
		Map<String, BPMNNode> id2node = new HashMap<String, BPMNNode>();

		/*
		 * Convert HN to BPMN.
		 */
		convertAllActivities(bpmn, id2node, keys);
		createStartEventAndConnectToStartActivity(ahn, bpmn, id2node);
		createEndEventAndConnectToEndActivity(ahn, bpmn, id2node);
		connectSequencesAndSplitGateways(ahn, bpmn, bpmnSettings, id2node, keys);
		connectJoinGateways(ahn, bpmn, bpmnSettings, id2node, keys);

		/*
		 * Set the label of the BPMN diagram.
		 */
		context.getProgress().setCaption("Getting BPMN Visualization");
		context.getProgress().setIndeterminate(false);
		context.getFutureResult(0).setLabel("Mined BPMN Model");
		return bpmn;

	}

	/**
	 * @param ahn
	 * @param bpmn
	 * @param id2node
	 */
	private static void createEndEventAndConnectToEndActivity(AnnotatedHeuristicsNet ahn, BPMNDiagram bpmn,
			Map<String, BPMNNode> id2node) {
		HNSubSet endActivities = ahn.getEndActivities();
		Event e2 = bpmn.addEvent("END EVENT", EventType.END, null, EventUse.THROW, null);
		id2node.put("EID", e2);

		if (endActivities.size() > 1) {
			//GIVE USER INFO TO SELECT THE END ACTIVITY GATEWAY TYPE
		} else if (endActivities.size() == 1) {
			BPMNNode fromNode = id2node.get("ID" + endActivities.get(0));
			BPMNNode toNode = id2node.get("EID");
			bpmn.addFlow(fromNode, toNode, null);
		}
	}

	/**
	 * @param ahn
	 * @param bpmn
	 * @param id2node
	 */
	private static void createStartEventAndConnectToStartActivity(AnnotatedHeuristicsNet ahn, BPMNDiagram bpmn,
			Map<String, BPMNNode> id2node) {
		HNSubSet startActivities = ahn.getStartActivities();

		Event e1 = bpmn.addEvent("START EVENT", EventType.START, null, EventUse.CATCH, null);
		id2node.put("SID", e1);

		if (startActivities.size() > 1) {
			//GIVE USER INFO TO SELECT THE START ACTIVITY GATEWAY TYPE LATER ON
		} else if (startActivities.size() == 1) {
			BPMNNode fromNode = id2node.get("SID");
			BPMNNode toNode = id2node.get("ID" + startActivities.get(0));
			bpmn.addFlow(fromNode, toNode, null);
		}
	}

	/**
	 * @param bpmn
	 * @param id2node
	 * @param keys
	 */
	private static void convertAllActivities(BPMNDiagram bpmn, Map<String, BPMNNode> id2node,
			HashMap<String, String> keys) {
		for (java.util.Map.Entry<String, String> entry : keys.entrySet()) {
			int index = Integer.valueOf(entry.getKey());
			String name = entry.getValue().substring(0, entry.getValue().indexOf("+"));
			Activity a = bpmn.addActivity(name, false, false, false, false, false);
			a.getAttributeMap().put(AttributeMap.SIZE, new Dimension(aWidth, aHeight));
			id2node.put("ID" + index, a);
		}
	}

	/**
	 * @param ahn
	 * @param bpmn
	 * @param id2node
	 * @param keys
	 */
	private static void connectSequencesAndSplitGateways(AnnotatedHeuristicsNet ahn, BPMNDiagram bpmn, BPMNVendorSettings settings, 
			Map<String, BPMNNode> id2node, HashMap<String, String> keys) {
		for (java.util.Map.Entry<String, String> entry : keys.entrySet()) {

			int index = Integer.valueOf(entry.getKey());
			Operator operator = ahn.getSplit(entry.getKey());
			/*
			 * No Pattern exists if patternType.key is 0, but and-join/split if
			 * it is 1, 2 for Xor, 3 for or
			 */
			HashMap<Integer, List<Integer>> patternType = identifyPatternType(keys, operator);
			int patternTypeCode = ((Integer) patternType.keySet().toArray()[0]).intValue();

			/*
			 * patternTypeCode is 0 which means there is no pattern so we need
			 * just a transition between TWO events//SEQUENCE
			 * 
			 * patternType.key is 1,2,3 which means there is AND,XOR,OR-split
			 * pattern respectively so wee need one activity for the symbol and
			 * transitions connected to this activity
			 */
			//THE SYNCHRONIZATION GATEWAY CAN BE VENDOR SPECIFIC SO LET SEE THEVENDOR SETTINGS AND IF IT IS EXPLICIT THEN CREATE THE GATEWAY, CONNECT THE EVENTS WITH THE TRANSITIONS
			//IF THE SYNCHRONIZATION OF EVENTS IS IMPLICITLY DONE BY VENDOR SO THERE IS NO NEED FOR IT

			if (patternTypeCode == 0) {

				int indexOut = patternType.get(0).get(0).intValue();

				BPMNNode fromNode = id2node.get("ID" + index);
				BPMNNode toNode = id2node.get("ID" + indexOut);
				bpmn.addFlow(fromNode, toNode, null);

			} else if (patternTypeCode == 1) {//AND SPLIT
				if (settings.getANDSplit()
						.equals(BPMNVendorSettings.GatewayVisibilitySettings.Explicit)) {
					String activityNameStr = "PARALLEL-GATEWAY";
					fillSplitGatewaySettingsExplicit(index, patternType, patternTypeCode, activityNameStr, id2node,
							bpmn);
				} else {
					fillSplitGatewaySettingsImplicit(bpmn, id2node, index, patternType, patternTypeCode);
				}
			} else if (patternTypeCode == 2) {//XOR SPLIT 
				if (settings.getXORSplit()
						.equals(BPMNVendorSettings.GatewayVisibilitySettings.Explicit)) {
					String activityNameStr = "XOR-GATEWAY";
					fillSplitGatewaySettingsExplicit(index, patternType, patternTypeCode, activityNameStr, id2node,
							bpmn);
				} else {
					fillSplitGatewaySettingsImplicit(bpmn, id2node, index, patternType, patternTypeCode);
				}
			} else if (patternTypeCode == 3) {//OR SPLIT
				if (settings.getORSplit()
						.equals(BPMNVendorSettings.GatewayVisibilitySettings.Explicit)) {
					String activityNameStr = "OR-GATEWAY";
					fillSplitGatewaySettingsExplicit(index, patternType, patternTypeCode, activityNameStr, id2node,
							bpmn);
				} else {
					fillSplitGatewaySettingsImplicit(bpmn, id2node, index, patternType, patternTypeCode);
				}
			}
		}
	}

	/**
	 * @param ahn
	 * @param bpmn
	 * @param id2node
	 * @param keys
	 */
	private static void connectJoinGateways(AnnotatedHeuristicsNet ahn, BPMNDiagram bpmn, BPMNVendorSettings settings, 
			Map<String, BPMNNode> id2node, HashMap<String, String> keys) {
		//JOINS
		//FILL WORKFLOW PROCESS TRANSITION VARIABLES BUT REMOVE TRANSITIONS IN  WHICH JOIN EXISTS AND RECONNECT TO THE JOIN GATEWAY
		/*
		 * But we connected the graphs according to just splits however we need
		 * to add joins but while adding joins we need to remove the transitions
		 * we made then we will create the gateway and then connect the events
		 * to them lets do it
		 */

		for (java.util.Map.Entry<String, String> entry : keys.entrySet()) {
			int index = Integer.valueOf(entry.getKey());
			Operator operator = ahn.getJoin(entry.getKey());
			HashMap<Integer, List<Integer>> patternType = identifyPatternType(keys, operator);
			int patternTypeCode = ((Integer) patternType.keySet().toArray()[0]).intValue();

			if (patternTypeCode == 1) {
				if (settings.getANDJoin()
						.equals(BPMNVendorSettings.GatewayVisibilitySettings.Explicit)) {
					String activityNameStr = "PARALLEL-GATEWAY";
					fillJoinGatewaySettings(ahn, index, patternType, patternTypeCode, activityNameStr, id2node, bpmn, settings);

				}
			} else if (patternTypeCode == 2) {
				if (settings.getXORJoin()
						.equals(BPMNVendorSettings.GatewayVisibilitySettings.Explicit)) {
					String activityNameStr = "XOR-GATEWAY";
					fillJoinGatewaySettings(ahn, index, patternType, patternTypeCode, activityNameStr, id2node, bpmn, settings);
				}

			} else if (patternTypeCode == 3) {
				if (settings.getORJoin()
						.equals(BPMNVendorSettings.GatewayVisibilitySettings.Explicit)) {

					String activityNameStr = "OR-GATEWAY";
					fillJoinGatewaySettings(ahn, index, patternType, patternTypeCode, activityNameStr, id2node, bpmn, settings);
				}

			}

		}
	}

	/**
	 * @param bpmn
	 * @param id2node
	 * @param index
	 * @param patternType
	 * @param patternTypeCode
	 */
	private static void fillSplitGatewaySettingsImplicit(BPMNDiagram bpmn, Map<String, BPMNNode> id2node, int index,
			HashMap<Integer, List<Integer>> patternType, int patternTypeCode) {
		for (Integer patternEvent : patternType.get(patternTypeCode)) {
			BPMNNode fromNode = id2node.get("ID" + index);
			BPMNNode toNode = id2node.get("ID" + patternEvent);
			bpmn.addFlow(fromNode, toNode, null);
		}
	}

	/**
	 * @param ahn
	 * @param index
	 * @param patternType
	 * @param patternTypeCode
	 * @param activityNameStr
	 * @param gatewayTypeStr
	 * @param splitJoinStr
	 */
	private static void fillJoinGatewaySettings(AnnotatedHeuristicsNet ahn, int index,
			HashMap<Integer, List<Integer>> patternType, int patternTypeCode, String activityNameStr,
			Map<String, BPMNNode> id2node, BPMNDiagram bpmn, BPMNVendorSettings settings) {
		String id = "" + index;
		for (Integer patternEvent : patternType.get(patternTypeCode)) {
			id += patternEvent;
		}
		//CREATE GATEWAY ACTIVITY

		String name = activityNameStr;
		GatewayType type = getGatewatType(activityNameStr);
		Gateway g = bpmn.addGateway(name, type);
		g.getAttributeMap().put(AttributeMap.SIZE, new Dimension(((aHeight + aWidth) / 2), ((aHeight + aWidth) / 2)));
		id2node.put("ID" + id, g);

		//REMOVE TRANSITIONS COMING FROM THE SPLIT WHICH CONNECTS DIRECTLY EVENTS TOGETHER
		//AND PUT A GATEWAY BETWEEN THESE ACTIVITIES AND CONNECT THEM
		for (Integer patternEvent : patternType.get(patternTypeCode)) {
			Operator tempOperator = ahn.getSplit("" + patternEvent);
			HashMap<Integer, List<Integer>> tempPatternTypeMap = identifyPatternType(ahn.getInvertedKeys(),
					tempOperator);
			int tempPatternTypeCode = ((Integer) tempPatternTypeMap.keySet().toArray()[0]).intValue();

			//IF THERE IS XOR/AND/OR SPLIT GATEWAY BETWEEN patternEvent AND index EVENT IDS, WE NEED TO REMOVE THE TRANSITION BETWEEN GATEWAY AND index 
			if (tempPatternTypeCode >= 1) {
				String tempID = "" + patternEvent;
				for (Integer tempPatternElement : tempPatternTypeMap.get(tempPatternTypeCode)) {
					tempID += tempPatternElement;
				}
				//CHECK IF THE VENDOR SPECIFIC NOTATION HAS EXPLICIT GATEWAY FOR THE SPECIFIC SPLIT
				if (tempPatternTypeCode == 1
						&& settings.getANDSplit()
								.equals(BPMNVendorSettings.GatewayVisibilitySettings.Explicit)) {
					removeOldAndConnectNewTransitions2Gateway(index, id, tempID, id2node, bpmn);

				} else if (tempPatternTypeCode == 2
						&& settings.getXORSplit()
								.equals(BPMNVendorSettings.GatewayVisibilitySettings.Explicit)) {
					removeOldAndConnectNewTransitions2Gateway(index, id, tempID, id2node, bpmn);
				} else if (tempPatternTypeCode == 3
						&& settings.getORSplit()
								.equals(BPMNVendorSettings.GatewayVisibilitySettings.Explicit)) {
					removeOldAndConnectNewTransitions2Gateway(index, id, tempID, id2node, bpmn);

				} else {
					//THE VENDOR SPECIFIC NOTATION HAS NO EXPLICIT GATEWAY FOR THE SPECIFIC SPLIT
					removeOldAndConnectNewTransitions2Gateway(index, id, "" + patternEvent, id2node, bpmn);
				}
			} else {
				//IF THERE IS NO XOR/AND/OR SPLIT GATEWAY BETWEEN patternEvent AND index EVENT IDS, JUST REMOVE THE TRANSITION BETWEEN THEM
				removeOldAndConnectNewTransitions2Gateway(index, id, "" + patternEvent, id2node, bpmn);
			}
		}

		// CREATE NEW TRANSITION WHICH CONNECTS THE GATEWAY TO THE INDEX ELEMENT
		BPMNNode fromNode = id2node.get("ID" + id);
		BPMNNode toNode = id2node.get("ID" + index);
		bpmn.addFlow(fromNode, toNode, null);

	}

	/**
	 * @param activityNameStr
	 * @return
	 */
	private static GatewayType getGatewatType(String activityNameStr) {
		GatewayType type = GatewayType.DATABASED;

		if (activityNameStr.equalsIgnoreCase("XOR-GATEWAY")) {
			type = GatewayType.DATABASED;
		} else if (activityNameStr.equalsIgnoreCase("PARALLEL-GATEWAY")) {
			type = GatewayType.PARALLEL;
		} else if (activityNameStr.equalsIgnoreCase("OR-GATEWAY")) {
			type = GatewayType.INCLUSIVE;
		}
		return type;
	}

	/**
	 * @param index
	 * @param id
	 * @param tempID
	 */
	private static void removeOldAndConnectNewTransitions2Gateway(int index, String id, String tempID,
			Map<String, BPMNNode> id2node, BPMNDiagram bpmn) {
		BPMNNode fromNode = id2node.get("ID" + tempID);
		BPMNNode toNode = id2node.get("ID" + index);
		Flow f = new Flow(fromNode, toNode, null);
		bpmn.removeEdge(f);
		BPMNNode fromNode2 = id2node.get("ID" + tempID);
		BPMNNode toNode2 = id2node.get("ID" + id);
		bpmn.addFlow(fromNode2, toNode2, null);
	}

	private static void fillSplitGatewaySettingsExplicit(int index, HashMap<Integer, List<Integer>> patternType,
			int patternTypeCode, String activityNameStr, Map<String, BPMNNode> id2node, BPMNDiagram bpmn) {
		String id = "" + index;
		for (Integer patternEvent : patternType.get(patternTypeCode)) {
			id += patternEvent;
		}
		//CREATE GATEWAY ACTIVITY
		GatewayType type = getGatewatType(activityNameStr);
		Gateway g = bpmn.addGateway(activityNameStr, type);
		id2node.put("ID" + id, g);
		g.getAttributeMap().put(AttributeMap.SIZE, new Dimension(((aHeight + aWidth) / 2), ((aHeight + aWidth) / 2)));
		// CREATE NEW TRANSITIONS TO CONNECT ACTIVITIES TO GATEWAY
		BPMNNode fromNode = id2node.get("ID" + index);
		BPMNNode toNode = id2node.get("ID" + id);
		bpmn.addFlow(fromNode, toNode, null);

		for (Integer patternEvent : patternType.get(patternTypeCode)) {
			BPMNNode tempFromNode = id2node.get("ID" + id);
			BPMNNode tempToNode = id2node.get("ID" + patternEvent);
			bpmn.addFlow(tempFromNode, tempToNode, null);
		}
	}

	/**
	 * @param keys
	 *            returns 1 for AND, 2 for XOR, 3 for OR
	 * @return int
	 */
	public static HashMap<Integer, List<Integer>> identifyPatternType(HashMap<String, String> keys, Operator operator) {
		//returns 1 for AND, 2 for XOR, 3 for OR pattern with the element indices list
		List<Integer> eventIndexList = new ArrayList<Integer>();
		HashMap<Integer, List<Integer>> patternElementsMap = new HashMap<Integer, List<Integer>>();

		int index = 0;
		if (operator.getElements().size() > 1) {
			List<String> eventNameList = new ArrayList<String>();

			for (int k = 0; k < operator.getElements().size(); k++) {
				eventIndexList.add(operator.getElements().get(k));

				String eventNameType = keys.get(String.valueOf(operator.getElements().get(k)));
				String eventName = eventNameType.substring(0, eventNameType.indexOf("+"));
				eventNameList.add(eventName);
				System.out.print(eventName + " ");
			}
			System.out.println("");

			//bitmapStack contains a string like 111,100,010 which is called CODE for the event A,B,C in eventNameList 
			//having only 111 in bitmapStack means and-join pattern
			//having 100,010,001 in bitmapStack means xor-join pattern
			//having 111,110,011,100,001,... in bitmapStack means or-join pattern
			List<String> bitmapStack = new ArrayList<String>();
			for (Entry<String, Stats> entry : operator.getLearnedPatterns().entrySet()) {
				String code = entry.getKey();

				//int occurences = entry.getValue().getOccurrences();
				//int totalOccurences = operator.getLearnedPatterns().size();
				//float percentage = 100f * occurences / totalOccurences;
				//if (percentage > 5)
				bitmapStack.add(code);
			}

			for (String code : bitmapStack) {
				for (char value : code.toCharArray())
					System.out.print(value + " ");
				System.out.println("");
			}

			List<Integer> codeTotalList = new ArrayList<Integer>();
			if (!bitmapStack.isEmpty()) {
				for (String code : bitmapStack) {

					int value = 0;
					for (char bitmapValue : code.toCharArray()) {
						value += Character.getNumericValue(bitmapValue);
					}
					codeTotalList.add(value);
				}
				boolean isXOR = true;
				for (Integer codeTotal : codeTotalList) {
					if (codeTotal == operator.getElements().size() && codeTotalList.size() == 1) {
						System.out.println("\n AND-OPERATOR");
						index = 1;
						isXOR = false;
						break;
					} else if (codeTotal == 1) {
						continue;
					} else if (codeTotal > 1 && codeTotal <= operator.getElements().size() && codeTotalList.size() > 1) {
						System.out.println("\n OR-OPERATOR");
						isXOR = false;
						index = 3;
						break;
					}
				}
				if (isXOR) {
					index = 2;
					System.out.println("\n XOR-OPERATOR");
				}
			}

		} else if (operator.getElements().size() == 1) {
			eventIndexList.add(operator.getElements().get(0));
		} else if (operator.getElements().size() == 0)
			index = -1;

		patternElementsMap.put(index, eventIndexList);
		return patternElementsMap;

		/*
		 * Set<Integer> eventIndicesSet = new TreeSet<Integer>();
		 * 
		 * for (String event : eventNameList) {
		 * eventIndicesSet.add(eventNameList.indexOf(event)); }
		 * 
		 * List<TreeSet<Integer>> eventIndicesSubSets =
		 * powerSet(eventIndicesSet);
		 * 
		 * for (TreeSet<Integer> eventIndicesSubSet : eventIndicesSubSets) { for
		 * (Integer eventIndex : eventIndicesSubSet.) {
		 * 
		 * } }
		 * 
		 * for (;;) {
		 * 
		 * break; }
		 */

	}

	public void test(TreeSet<Integer> eventIndicesTreeSet, Integer eventIndex) {

		if (eventIndex == eventIndicesTreeSet.size())
			return;
		else {
			NavigableSet<Integer> eventIndicesNavigableSet = eventIndicesTreeSet.tailSet(eventIndex, false);

			test(eventIndicesTreeSet, eventIndex + 1);
			test2(eventIndicesNavigableSet, eventIndex + 1);
		}

	}

	public void test2(NavigableSet<Integer> eventIndicesTreeSet, Integer eventIndex) {

	}

	public static <T> List<TreeSet<T>> powerSet(Set<T> set) {

		List<TreeSet<T>> listOfSubsets = new ArrayList<TreeSet<T>>();
		if (set.isEmpty()) {
			listOfSubsets.add(new TreeSet<T>());
			return listOfSubsets;
		}

		List<T> list = new ArrayList<T>(set);
		T head = list.get(0);
		TreeSet<T> rest = new TreeSet<T>(list.subList(1, list.size()));

		for (TreeSet<T> restSubsets : powerSet(rest)) {
			TreeSet<T> newSet = new TreeSet<T>();
			newSet.add(head);
			newSet.addAll(restSubsets);
			listOfSubsets.add(newSet);
			listOfSubsets.add(restSubsets);
		}
		return listOfSubsets;
	}

}
