/**
 * 
 */
package org.processmining.plugins.bpmn.converter.ui;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import org.processmining.plugins.bpmn.converter.BPMNVendorSettings;

import com.fluxicon.slickerbox.factory.SlickerDecorator;
import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * @author ibayraktar
 * 
 */
public class CNET2BPMNConverterGUI extends JPanel implements ActionListener {

	private static final long serialVersionUID = -2608532190536549047L;

	private SlickerFactory factory = SlickerFactory.instance();
	private SlickerDecorator decorator = SlickerDecorator.instance();

	public static String VENDOR_STANDARD = "Standard BPMN";
	public static String VENDOR_CORDYS = "Cordys";
	public static String VENDOR_CUSTOM = "Custom Settings";

	public static String[] vendorList = { VENDOR_STANDARD, VENDOR_CORDYS, VENDOR_CUSTOM };

	private String vendorName;
	private JPanel companySelectionPanel, synchronizationPanel, branchingPanel, otherSelectionPanel,
			markingVisibilityPanel;

	private JLabel companySelectionPanelTitle, synchronizationTitle, branchingTitle, otherSelectionPanelTitle;
	private JLabel companySelectionLabel;
	private JLabel markingVisibilityTitle;
	private JLabel synchronizationANDLabel, synchronizationXORLabel, synchronizationORLabel;
	private JLabel branchingANDLabel, branchingXORLabel, branchingORLabel;

	private JCheckBox markingVisibilityANDCheckBox, markingVisibilityXORCheckBox, markingVisibilityORCheckBox;

	private JComboBox companySelectionCombo;
	private JComboBox synchronizationANDCombo, synchronizationXORCombo, synchronizationORCombo;
	private JComboBox branchingANDCombo, branchingXORCombo, branchingORCombo;
	
	private BPMNVendorSettings bpmnSettings;

	public CNET2BPMNConverterGUI(BPMNVendorSettings bpmnSettings) {
		//Set default vendor settings
		this.bpmnSettings = bpmnSettings;
		this.init();
	}

	private void init() {

		this.companySelectionPanel = factory.createRoundedPanel(15, Color.gray);
		this.companySelectionPanel.setBounds(0, 0, 800, 100);

		this.companySelectionPanelTitle = factory.createLabel("Vendor Specific Templates");
		this.companySelectionPanelTitle.setHorizontalAlignment(SwingConstants.CENTER);
		this.companySelectionPanelTitle.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 18));
		this.companySelectionPanelTitle.setForeground(new Color(40, 40, 40));
		this.companySelectionPanelTitle.setBounds(10, 10, 750, 30);

		this.companySelectionLabel = factory.createLabel("Vendor Selection :");
		this.companySelectionLabel.setForeground(new Color(40, 40, 40));
		this.companySelectionLabel.setBounds(20, 60, 200, 30);
		companySelectionPanel.add(companySelectionLabel);
		this.companySelectionLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 16));

		this.companySelectionCombo = factory.createComboBox(vendorList);
		this.companySelectionCombo.setSelectedItem(VENDOR_STANDARD);

		//add action listener for vendor selection combo
		this.companySelectionCombo.addActionListener(this);
		this.companySelectionCombo.setBounds(225, 60, 200, 30);
		this.companySelectionCombo.setBackground(Color.GRAY);
		decorator.decorate(this.companySelectionCombo);
		companySelectionPanel.add(companySelectionCombo);

		this.companySelectionPanel.setLayout(null);
		this.companySelectionPanel.add(this.companySelectionPanelTitle);

		this.otherSelectionPanel = factory.createRoundedPanel(15, Color.gray);
		this.otherSelectionPanel.setBounds(0, 120, 770, 50);

		this.otherSelectionPanelTitle = factory.createLabel("Your Template");
		this.otherSelectionPanelTitle.setHorizontalAlignment(SwingConstants.CENTER);
		this.otherSelectionPanelTitle.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 18));
		this.otherSelectionPanelTitle.setForeground(new Color(40, 40, 40));
		this.otherSelectionPanelTitle.setBounds(10, 10, 750, 30);

		this.otherSelectionPanel.setLayout(null);
		this.otherSelectionPanel.add(this.otherSelectionPanelTitle);

		this.synchronizationPanel = factory.createRoundedPanel(15, Color.gray);
		this.synchronizationPanel.setBounds(0, 180, 260, 200);

		this.synchronizationTitle = factory.createLabel("Synchronization Settings");
		this.synchronizationTitle.setHorizontalAlignment(SwingConstants.CENTER);
		this.synchronizationTitle.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 18));
		this.synchronizationTitle.setForeground(new Color(40, 40, 40));
		this.synchronizationTitle.setBounds(10, 10, 240, 30);

		initSychronizationSettingsPanel();
		initBrachingSettingsPanel();
		initMarkingVisibilityPanel();

		this.setLayout(null);
		this.add(this.companySelectionPanel);
		this.add(this.otherSelectionPanel);
		this.add(this.synchronizationPanel);
		this.add(this.branchingPanel);
		this.add(this.markingVisibilityPanel);
		this.validate();
		this.repaint();
	}

	private void initBrachingSettingsPanel() {
		this.branchingPanel = factory.createRoundedPanel(15, Color.gray);
		this.branchingPanel.setBounds(270, 180, 260, 200);

		this.branchingTitle = factory.createLabel("Branching Settings");
		this.branchingTitle.setHorizontalAlignment(SwingConstants.CENTER);
		this.branchingTitle.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 18));
		this.branchingTitle.setForeground(new Color(40, 40, 40));
		this.branchingTitle.setBounds(10, 10, 240, 30);

		this.branchingPanel.setLayout(null);
		this.branchingPanel.add(this.branchingTitle);

		this.branchingANDLabel = factory.createLabel("Parallel Split :");
		this.branchingANDLabel.setForeground(new Color(40, 40, 40));
		this.branchingANDLabel.setBounds(10, 50, 150, 30);
		this.branchingANDLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 16));
		branchingPanel.add(branchingANDLabel);

		this.branchingANDCombo = factory.createComboBox(BPMNVendorSettings.settingsArray);
		this.branchingANDCombo.setSelectedItem(bpmnSettings.getANDSplit());
		this.branchingANDCombo.setBounds(160, 50, 90, 30);
		this.branchingANDCombo.setBackground(Color.GRAY);
		decorator.decorate(this.branchingANDCombo);
		branchingPanel.add(branchingANDCombo);

		this.branchingXORLabel = factory.createLabel("Exclusive Choice :");
		this.branchingXORLabel.setForeground(new Color(40, 40, 40));
		this.branchingXORLabel.setBounds(10, 90, 150, 30);
		this.branchingXORLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 16));
		branchingPanel.add(branchingXORLabel);

		this.branchingXORCombo = factory.createComboBox(BPMNVendorSettings.settingsArray);
		this.branchingXORCombo.setSelectedItem(bpmnSettings.getXORSplit());
		this.branchingXORCombo.setBounds(160, 90, 90, 30);
		this.branchingXORCombo.setBackground(Color.GRAY);
		decorator.decorate(this.branchingXORCombo);
		branchingPanel.add(branchingXORCombo);

		this.branchingORLabel = factory.createLabel("Multi Choice :");
		this.branchingORLabel.setForeground(new Color(40, 40, 40));
		this.branchingORLabel.setBounds(10, 130, 150, 30);
		this.branchingORLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 16));
		branchingPanel.add(branchingORLabel);

		this.branchingORCombo = factory.createComboBox(BPMNVendorSettings.settingsArray);
		this.branchingORCombo.setSelectedItem(bpmnSettings.getORSplit());
		this.branchingORCombo.setBounds(160, 130, 90, 30);
		this.branchingORCombo.setBackground(Color.GRAY);
		decorator.decorate(this.branchingORCombo);
		branchingPanel.add(branchingORCombo);
	}

	private void initMarkingVisibilityPanel() {
		this.markingVisibilityPanel = factory.createRoundedPanel(15, Color.gray);
		this.markingVisibilityPanel.setBounds(540, 180, 260, 200);
		this.markingVisibilityPanel.setLayout(null);

		this.markingVisibilityTitle = factory.createLabel("Marking Visibility");
		this.markingVisibilityTitle.setHorizontalAlignment(SwingConstants.CENTER);
		this.markingVisibilityTitle.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 18));
		this.markingVisibilityTitle.setForeground(new Color(40, 40, 40));
		this.markingVisibilityTitle.setBounds(10, 10, 240, 30);

		this.markingVisibilityPanel.add(this.markingVisibilityTitle);

		this.markingVisibilityANDCheckBox = factory.createCheckBox("Parallel Gateway is Visible", bpmnSettings.getANDMarker());
		this.markingVisibilityANDCheckBox.setBounds(10, 50, 240, 30);
		this.markingVisibilityANDCheckBox.setBackground(Color.GRAY);
		decorator.decorate(this.markingVisibilityANDCheckBox);
		markingVisibilityPanel.add(markingVisibilityANDCheckBox);

		this.markingVisibilityXORCheckBox = factory.createCheckBox("Databased Gateway is Visible", bpmnSettings.getXORMarker());
		this.markingVisibilityXORCheckBox.setBounds(10, 90, 240, 30);
		this.markingVisibilityXORCheckBox.setBackground(Color.GRAY);
		decorator.decorate(this.markingVisibilityXORCheckBox);
		markingVisibilityPanel.add(markingVisibilityXORCheckBox);

		this.markingVisibilityORCheckBox = factory.createCheckBox("Inclusive Gateway is Visible", bpmnSettings.getORMarker());
		this.markingVisibilityORCheckBox.setBounds(10, 130, 240, 30);
		this.markingVisibilityORCheckBox.setBackground(Color.GRAY);
		decorator.decorate(this.markingVisibilityORCheckBox);
		markingVisibilityPanel.add(markingVisibilityORCheckBox);
	}

	private void initSychronizationSettingsPanel() {
		this.synchronizationPanel.setLayout(null);
		this.synchronizationPanel.add(this.synchronizationTitle);

		this.synchronizationANDLabel = factory.createLabel("Parallel Synch :");
		this.synchronizationANDLabel.setForeground(new Color(40, 40, 40));
		this.synchronizationANDLabel.setBounds(10, 50, 150, 30);
		this.synchronizationANDLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 16));
		synchronizationPanel.add(synchronizationANDLabel);

		this.synchronizationANDCombo = factory.createComboBox(BPMNVendorSettings.settingsArray);
		this.synchronizationANDCombo.setSelectedItem(bpmnSettings.getANDJoin());
		this.synchronizationANDCombo.setBounds(160, 50, 90, 30);
		this.synchronizationANDCombo.setBackground(Color.GRAY);
		decorator.decorate(this.synchronizationANDCombo);
		synchronizationPanel.add(synchronizationANDCombo);

		this.synchronizationXORLabel = factory.createLabel("Simple Merge :");
		this.synchronizationXORLabel.setForeground(new Color(40, 40, 40));
		this.synchronizationXORLabel.setBounds(10, 90, 150, 30);
		this.synchronizationXORLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 16));
		synchronizationPanel.add(synchronizationXORLabel);

		this.synchronizationXORCombo = factory.createComboBox(BPMNVendorSettings.settingsArray);
		this.synchronizationXORCombo.setSelectedItem(bpmnSettings.getXORJoin());
		this.synchronizationXORCombo.setBounds(160, 90, 90, 30);
		this.synchronizationXORCombo.setBackground(Color.GRAY);
		decorator.decorate(this.synchronizationXORCombo);
		synchronizationPanel.add(synchronizationXORCombo);

		this.synchronizationORLabel = factory.createLabel("Synchronizing Merge :");
		this.synchronizationORLabel.setForeground(new Color(40, 40, 40));
		this.synchronizationORLabel.setBounds(10, 130, 150, 30);
		this.synchronizationORLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 16));
		synchronizationPanel.add(synchronizationORLabel);

		this.synchronizationORCombo = factory.createComboBox(BPMNVendorSettings.settingsArray);
		this.synchronizationORCombo.setSelectedItem(bpmnSettings.getORJoin());
		this.synchronizationORCombo.setBounds(160, 130, 90, 30);
		this.synchronizationORCombo.setBackground(Color.GRAY);
		decorator.decorate(this.synchronizationORCombo);
		synchronizationPanel.add(synchronizationORCombo);
	}

	public void setUIfromBPMNSettings() {

		this.synchronizationANDCombo.setSelectedItem(this.bpmnSettings.getANDJoin());
		this.synchronizationXORCombo.setSelectedItem(this.bpmnSettings.getXORJoin());
		this.synchronizationORCombo.setSelectedItem(this.bpmnSettings.getORJoin());
		this.branchingANDCombo.setSelectedItem(this.bpmnSettings.getANDSplit());
		this.branchingXORCombo.setSelectedItem(this.bpmnSettings.getXORSplit());
		this.branchingORCombo.setSelectedItem(this.bpmnSettings.getORSplit());
		this.markingVisibilityANDCheckBox.setSelected(this.bpmnSettings.getANDMarker());
		this.markingVisibilityXORCheckBox.setSelected(this.bpmnSettings.getXORMarker());
		this.markingVisibilityORCheckBox.setSelected(this.bpmnSettings.getORMarker());
	}

	public void updateBPMNsettingsFromUI() {
			this.bpmnSettings.setANDJoin(
					(BPMNVendorSettings.GatewayVisibilitySettings) this.synchronizationANDCombo.getSelectedItem());
			this.bpmnSettings.setXORJoin(
					(BPMNVendorSettings.GatewayVisibilitySettings) this.synchronizationXORCombo.getSelectedItem());
			this.bpmnSettings.setORJoin(
					(BPMNVendorSettings.GatewayVisibilitySettings) this.synchronizationORCombo.getSelectedItem());
			this.bpmnSettings.setANDSplit(
					(BPMNVendorSettings.GatewayVisibilitySettings) this.branchingANDCombo.getSelectedItem());
			this.bpmnSettings.setXORSplit(
					(BPMNVendorSettings.GatewayVisibilitySettings) this.branchingXORCombo.getSelectedItem());
			this.bpmnSettings.setORSplit(
					(BPMNVendorSettings.GatewayVisibilitySettings) this.branchingORCombo.getSelectedItem());
			this.bpmnSettings.setANDMarker(this.markingVisibilityANDCheckBox.isSelected());
			this.bpmnSettings.setXORMarker(this.markingVisibilityXORCheckBox.isSelected());
			this.bpmnSettings.setORMarker(this.markingVisibilityORCheckBox.isSelected());

	}

	public void actionPerformed(ActionEvent e) {
		//set selected vendor settings
		if (this.companySelectionCombo.getSelectedItem().equals(VENDOR_CORDYS)) {
			this.setVendorName(VENDOR_CORDYS);
			this.setCordysSettings();
			this.setUIfromBPMNSettings();
			this.validate();
			this.repaint();
		} else if (this.companySelectionCombo.getSelectedItem().equals(VENDOR_STANDARD)) {
			this.setVendorName(VENDOR_STANDARD);
			this.bpmnSettings.setStandardSettings();
			this.setUIfromBPMNSettings();
			this.validate();
			this.repaint();
		} else if (e.getSource().equals(VENDOR_CUSTOM)) {
			this.setVendorName(VENDOR_CUSTOM);
			this.bpmnSettings.setStandardSettings();
			this.setUIfromBPMNSettings();
		}
	}

	public String getVendorName() {
		return vendorName;
	}

	public void setVendorName(String vendorName) {
		this.vendorName = vendorName;
	}

	public void setCordysSettings() {
		this.bpmnSettings.setANDJoin(BPMNVendorSettings.GatewayVisibilitySettings.Implicit);
		this.bpmnSettings.setANDSplit(BPMNVendorSettings.GatewayVisibilitySettings.Implicit);
		this.bpmnSettings.setANDMarker(false);
		this.bpmnSettings.setXORJoin(BPMNVendorSettings.GatewayVisibilitySettings.Implicit);
		this.bpmnSettings.setXORSplit(BPMNVendorSettings.GatewayVisibilitySettings.Explicit);
		this.bpmnSettings.setXORMarker(true);
		this.bpmnSettings.setORJoin(BPMNVendorSettings.GatewayVisibilitySettings.Implicit);
		this.bpmnSettings.setORSplit(BPMNVendorSettings.GatewayVisibilitySettings.Explicit);
		this.bpmnSettings.setORMarker(false);
	}
	/*
	 * public void setOryxSettings() {
	 * this.bpmnSettings.setANDJoin
	 * (BPMNVendorSettings.GatewayVisibilitySettings.Explicit);
	 * this.bpmn.getVendorSpecificSettings
	 * ().setANDSplit(BPMNVendorSettings.GatewayVisibilitySettings.Explicit);
	 * this.bpmnSettings.setANDMarker(true);
	 * this.bpmn.getVendorSpecificSettings
	 * ().setXORJoin(BPMNVendorSettings.GatewayVisibilitySettings.Explicit);
	 * this.bpmnSettings.setXORSplit(BPMNVendorSettings.
	 * GatewayVisibilitySettings.Explicit);
	 * this.bpmnSettings.setXORMarker(true);
	 * this.bpmn.getVendorSpecificSettings
	 * ().setORJoin(BPMNVendorSettings.GatewayVisibilitySettings.Explicit);
	 * this.bpmnSettings.setORSplit(BPMNVendorSettings.
	 * GatewayVisibilitySettings.Explicit);
	 * this.bpmnSettings.setORMarker(true); }
	 * 
	 * public void setTIBCOSettings() {
	 * this.bpmnSettings.setANDJoin
	 * (BPMNVendorSettings.GatewayVisibilitySettings.Explicit);
	 * this.bpmn.getVendorSpecificSettings
	 * ().setANDSplit(BPMNVendorSettings.GatewayVisibilitySettings.Explicit);
	 * this.bpmnSettings.setANDMarker(true);
	 * this.bpmn.getVendorSpecificSettings
	 * ().setXORJoin(BPMNVendorSettings.GatewayVisibilitySettings.Explicit);
	 * this.bpmnSettings.setXORSplit(BPMNVendorSettings.
	 * GatewayVisibilitySettings.Explicit);
	 * this.bpmnSettings.setXORMarker(true);
	 * this.bpmn.getVendorSpecificSettings
	 * ().setORJoin(BPMNVendorSettings.GatewayVisibilitySettings.Explicit);
	 * this.bpmnSettings.setORSplit(BPMNVendorSettings.
	 * GatewayVisibilitySettings.Explicit);
	 * this.bpmnSettings.setORMarker(true); }
	 * 
	 * public void setGlobal360Settings() {
	 * this.bpmnSettings
	 * .setANDJoin(BPMNVendorSettings.GatewayVisibilitySettings.Explicit);
	 * this.bpmnSettings.setANDSplit(BPMNVendorSettings.
	 * GatewayVisibilitySettings.Explicit);
	 * this.bpmnSettings.setANDMarker(true);
	 * this.bpmn.getVendorSpecificSettings
	 * ().setXORJoin(BPMNVendorSettings.GatewayVisibilitySettings.Explicit);
	 * this.bpmnSettings.setXORSplit(BPMNVendorSettings.
	 * GatewayVisibilitySettings.Explicit);
	 * this.bpmnSettings.setXORMarker(true);
	 * this.bpmn.getVendorSpecificSettings
	 * ().setORJoin(BPMNVendorSettings.GatewayVisibilitySettings.Explicit);
	 * this.bpmnSettings.setORSplit(BPMNVendorSettings.
	 * GatewayVisibilitySettings.Explicit);
	 * this.bpmnSettings.setORMarker(true); }
	 */

}
