package org.processmining.plugins.bpmn.converter.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;

import org.processmining.models.flexiblemodel.EndTaskNodesSet;
import org.processmining.models.flexiblemodel.Flex;
import org.processmining.models.flexiblemodel.StartTaskNodesSet;
import org.processmining.plugins.bpmn.converter.CNetFixModel;

import com.fluxicon.slickerbox.factory.SlickerDecorator;
import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * @author ibayraktar
 * 
 */
public class CNetCheckGUI extends JPanel implements ActionListener {
	private static final long serialVersionUID = -2608532190536549047L;
	private SlickerFactory factory = SlickerFactory.instance();
	private SlickerDecorator decorator = SlickerDecorator.instance();

	private StartTaskNodesSet startTaskNodesSet;
	private EndTaskNodesSet endTaskNodesSet;

	private Flex model;

	private JPanel checkDetailsHeaderPanel, checkDetailsPanel, errorDetailsPanel, fixingDetailsMessagePanel,
			fixingDetailsPanel;
	private JButton checkButton, fixButton;
	private JLabel checkDetailsPanelTitle, errorDetailsPanelTitle, fixingDetailsMessagePanelTitle, fixMessageLabel;
	private JLabel[] messageLabelList = null;
	private JScrollPane fixMessagePane, errorMessagePane;
	
	private boolean isModelValid = true;

	public CNetCheckGUI(Flex flex, StartTaskNodesSet startTaskNodesSet, EndTaskNodesSet endTaskNodesSet) {
		this.init();
		this.startTaskNodesSet = startTaskNodesSet;
		this.endTaskNodesSet = endTaskNodesSet;
		this.model = flex;
	}

	private void init() {

		this.checkDetailsHeaderPanel = factory.createRoundedPanel(15, Color.LIGHT_GRAY);
		this.checkDetailsHeaderPanel.setBounds(0, 0, 770, 50);
		this.checkDetailsHeaderPanel.setLayout(null);

		this.checkDetailsPanelTitle = factory.createLabel("Checking the suitability of C-Net");
		this.checkDetailsPanelTitle.setHorizontalAlignment(SwingConstants.CENTER);
		this.checkDetailsPanelTitle.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 16));
		this.checkDetailsPanelTitle.setForeground(new Color(40, 40, 40));
		this.checkDetailsPanelTitle.setBounds(10, 10, 750, 30);
		this.checkDetailsHeaderPanel.add(this.checkDetailsPanelTitle);
		////////////////////
		this.checkDetailsPanel = factory.createRoundedPanel(15, Color.LIGHT_GRAY);
		this.checkDetailsPanel.setBounds(0, 60, 770, 50);
		this.checkDetailsPanel.setLayout(null);

		this.checkButton = factory.createButton("Start Checking");
		this.checkButton.setForeground(Color.DARK_GRAY);
		this.checkButton.setBackground(Color.GREEN);
		this.checkButton.setBounds(20, 10, 150, 30);
		this.checkButton.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 14));
		this.checkButton.addActionListener(this);
		this.checkDetailsPanel.add(this.checkButton);
		//////////		

		this.errorDetailsPanel = new JPanel();
		this.errorDetailsPanel.setBackground(Color.LIGHT_GRAY);
		this.errorDetailsPanel.setBounds(0, 120, 770, 130);
		this.errorDetailsPanel.setLayout(new GridLayout(0, 1));
		
		this.errorMessagePane = new JScrollPane(errorDetailsPanel, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		this.errorMessagePane.setBounds(0, 120, 770, 130);
		this.errorMessagePane.setVisible(false);
		this.errorMessagePane.setBorder(null);
		decorator.decorate(errorMessagePane, Color.LIGHT_GRAY, Color.GRAY, Color.DARK_GRAY);
	
		////////////////////
		this.fixingDetailsPanel = factory.createRoundedPanel(15, Color.LIGHT_GRAY);
		this.fixingDetailsPanel.setBounds(0, 260, 770, 50);
		this.fixingDetailsPanel.setLayout(null);
		this.fixingDetailsPanel.setVisible(false);

		//////////		

		this.fixingDetailsMessagePanel = new JPanel(new BorderLayout());
		this.fixingDetailsMessagePanel.setBackground(Color.LIGHT_GRAY);
		
		this.fixingDetailsMessagePanelTitle = factory.createLabel("Fixing Results:");
		this.fixingDetailsMessagePanelTitle.setHorizontalAlignment(SwingConstants.LEFT);
		this.fixingDetailsMessagePanelTitle.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 16));
		this.fixingDetailsMessagePanelTitle.setForeground(new Color(40, 40, 40));
		this.fixingDetailsMessagePanelTitle.setSize(750, 30);
		this.fixingDetailsMessagePanel.add(this.fixingDetailsMessagePanelTitle, BorderLayout.NORTH);
		
		this.fixMessageLabel = factory.createLabel("");
		this.fixMessageLabel.setToolTipText("");
		this.fixMessageLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));
		this.fixMessageLabel.setForeground(Color.BLACK);
		this.fixingDetailsMessagePanel.add(this.fixMessageLabel, BorderLayout.CENTER);
		
		this.fixMessagePane = new JScrollPane(fixingDetailsMessagePanel, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		this.fixMessagePane.setBounds(0, 320, 770, 100);
		this.fixMessagePane.setVisible(false);
		this.fixMessagePane.setBorder(null);
		decorator.decorate(fixMessagePane, Color.LIGHT_GRAY, Color.GRAY, Color.DARK_GRAY);

		this.setLayout(null);
		this.add(this.checkDetailsHeaderPanel);
		this.add(this.checkDetailsPanel);
		this.add(this.errorMessagePane);
		this.add(this.fixingDetailsPanel);
		this.add(this.fixMessagePane);
		this.validate();
		this.repaint();
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource().equals(checkButton)) {

			List<String> messageList = new LinkedList<String>();
			List<String> codeList = new LinkedList<String>();
			boolean needsFixing = CNetFixModel.modelHasErrors(model, startTaskNodesSet, endTaskNodesSet, messageList, codeList);
			isModelValid = !needsFixing;
			
			if (!needsFixing) {
				this.fixMessagePane.setVisible(true);
				String fixMessage = "There are no identified problems!";
				this.fixMessageLabel = factory.createLabel(fixMessage);
				this.fixMessageLabel.setHorizontalAlignment(SwingConstants.LEFT);
				this.fixMessageLabel.setToolTipText(fixMessage);
				this.fixMessageLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));
				this.fixMessageLabel.setForeground(new Color(40, 40, 40));
				this.fixMessageLabel.setBounds(10, 50, 750, 30);
				SlickerDecorator.instance().decorate(this.fixMessageLabel);
				this.fixingDetailsMessagePanel.add(fixMessageLabel);

				this.fixMessagePane.repaint();
				this.fixingDetailsMessagePanel.repaint();
				this.revalidate();
				this.repaint();
			} else {
				this.errorDetailsPanel.removeAll();
				
				this.errorMessagePane.setVisible(true);

				this.errorDetailsPanelTitle = factory.createLabel("Checking Results:");
				this.errorDetailsPanelTitle.setHorizontalAlignment(SwingConstants.LEFT);
				this.errorDetailsPanelTitle.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 16));
				this.errorDetailsPanelTitle.setForeground(new Color(40, 40, 40));
				this.errorDetailsPanel.add(this.errorDetailsPanelTitle);

				if (messageList.size() > 5) {
					messageList = messageList.subList(0, 5);
				}
				messageLabelList = new JLabel[messageList.size()];
				for (String message : messageList) {
					this.messageLabelList[messageList.indexOf(message)] = factory.createLabel("Error Message "
							+ (messageList.indexOf(message) + 1) + "- " + message);
					this.messageLabelList[messageList.indexOf(message)].setHorizontalAlignment(SwingConstants.LEFT);
					this.messageLabelList[messageList.indexOf(message)].setToolTipText(message);
					this.messageLabelList[messageList.indexOf(message)].setFont(new java.awt.Font("Dialog",
							java.awt.Font.BOLD, 12));
					this.messageLabelList[messageList.indexOf(message)].setForeground(Color.DARK_GRAY);
					SlickerDecorator.instance().decorate(this.messageLabelList[messageList.indexOf(message)]);
					this.errorDetailsPanel.add(messageLabelList[messageList.indexOf(message)]);
				}

				this.errorMessagePane.repaint();
				this.errorDetailsPanel.repaint();
				this.revalidate();
				this.repaint();

				this.fixingDetailsPanel.setVisible(true);

				this.fixButton = factory.createButton("Start Fixing");
				this.fixButton.setForeground(Color.DARK_GRAY);
				this.fixButton.setBackground(Color.GREEN);
				this.fixButton.setBounds(20, 10, 150, 30);
				this.fixButton.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 14));
				this.fixButton.addActionListener(this);
				this.fixingDetailsPanel.add(this.fixButton);

				this.fixingDetailsPanel.revalidate();
				this.fixingDetailsPanel.repaint();
				this.revalidate();
				this.repaint();
			}

		} else if (e.getSource().equals(fixButton)) {
			List<String> analysisResult = new LinkedList<String>();
			isModelValid = CNetFixModel.solveErrors(model, startTaskNodesSet, endTaskNodesSet, analysisResult );
			
			StringBuilder fixMessage = new StringBuilder();
			if (isModelValid) {
				fixMessage.append("<html><p>Model fixed.</p></html>");
			} else {
				fixMessage.append("<html><p>Model could not be fixed.</p>");
				for (String m : analysisResult) {
					fixMessage.append(m);
				}
				fixMessage.append("</html>");
			}
			String message = fixMessage.toString();
			
			this.fixMessageLabel.setText(message);
			this.fixMessageLabel.setToolTipText(message);
			this.fixMessageLabel.repaint();
			
			this.fixMessagePane.setVisible(true);
			this.fixingDetailsMessagePanel.repaint();
			this.fixMessagePane.repaint();
			this.fixMessageLabel.repaint();
			
		}
	}
	
	public boolean isModelValid() {
		return isModelValid;
	}

}
